/*=========================================================================*/
/*                        BlockedListDlg.cpp                               */
/*                                                                         */
/*                  Edit entries in the blocked site list                  */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/

#include "stdafx.h"
#include "cookiecop.h"
#include "BlockedListDlg.h"
#include "EditUrlDlg.h"
#include "DlgPropSheet.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

enum {BLOCKED_SITE,BLOCKED_IMAGE};

/////////////////////////////////////////////////////////////////////////////
// CBlockedListDlg dialog


CBlockedListDlg::CBlockedListDlg() : CPropertyPage(CBlockedListDlg::IDD)
{
	//{{AFX_DATA_INIT(CBlockedListDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	
	m_bDirty = FALSE;
}


void CBlockedListDlg::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBlockedListDlg)
	DDX_Control(pDX, IDC_BLOCKED_LIST, m_BlockedListCtrl);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CBlockedListDlg, CPropertyPage)
	//{{AFX_MSG_MAP(CBlockedListDlg)
	ON_BN_CLICKED(IDC_NEW, OnAdd)
	ON_BN_CLICKED(IDC_EDIT, OnEdit)
	ON_BN_CLICKED(IDC_DELETE, OnDelete)
	ON_NOTIFY(NM_DBLCLK, IDC_BLOCKED_LIST, OnDblclkBlockedList)
	ON_MESSAGE( MSG_DRAGDROP, OnDrop)
	ON_BN_CLICKED(IDC_BLOCK_OPTIONS, OnBlockOptions)
	ON_BN_CLICKED(IDC_DISABLEBLOCKLIST, OnDisableblocklist)
	ON_EN_CHANGE(IDC_BLOCKED_IMAGE_FILE, OnChangeBlockedImageFile)
	ON_EN_CHANGE(IDC_BLOCKED_SITE_FILE, OnChangeBlockedSiteFile)
	ON_BN_CLICKED(IDC_BROWSE_IMAGE, OnBrowseImage)
	ON_BN_CLICKED(IDC_BROWSE_SITE, OnBrowseSite)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CBlockedListDlg message handlers


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CBlockedListDlg::PreTranslateMessage(MSG* pMsg) 
{
	m_ToolTip.RelayEvent( pMsg );	
	return CPropertyPage::PreTranslateMessage(pMsg);
}



/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::PopulateBlockedList()
{
	m_BlockedListCtrl.DeleteAllItems( );

	//populate
	int i,iCount;
	LV_ITEM lv;
	CString s;
	memset( &lv, 0, sizeof( lv ) );

	iCount = m_theApp->m_BlockedSiteList.GetSize( );
	for( i = 0; i < iCount; i++ )
	{
		int index = m_BlockedListCtrl.GetItemCount( );
		lv.mask =  LVIF_IMAGE | LVIF_TEXT;
		lv.iItem = index;
		s = m_theApp->m_BlockedSiteList.GetAt( i );
		lv.pszText = (LPSTR)(LPCSTR)s;
		lv.iImage = BLOCKED_SITE;
		index = m_BlockedListCtrl.InsertItem( &lv );
	}

	iCount = m_theApp->m_BlockedImageList.GetSize( );
	for( i = 0; i < iCount; i++ )
	{
		int index = m_BlockedListCtrl.GetItemCount( );
		lv.mask =  LVIF_IMAGE | LVIF_TEXT;
		lv.iItem = index;
		s = m_theApp->m_BlockedImageList.GetAt( i );
		lv.pszText = (LPSTR)(LPCSTR)s;
		lv.iImage = BLOCKED_IMAGE;
		index = m_BlockedListCtrl.InsertItem( &lv );
	}
	
	//always force a selection
	if( m_BlockedListCtrl.GetNextItem(-1, LVNI_SELECTED) == -1 )
		HighlightItem( 0 );

	//options
	CheckDlgButton( IDC_DISABLEBLOCKLIST, m_theApp->m_bDisableBlockedList );  
	SetDlgItemText( IDC_BLOCKED_SITE_FILE, m_theApp->m_sBlockedSiteFile );
	SetDlgItemText( IDC_BLOCKED_IMAGE_FILE, m_theApp->m_sBlockedImageFile );

}



/////////////////////////////////////////////////////////////////////////////
// 
BOOL CBlockedListDlg::OnSetActive() 
{
	return CPropertyPage::OnSetActive();
}

/////////////////////////////////////////////////////////////////////////////
// 
BOOL CBlockedListDlg::OnKillActive() 
{
	if( m_bDirty )
		VerifyBlockingFiles( );
	return CPropertyPage::OnKillActive();
}

/////////////////////////////////////////////////////////////////////////////
// 
int CBlockedListDlg::FindString( LPCSTR pItem )
{
	LVFINDINFO lv;

	lv.flags = LVFI_STRING;
	lv.psz = pItem;
	return( m_BlockedListCtrl.FindItem( &lv, -1 ) );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::HighlightItem( int iIndex )
{
	LVITEM lv;
	memset( &lv, 0, sizeof( lv ) );

	lv.mask = LVIF_STATE;
	lv.iItem = iIndex;
	lv.stateMask = LVIS_SELECTED | LVIS_FOCUSED; 
	lv.state = LVIS_SELECTED | LVIS_FOCUSED; 
	m_BlockedListCtrl.EnsureVisible( iIndex, FALSE );
	m_BlockedListCtrl.SetItem( &lv );

}


/////////////////////////////////////////////////////////////////////////////
// 
//if index is -1 then we are adding a new item
void CBlockedListDlg::EditItem( int iIndex, CString* pItem /* = NULL */ )
{
	CString sUrl;
	
	if( pItem )
		sUrl = *pItem;

	BOOL bAccept = (BOOL)BLOCKED_SITE;
	
	if( iIndex >= 0 )
	{
		sUrl = m_BlockedListCtrl.GetItemText( iIndex, 0 );
		LV_ITEM lv;
		memset( &lv, 0, sizeof( lv ) );
		lv.iItem = iIndex;
		lv.mask = LVIF_IMAGE;
		m_BlockedListCtrl.GetItem( &lv );
		bAccept = (lv.iImage == BLOCKED_SITE);
	}

	CEditUrlDlg dlg;
	dlg.m_sUrl = sUrl;
	dlg.m_ListType = BLOCKED_LIST;
	dlg.m_bAcceptCookie = bAccept;

	if( dlg.DoModal( ) == IDOK )
	{
		if( iIndex >= 0 ) {
			CString s = m_BlockedListCtrl.GetItemText( iIndex, 0 );
			if( s.Compare( dlg.m_sUrl ) == 0 && ( dlg.m_bAcceptCookie == bAccept ) )
				return;
			m_BlockedListCtrl.DeleteItem( iIndex );
		}

		//check for duplicate entries		
		int i = FindString(dlg.m_sUrl);
		if( i >= 0 ) 
		{
			HighlightItem( i );
			ShowError( IDS_ENTRYEXISTS, MB_OK );
		} else {
			InsertUrl( dlg.m_sUrl, dlg.m_bAcceptCookie ); 
		}
		UpdateBlockedList( );
	}
} 


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::InsertUrl( LPCSTR pUrl, BOOL bBlockSite ) 
{
	LV_ITEM lv;
	memset( &lv, 0, sizeof( lv ) );

	int iIndex = m_BlockedListCtrl.GetItemCount( );
	lv.mask =  LVIF_IMAGE | LVIF_TEXT;
	lv.iItem = iIndex;
	lv.pszText = (LPSTR)pUrl;

	if( bBlockSite )
		lv.iImage = BLOCKED_SITE;
	else
		lv.iImage =  BLOCKED_IMAGE;

	iIndex = m_BlockedListCtrl.InsertItem( &lv );
	lv.mask = LVIF_STATE;
	lv.iItem = iIndex;
	HighlightItem( iIndex );
	m_BlockedListCtrl.SetFocus( );
	DoModified( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::DoModified( BOOL bDirty )
{
	m_bDirty = bDirty;
	SetModified( bDirty );
	EnableDisableControls( );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::OnAdd() 
{
	EditItem( -1 );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::OnEdit() 
{
	int nItem = m_BlockedListCtrl.GetNextItem(-1, LVNI_SELECTED ); 
	if( nItem >= 0 )
		EditItem( nItem );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::OnDelete() 
{
	int nItem = m_BlockedListCtrl.GetNextItem(-1, LVNI_SELECTED ); 
	if( nItem >= 0 )
	{
		m_BlockedListCtrl.DeleteItem( nItem );
			while( nItem >= m_BlockedListCtrl.GetItemCount( ) )
			nItem--;
		HighlightItem( nItem );
		UpdateBlockedList( );
		DoModified( TRUE );
	}

}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::OnDblclkBlockedList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UNREFERENCED_PARAMETER( pNMHDR );
	
	OnEdit( );	
	*pResult = 0;
}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::OnReset() 
{
	m_theApp->ReadDataFiles();
	PopulateBlockedList( );		
	DoModified( FALSE );
	CPropertyPage::OnReset();
}

/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::UpdateBlockedList( )
{
	char sz[512];
	LV_ITEM lv;
	
	m_theApp->m_BlockedSiteList.RemoveAll( );
	m_theApp->m_BlockedImageList.RemoveAll( );
	
	int iCount = m_BlockedListCtrl.GetItemCount( );
	for( int i = 0; i < iCount; i++ )
	{
		memset( &lv, 0, sizeof( lv ) );
		lv.mask = LVIF_TEXT | LVIF_IMAGE;
		lv.iItem = i;
		lv.cchTextMax = sizeof( sz );
		lv.pszText = sz;

		m_BlockedListCtrl.GetItem( &lv );
		if( lv.iImage == BLOCKED_SITE )
			m_theApp->m_BlockedSiteList.Add( sz );
		else
			m_theApp->m_BlockedImageList.Add( sz );


	}
}

/////////////////////////////////////////////////////////////////////////////
// 
BOOL CBlockedListDlg::OnApply() 
{

    UpdateBlockedList( );
	WriteBlockedFile( );
	WriteCookieCopFile( );
	DoModified( FALSE );
	PopulateBlockedList();

    //load the data files - popups and blocked sites/images
    LoadDataFiles( );

	DoModified( FALSE );
	return CPropertyPage::OnApply();

}

/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::OnBrowseImage() 
{
	OnBrowse( IDC_BLOCKED_IMAGE_FILE, &m_theApp->m_sBlockedImageFile );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::OnBrowseSite() 
{
	OnBrowse( IDC_BLOCKED_SITE_FILE, &m_theApp->m_sBlockedSiteFile );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::OnBrowse( int nID, CString* pFile ) 
{
	int index;
	LPCSTR p = strrchr( *pFile , '.' );
	CString sTmp = *pFile;
	CString sExt;
	if( p )
		sExt.Format( "*%s", p );
	else
		sExt = "*.htm";

	sExt.MakeLower( );
	if( strstr( sExt, ".htm" ) )
		index = 1;
	else
		index = 2;
	

	CFileDialog dlg(TRUE,
		sExt,
		sTmp, 0,
		"HTML Files (*.htm)|*.htm|Images (*.gif;*.jpg)|*.gif;*.jpg|All Files (*.*)|*.*||",
		this );

	dlg.m_ofn.lpstrTitle = "CookieCop HTML File";
	dlg.m_ofn.Flags |= OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_NOCHANGEDIR;
	dlg.m_ofn.nFilterIndex = index;

	if( dlg.DoModal() == IDOK ) 
	{
		SetDlgItemText( nID, dlg.GetPathName() );	
	}
	
}

/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::OnChangeBlockedImageFile() 
{
	CString s;
	GetDlgItemText( IDC_BLOCKED_IMAGE_FILE, s );	
	if( s.CompareNoCase( m_theApp->m_sBlockedImageFile ) )
	{
		DoModified( TRUE );
		m_theApp->m_sBlockedImageFile = s;
	}
}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::OnChangeBlockedSiteFile() 
{
	CString s;
	GetDlgItemText( IDC_BLOCKED_SITE_FILE, s );	
	if( s.CompareNoCase( m_theApp->m_sBlockedSiteFile ) )
	{
		DoModified( TRUE );
		m_theApp->m_sBlockedSiteFile = s;
	}
	
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CBlockedListDlg::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	m_DropTarget.Register( GetDlgItem( IDC_BLOCKED_LIST ) );

	m_ToolTip.Create( this );
	m_ToolTip.AddTool( GetDlgItem( IDC_DELETE ), IDS_DELETE );
	m_ToolTip.AddTool( GetDlgItem( IDC_NEW ), IDS_NEW );
	m_ToolTip.AddTool( GetDlgItem( IDC_EDIT ),IDS_EDIT );

	COLORREF crBackground = RGB(192,192,192);	//v1.2
	CSize size;
	BITMAP bm;
	CBitmap bmp;
	bmp.LoadBitmap( IDB_BLOCKED_IMAGE );  //order is important see enum {BLOCKED_SITE,BLOCKED_IMAGE}
	bmp.GetBitmap( &bm );	
	m_ImageList.Create(	IDB_BLOCKED, bm.bmWidth, 1, crBackground );
	m_ImageList.Add( &bmp, crBackground );

	m_BlockedListCtrl.SetImageList( &m_ImageList, LVSIL_SMALL );

	//force report style
	HWND hwndLV = m_BlockedListCtrl.m_hWnd;
    DWORD dwStyle = GetWindowLong( hwndLV, GWL_STYLE );  
    SetWindowLong( hwndLV, GWL_STYLE, dwStyle | LVS_REPORT ); 

	//CListCtrl
	CRect rc;
	m_BlockedListCtrl.GetClientRect( &rc );            
	
	LV_COLUMN lvcol;
	lvcol.mask = LVCF_TEXT;
	lvcol.cx = rc.Width( );
	lvcol.pszText = "URL";
	m_BlockedListCtrl.InsertColumn( 0, &lvcol );
	m_BlockedListCtrl.SetColumnWidth( 0, rc.Width( ) );


	PopulateBlockedList();
	VerifyBlockingFiles( );

	DoModified( m_bDirty );
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


/////////////////////////////////////////////////////////////////////////////
// 
LRESULT CBlockedListDlg::OnDrop(WPARAM wParam, LPARAM lParam )
{
	UNREFERENCED_PARAMETER(wParam);

	if( !m_theApp->m_bDisableBlockedList )
	{
		m_theApp->m_pDlg->SetForegroundWindow( );	
			
		CString s = (LPSTR)lParam;
		if( ExtractUrl( &s, FALSE ) )
			EditItem( -1, &s );
		else
			ShowError( IDS_BADURL, MB_ICONHAND );
	}
	return 0;
}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::OnBlockOptions() 
{
	DoModified( TRUE );	
}

/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::OnDisableblocklist() 
{
	m_theApp->m_bDisableBlockedList = IsDlgButtonChecked( IDC_DISABLEBLOCKLIST );
	DoModified( TRUE );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::EnableDisableControls( )
{
	BOOL bEnable = ( m_theApp->m_bDisableBlockedList == FALSE );
	BOOL bEnableSelected = FALSE;
	BOOL bEnableUpdated = FALSE;
	
	if( bEnable )
	{
		bEnableSelected = ( m_BlockedListCtrl.GetNextItem(-1, LVNI_SELECTED) != -1 );
		bEnableUpdated = m_bDirty;
	} 

	m_BlockedListCtrl.EnableWindow( bEnable );

	GetDlgItem( IDC_NEW )->EnableWindow( bEnable );

	//if nothing is selected disable appropriate buttons
	GetDlgItem( IDC_DELETE )->EnableWindow( bEnableSelected );
	GetDlgItem( IDC_EDIT )->EnableWindow( bEnableSelected );

}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedListDlg::VerifyBlockingFiles( )
{
	CString sFile;

	//qualify the path name if necessary
	if( m_theApp->m_sBlockedSiteFile.IsEmpty() == FALSE )
	{
		if( strchr( m_theApp->m_sBlockedSiteFile ,'\\' ) == 0 )
		{
			QualifyPathName( &sFile, m_theApp->m_sBlockedSiteFile );
			m_theApp->m_sBlockedSiteFile = sFile;
		}
	}

	if( m_theApp->m_sBlockedImageFile.IsEmpty() == FALSE )
	{
		if( strchr( m_theApp->m_sBlockedImageFile ,'\\' ) == 0 )
		{
			QualifyPathName( &sFile, m_theApp->m_sBlockedImageFile );
			m_theApp->m_sBlockedImageFile = sFile;
		}
	}

	VerifyFile( "Blocking", m_theApp->m_sBlockedSiteFile );
	VerifyFile( "Blocking", m_theApp->m_sBlockedImageFile );
}


