/*=========================================================================*/
/*                          BlockedSiteDlg.cpp                             */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*                                                                         */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/

#include "stdafx.h"
#include "cookiecop.h"
#include "BlockedSiteDlg.h"
#include "BlockedListDlg.h"
#include "hlp/helpids.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBlockedSiteDlg 


CBlockedSiteDlg::CBlockedSiteDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CBlockedSiteDlg::IDD, pParent)

{
	//{{AFX_DATA_INIT(CBlockedSiteDlg)
	m_sTime = _T("");
	m_sEvent = _T("");
	m_sURL = _T("");
	m_sPattern = _T("");
	m_sContents = _T("");
	//}}AFX_DATA_INIT
}


void CBlockedSiteDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBlockedSiteDlg)
	DDX_Text(pDX, IDD_TIME, m_sTime);
	DDX_Text(pDX, IDC_EVENT, m_sEvent);
	DDX_Text(pDX, IDD_URL, m_sURL);
	DDX_Text(pDX, IDC_PATTERN, m_sPattern);
	DDX_Text(pDX, IDD_CONTENTS, m_sContents);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CBlockedSiteDlg, CDialog)
	//{{AFX_MSG_MAP(CBlockedSiteDlg)
	ON_WM_CTLCOLOR()
	ON_BN_CLICKED(IDC_NEXT, OnNext)
	ON_BN_CLICKED(IDC_PREVIOUS, OnPrevious)
	ON_BN_CLICKED(IDHELP, OnHelp)
	ON_BN_CLICKED(IDC_BLOCK_IMAGE, OnBlockImage)
	ON_BN_CLICKED(IDC_BLOCK_SITE, OnBlockSite)
	ON_BN_CLICKED(IDC_BLOCK_IMAGE_TEXT, OnBlockImageText)
	ON_BN_CLICKED(IDC_BLOCK_SITE_TEXT, OnBlockSiteText)
	ON_BN_CLICKED(IDSAVE, OnSave)
	ON_BN_CLICKED(IDC_DELETE, OnDelete)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// BlockedSiteDlg message handlers
BOOL CBlockedSiteDlg::OnInitDialog() 
{

	//save the initial event
	m_eventtype = m_pEventData->event;

	CDialog::OnInitDialog();

	CString s;

	switch( m_pEventData->event )
	{
		case EVENT_IMAGE_BLOCKED:
			s = "An attempt to retrieve an image from this site was blocked.";
			break;

		case EVENT_SITE_BLOCKED:
			s = "Connection to this site was blocked.";
			break;
	
		case EVENT_CONNECT_SSL:
			s = "Connection to a secure site.  Blocking is not recommended.";
			GetDlgItem( IDC_DELETE )->EnableWindow( FALSE );
			GetDlgItem( IDSAVE )->EnableWindow( FALSE );
			break;

		case EVENT_CONNECT:
			s = "Connection to site.";
			GetDlgItem( IDC_DELETE )->EnableWindow( FALSE );
			CheckRadioButton( IDC_BLOCK_SITE, IDC_BLOCK_IMAGE, IDC_BLOCK_IMAGE );
            break;

        case EVENT_REFERRER_REMOVED_JS:
            GetDlgItem( IDC_DELETE )->EnableWindow( FALSE );
            s = "A JavaScript referrer command has been removed.";
            break;

        case EVENT_REFERER_REMOVED:
            //s = "The referer field has been removed before connecting to this site.";
            GetDlgItem( IDC_DELETE )->EnableWindow( FALSE );
            s = "Connection to site. The referer field has been removed.";
            break;
	
	}

	SetDlgItemText( IDC_EVENT, s );

	//v1.2 -- corrected background colors
	HANDLE hBitmap = LoadImage( AfxGetInstanceHandle( ), MAKEINTRESOURCE( IDB_BLOCKED ),IMAGE_BITMAP,0,0,LR_LOADTRANSPARENT|LR_LOADMAP3DCOLORS);
	hBitmap = (HBITMAP)GetDlgItem( IDC_BLOCK_SITE )->SendMessage( BM_SETIMAGE, IMAGE_BITMAP, (LPARAM)(HBITMAP)hBitmap );  
	if( hBitmap )
		::DeleteObject( hBitmap );

 	hBitmap = LoadImage( AfxGetInstanceHandle( ), MAKEINTRESOURCE( IDB_BLOCKED_IMAGE  ),IMAGE_BITMAP,0,0,LR_LOADTRANSPARENT|LR_LOADMAP3DCOLORS);
	hBitmap = (HBITMAP)GetDlgItem( IDC_BLOCK_IMAGE )->SendMessage( BM_SETIMAGE, IMAGE_BITMAP, (LPARAM)(HBITMAP)hBitmap );  
	if( hBitmap )
		::DeleteObject( hBitmap );

	hBitmap = LoadImage( AfxGetInstanceHandle( ), MAKEINTRESOURCE( IDB_SMALL_COP ),IMAGE_BITMAP,0,0,LR_LOADTRANSPARENT|LR_LOADMAP3DCOLORS);
	hBitmap = (HBITMAP)GetDlgItem( IDC_COP )->SendMessage( STM_SETIMAGE, IMAGE_BITMAP, (LPARAM)(HBITMAP)hBitmap );  
	if( hBitmap )
		::DeleteObject( hBitmap );


	UpdateData( TRUE );	
	UpdateControls( );
			
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedSiteDlg::UpdateControls( )
{
	int index = (int)m_pListCtrl->GetItemData( m_nItem );
	
	ASSERT (index <= m_theApp->m_EventList.GetUpperBound() );
	
	m_EventData = m_theApp->m_EventList.GetAt( index );

	switch( m_EventData.event )
	{
		case EVENT_SITE_BLOCKED:
			//CheckDlgButton( IDC_BLOCK_SITE, TRUE );
			CheckRadioButton( IDC_BLOCK_SITE, IDC_BLOCK_IMAGE, IDC_BLOCK_SITE );
			m_sPattern = m_EventData.sPattern;
			break;
		case EVENT_IMAGE_BLOCKED:
			//CheckDlgButton( IDC_BLOCK_IMAGE, TRUE );
			CheckRadioButton( IDC_BLOCK_SITE, IDC_BLOCK_IMAGE, IDC_BLOCK_IMAGE );
			m_sPattern = m_EventData.sPattern;
			break;
	
		case EVENT_CONNECT_SSL:
		case EVENT_CONNECT:
        case EVENT_REFERER_REMOVED:
        case EVENT_REFERRER_REMOVED_JS:
		//	CheckDlgButton( IDC_BLOCK_IMAGE, FALSE );
		//	CheckDlgButton( IDC_BLOCK_SITE, FALSE );
			//CheckRadioButton( IDC_BLOCK_SITE
			m_sPattern = m_EventData.sUrl;
			break;
	}
	

	m_sTime.Format( "%s  entry %u", m_EventData.sTime, m_nItem + 1 );
	m_sURL			= m_EventData.sUrl;
	m_sContents		= m_EventData.sCommand;

	SetDlgItemText( IDC_PATTERN, m_sPattern );
	UpdateData( FALSE );

}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedSiteDlg::RemoveBlockedSite() 
{
	int iCount = m_theApp->m_BlockedSiteList.GetSize( );

	for( int i=0; i < iCount; i++ )
	{
		if( strcmp( m_sPattern, m_theApp->m_BlockedSiteList.GetAt( i ) ) == 0 ) 
		{
			m_theApp->m_BlockedSiteList.RemoveAt( i );
			break;
		}
	}						
}

/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedSiteDlg::RemoveBlockedImage() 
{
	int iCount = m_theApp->m_BlockedImageList.GetSize( );

	for( int i=0; i < iCount; i++ )
	{
		if( strcmp( m_sPattern, m_theApp->m_BlockedImageList.GetAt( i ) ) == 0 ) 
		{
			m_theApp->m_BlockedImageList.RemoveAt( i );
			break;
		}
	}						

}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedSiteDlg::OnDelete() 
{
	UpdateData( TRUE );
	if( m_sPattern.GetLength( ) )
	{
		RemoveBlockedImage( );
		RemoveBlockedSite( );
		m_theApp->m_pBlockedListDlg->PopulateBlockedList( );
		m_theApp->m_pBlockedListDlg->DoModified( TRUE );
	}
	EndDialog( TRUE );
	
}

/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedSiteDlg::OnSave() 
{
	UpdateData( TRUE );
	if( m_sPattern.GetLength( ) )
	{
		RemoveBlockedImage( );
		RemoveBlockedSite( );

		if( IsDlgButtonChecked( IDC_BLOCK_SITE ) )
			m_theApp->m_BlockedSiteList.Add( m_sPattern );
		else
			m_theApp->m_BlockedImageList.Add( m_sPattern );
		
		m_theApp->m_pBlockedListDlg->PopulateBlockedList( );
		m_theApp->m_pBlockedListDlg->DoModified( TRUE );
	}
	EndDialog( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
HBRUSH CBlockedSiteDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
	
	COLORREF crColor;
	
	if( nCtlColor == CTLCOLOR_STATIC )
	{
		int iID = pWnd->GetDlgCtrlID( );
		switch( iID )
		{
			case IDC_EVENT_TEXT	   :
			case IDC_TIME_TEXT     :
			case IDC_URL_TEXT	   :
			case IDC_CONTENTS_TEXT :
			case IDC_SITE          :
				 crColor = COLOR_DIALOG_EVENT;
				 break;
			default:
				return hbr;
		}
		pDC->SetTextColor( crColor );
	}
	
	
	// TODO: Return a different brush if the default is not desired
	return hbr;
}

/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedSiteDlg::OnNext() 
{
int nSave = m_nItem;

	if( m_pListCtrl->GetItemCount( ) == 0 )
		return;
	
	while( TRUE )
	{
		if( (m_nItem+1) >= m_pListCtrl->GetItemCount( ) )
		{
			if( ShowError( IDS_LASTITEM, MB_YESNO, this ) == IDYES ) 
			{
				m_nItem = -1;
			} else {
				m_nItem = nSave;
				return;	
			}
		}
		//check to see if the same event type
		int index = (int)m_pListCtrl->GetItemData( ++m_nItem );
		m_EventData = m_theApp->m_EventList.GetAt( index );
		if( m_EventData.event == m_eventtype )
			break;
	}	

	UpdateControls( );

}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedSiteDlg::OnPrevious() 
{
int nSave = m_nItem;
	if( m_pListCtrl->GetItemCount( ) == 0 )
		return;

	while( TRUE )
	{
		if( (m_nItem-1) < 0 )
		{
			if( ShowError( IDS_FIRSTITEM, MB_YESNO, this ) == IDYES )
			{
				m_nItem = m_pListCtrl->GetItemCount( );
			} else {
				m_nItem = nSave;
				return;	
			}
		}
		//check to see if the same event type
		int index = (int)m_pListCtrl->GetItemData( --m_nItem );
		m_EventData = m_theApp->m_EventList.GetAt( index );
		if( m_EventData.event == m_eventtype )
			break;
	}	

	UpdateControls( );
	
}

/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedSiteDlg::OnHelp() 
{
	m_theApp->CWinApp::WinHelp(IDH_edit_blocked_site_list);
}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedSiteDlg::OnBlockImage() 
{
	CheckRadioButton( IDC_BLOCK_SITE, IDC_BLOCK_IMAGE, IDC_BLOCK_IMAGE );
	GetDlgItem( IDC_BLOCK_IMAGE )->SetFocus( );
	GetDlgItem( IDSAVE )->EnableWindow( TRUE );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedSiteDlg::OnBlockSite() 
{
	CheckRadioButton( IDC_BLOCK_SITE, IDC_BLOCK_IMAGE, IDC_BLOCK_SITE );
	GetDlgItem( IDC_BLOCK_SITE )->SetFocus( );
	GetDlgItem( IDSAVE )->EnableWindow( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedSiteDlg::OnBlockImageText() 
{
	OnBlockImage();	
}

/////////////////////////////////////////////////////////////////////////////
// 
void CBlockedSiteDlg::OnBlockSiteText() 
{
	OnBlockSite();
}

