/*=========================================================================*/
/*                         CDlgPropSheet.cpp                               */
/*                                                                         */
/*              Manages the context menu and tray icon                     */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include "stdafx.h"
#include "CookieCop.h"
#include "DlgPropSheet.h"
#include "resource.h"
#include "Globals.h"
#include "EventData.h"
#include "Log.h"
#include "WarningDlgs.h"
#include "PerformanceDlg.h"
#include "Viewer.h"
#include "BrowserDlg.h"
#include "hlp/helpids.h"



#define TIMER_SPEED 500

UINT TaskbarCallbackMsg = RegisterWindowMessage("TaskbarMsg");	
UINT SOCKET_MSG	= RegisterWindowMessage( "SocketMsg" );

FARPROC fpWTSRegisterSession; //fast user switching under WinXP

/////////////////////////////////////////////////////////////////////////////
// CDlgPropSheet

IMPLEMENT_DYNAMIC(CDlgPropSheet, CPropertySheet)

CDlgPropSheet::CDlgPropSheet(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(nIDCaption, pParentWnd, iSelectPage)
{
}

CDlgPropSheet::CDlgPropSheet(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{
}

CDlgPropSheet::~CDlgPropSheet()
{
	
}


BEGIN_MESSAGE_MAP(CDlgPropSheet, CPropertySheet)
	//{{AFX_MSG_MAP(CDlgPropSheet)
		ON_WM_TIMER()
		ON_WM_SYSCOMMAND()
		ON_WM_QUERYENDSESSION()
	ON_WM_CREATE()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
	ON_REGISTERED_MESSAGE(TaskbarCallbackMsg, OnTaskbarNotify)	//~9
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDlgPropSheet message handlers

BOOL CDlgPropSheet::OnInitDialog() 
{

	//This prevents our icon from being displayed on the task bar
	ModifyStyle( WS_VISIBLE , WS_MINIMIZEBOX, SWP_HIDEWINDOW | SWP_NOACTIVATE );
	BOOL bResult = CPropertySheet::OnInitDialog();
	
	m_bShowIconInTray = FALSE;

	// Configure our system menu
	GetSystemMenu( TRUE ); //revert the menu
	CMenu* pSysMenu = GetSystemMenu( FALSE ); //and grab the handle
	pSysMenu->DeleteMenu( SC_MAXIMIZE, MF_BYCOMMAND );	  
	pSysMenu->DeleteMenu( SC_SIZE, MF_BYCOMMAND );
	pSysMenu->EnableMenuItem( SC_RESTORE, MF_DISABLED | MF_GRAYED );
	

	//This is required to force MFC to create the controls
	//on the individual property pages!  Normally, they are not 
	//created until the user switches to the tab
	for( int i = 0; i < GetPageCount( ); i++ )
	{
		SetActivePage( i );
	}
	SetActivePage( 0 );

	// Set the icon for this dialog.  The framework does this automatically
	// when the application's main window is not a dialog
	HICON m_hIcon = AfxGetApp()->LoadIcon(IDI_ICON_IDLE);
	SetIcon(m_hIcon, TRUE);         // Set big icon
	SetIcon(m_hIcon, FALSE);        // Set small icon

	
	//Start minimized and hide icon
	//check for errors -- if a listen socket was not created start visble
	if( m_theApp->m_Listen.IsSocketValid( ) == FALSE || m_theApp->m_bShowOpeningWindow )
	{
		DisplayCookieCopWindow( );
		CPropertySheet::WinHelp(IDH_configuring_your_browser);

	} else {
		
		SetParent( m_theApp->m_pcwnd );	//hide our property sheet inside an invisible window
	}

	ShowTrayIcon( );

    //display the viewer
    if( m_theApp->m_bViewerVisible )
        m_theApp->m_pCookieDlg->OnShowViewer( );  

	return bResult;
}

/////////////////////////////////////////////////////////////////////////////
// 
void CDlgPropSheet::RemoveTrayIcon( )
{
    m_SystemTray.cbSize = sizeof(NOTIFYICONDATA);
    m_SystemTray.hWnd   = GetSafeHwnd( );
    m_SystemTray.uID    = 1;
    Shell_NotifyIcon( NIM_DELETE, &m_SystemTray );
}

/////////////////////////////////////////////////////////////////////////////
// 
BOOL CDlgPropSheet::OnCommand(WPARAM wParam, LPARAM lParam)
{
	switch( wParam )
	{
		case IDOK:
            SaveChanges( );
			//fall through

		case SC_MINIMIZE:
			ShowWindow( SW_HIDE );
            EnableTimers( FALSE );
			return( TRUE );
		
		case SC_CLOSE:
            if( ShowWarningMessage( ) == FALSE )
                return( TRUE );
            if( m_theApp->m_pViewerDlg )
                m_theApp->m_pViewerDlg->EndDialog( TRUE );
            m_theApp->m_Listen.StopListen( );   //TEST THIS
            m_theApp->m_pDlg->EndDialog( TRUE );
			return( FALSE );

		case IDCANCEL:
		{
			ShowWindow( SW_HIDE );
            EnableTimers( FALSE );
			for( int i = 0; i < GetPageCount( ); i++ )
			{
				GetPage(i)->OnReset();
                GetPage(i)->SetModified( FALSE );
			}
			return( TRUE );
		}

	    case ID_ABOUT:
            DisplayCookieCopWindow( );
            SetActivePage( GetPageCount( )-1 );
            break;

        case SC_RESTORE:
			DisplayCookieCopWindow( );
			break;

        case ID_SHOW_VIEWER:
            if( m_theApp->m_pViewerDlg == NULL ) {
                m_theApp->m_pCookieDlg->OnShowViewer();
            } else {
                CRect rc;
                m_theApp->m_pViewerDlg->GetWindowRect( rc );
                SaveViewerPosition( &rc );

                m_theApp->m_pViewerDlg->SendMessage( WM_COMMAND, IDCANCEL, 0 );

            }
            break;

		case ID_DISABLE:
        {
			if( m_theApp->m_bDisableCookieCop )
				m_theApp->m_bDisableCookieCop = FALSE;
			else
				m_theApp->m_bDisableCookieCop = TRUE;

			CString s;
			GetWindowText( s );

			if( m_theApp->m_bDisableCookieCop )
			{
				s += " -- Disabled";
				SetWindowText( s );
			} else {
				int i = s.Find(" -");
				if( i > 0 )
				{
					s = s.Left( i );
				}
			}
			SetWindowText( s );

            //save the state
            WritePrivateProfile( SZ_DISABLED, m_theApp->m_bDisableCookieCop );

            //update the browser tab
            m_theApp->m_pBrowserDlg->SetProxyOptions( );

			//update the systray icon
            OnTimer( 0 );   
            break;
            }

        case ID_DISABLE_PROXY:
        {
            m_theApp->m_bDisableProxy = !m_theApp->m_bDisableProxy;
            m_theApp->EnableProxy( !m_theApp->m_bDisableProxy );

            //save the state
            WritePrivateProfile( SZ_DISABLE_PROXY, m_theApp->m_bDisableProxy );
            
            //update the browser tab
            m_theApp->m_pBrowserDlg->SetProxyOptions( );
			
            //update the systray icon
            OnTimer( 0 );   
        }
        break;

        case IDCLOSE:
            if( m_theApp->m_pDlg->ShowWarningMessage( ) == FALSE )
                return(TRUE);  //cancel the close
        break;

        case ID_HELP:        
			m_theApp->CWinApp::WinHelp(IDH_cookiecop);
            return( TRUE );
	}

    return CPropertySheet::OnCommand(wParam, lParam);
}

/////////////////////////////////////////////////////////////////////////////
// 
LRESULT CDlgPropSheet::OnSocketMsg( WPARAM wParam, LPARAM lParam)	
{
	
	if( WSAGETSELECTERROR(lParam) != 0 )
	{
		closesocket( wParam );
		ShowWinsockError( wParam );
		return TRUE;
	}
	
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// 
LRESULT CDlgPropSheet::OnTaskbarNotify( WPARAM wParam, LPARAM lParam)	
{
	UNREFERENCED_PARAMETER(wParam);

	switch (lParam)
	{			
		case WM_MOUSEMOVE:
		{
			CString s;

			if( m_theApp->m_dwStats[STATS_ACCEPTED] || m_theApp->m_dwStats[STATS_REJECTED] )
			{
				CString a = FormatNumber( m_theApp->m_dwStats[STATS_ACCEPTED] );
				CString r = FormatNumber( m_theApp->m_dwStats[STATS_REJECTED] );
				s.Format( "Cookies - Accepted: %s Rejected: %s", a ,r   );
			 } else {
				
				s = "CookieCop";
			 }

			m_SystemTray.cbSize = sizeof(NOTIFYICONDATA);
			m_SystemTray.hWnd   = GetSafeHwnd( );
			m_SystemTray.uID    = 1;
			m_SystemTray.uFlags =  NIF_TIP;
			strncpy( m_SystemTray.szTip, s, sizeof(m_SystemTray.szTip) );
			Shell_NotifyIcon( NIM_MODIFY, &m_SystemTray );
		}
		break;

		case WM_RBUTTONUP:
		{
			POINT pt;
			GetCursorPos( &pt );
			
            CMenu menu;
            menu.LoadMenu( IDR_MENU1 );
            
            CMenu* pMenu;
            pMenu = menu.GetSubMenu( 0 );
            pMenu->SetDefaultItem( 0, TRUE );

			//Enable or Disable CC
            WORD wOptions = MF_CHECKED;
            WORD wEnabled = MF_ENABLED;  

			if( m_theApp->m_bDisableCookieCop )
                wOptions = MF_UNCHECKED;


            //no, you can't do this because the user may be using
            //something other than IE
            //if( m_theApp->m_bDisableProxy )
              //  wEnabled = MF_GRAYED;
            
            pMenu->CheckMenuItem( ID_DISABLE, wOptions );
            pMenu->EnableMenuItem( ID_DISABLE, wEnabled );

            //if the user has enabled the gateway forwarding option
            //then disabling the proxy setting in IE will block
            //them from connecting to the internet. Remove the menu item
            if( m_theApp->m_bEnableGateway || m_theApp->AllowEnableIEProxy( ) == FALSE )
            {
                pMenu->RemoveMenu( ID_DISABLE_PROXY, MF_BYCOMMAND );

            } else {
                //Enable or Disable Proxy
			    wOptions = MF_CHECKED;
                wEnabled = MF_ENABLED;  
                if( m_theApp->m_bDisableProxy )
                    wOptions = MF_UNCHECKED;

                pMenu->CheckMenuItem( ID_DISABLE_PROXY, wOptions );
                pMenu->EnableMenuItem( ID_DISABLE_PROXY, wEnabled );
            }

            //update the browser tab
            m_theApp->m_pBrowserDlg->SetProxyOptions( );


            //viewer options
			wOptions = MF_CHECKED;
            wEnabled = MF_ENABLED;  
            if( m_theApp->m_pViewerDlg == NULL )
                wOptions = MF_UNCHECKED;

            pMenu->CheckMenuItem( ID_SHOW_VIEWER, wOptions );
            
            
            //v1.2 -- see Q135788
			SetForegroundWindow( );
			pMenu->TrackPopupMenu( TPM_LEFTALIGN , pt.x, pt.y,this );
			PostMessage( WM_NULL, 0, 0);
			break;
		}

		case WM_LBUTTONDBLCLK:
			DisplayCookieCopWindow( );
			break;
		
	}
	return 0;
}



/////////////////////////////////////////////////////////////////////////////
// 
void CDlgPropSheet::OnTimer(UINT nIDEvent) 
{
    UNREFERENCED_PARAMETER( nIDEvent );
    UpdateTrayIcon( );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CDlgPropSheet::UpdateTrayIcon( ) 
{
static int nPreviousIcon;
static HICON hPrevIcon;

	if( m_bShowIconInTray == FALSE )
		return;
	
	int nID = IDI_ICON_IDLE;

	if(	m_theApp->m_CookieState & STATE_CONNECTED )
		nID = IDI_ICON_CONNECTED; 

	if(	m_theApp->m_CookieState & STATE_COOKIE_RECEIVED )
		nID = IDI_ICON_ACCEPTED;	

	if(	m_theApp->m_CookieState & STATE_COOKIE_REJECTED )
	{
		nID = IDI_ICON_REJECTED;	
		if(	m_theApp->m_CookieState & STATE_COOKIE_RECEIVED )
			nID = IDI_ACCEPT_AND_REJECT;
	}

	//disabled?
	if( m_theApp->m_bDisableCookieCop )
		nID = IDI_DISABLED;

    //no way to tell if the proxy is disabled using Netscape
    //if( m_theApp->m_bDisableProxy )
    //    nID = IDI_PROXY_DISABLED;

	//reset state here
	m_theApp->m_CookieState &= ~STATE_COOKIE_RECEIVED;
	m_theApp->m_CookieState &= ~STATE_COOKIE_REJECTED; 

	//if state has changed then update the systray icon
	if( nPreviousIcon != nID )
	{
        ZeroMemory( &m_SystemTray, sizeof( m_SystemTray ) );

        m_SystemTray.cbSize = sizeof(NOTIFYICONDATA);
		m_SystemTray.hWnd   = GetSafeHwnd( );
		m_SystemTray.uID    = 1;
        m_SystemTray.hIcon  = (HICON)LoadImage( AfxGetInstanceHandle( ), MAKEINTRESOURCE( nID ),IMAGE_ICON, 16,16,LR_DEFAULTCOLOR );
		m_SystemTray.uFlags = NIF_ICON;
		m_SystemTray.uCallbackMessage = TaskbarCallbackMsg;
		Shell_NotifyIcon( NIM_MODIFY, &m_SystemTray );
		nPreviousIcon = nID;

		if( hPrevIcon )
           DestroyIcon( hPrevIcon );

        hPrevIcon = m_SystemTray.hIcon;
	}

}


/////////////////////////////////////////////////////////////////////////////
// 
void CDlgPropSheet::OnSysCommand(UINT nID, LPARAM lParam) 
{
	if( nID == SC_CLOSE ) 
	{
        CRect rc;
        GetWindowRect( rc );

        //check to see if the user clicked on the X to close the window
        //if so, just minimize
        if( LOWORD( lParam ) > ( rc.left + rc.Width( ) / 2 ) && HIWORD( lParam ) < rc.Height( ) )
        {
            ShowWindow( SW_HIDE );
            EnableTimers( FALSE );
            return;
        }

        //lParam will be zero if the IDCLOSE message is sent from
        //our BHO
        if( lParam )
            if( ShowWarningMessage( ) == FALSE )
                return;
	}

	if( nID == SC_MINIMIZE )
	{
		ShowWindow( SW_HIDE );
        EnableTimers( FALSE );
		return;
	}

	CPropertySheet::OnSysCommand(nID, lParam);
}

/////////////////////////////////////////////////////////////////////////////
//called from Clisten to display our icon in the system tray
void CDlgPropSheet::ShowTrayIcon( )
{

	if( m_bShowIconInTray == FALSE )
	{
		ZeroMemory( &m_SystemTray, sizeof( m_SystemTray ) );
        m_SystemTray.cbSize = sizeof( NOTIFYICONDATA );
		m_SystemTray.hWnd   = GetSafeHwnd( );
		m_SystemTray.uID    = 1;
		m_SystemTray.hIcon  = (HICON)LoadImage( AfxGetInstanceHandle( ), MAKEINTRESOURCE( IDI_ICON_IDLE ),IMAGE_ICON, 16,16,LR_DEFAULTCOLOR );
		m_SystemTray.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
		m_SystemTray.uCallbackMessage = TaskbarCallbackMsg;
		_tcscpy( m_SystemTray.szTip, SZ_APPNAME );

        Shell_NotifyIcon( NIM_ADD, &m_SystemTray );

        //Ignore the return code. Sometimes other apps 
        //cause the NIM_ADD to initially fail, however the icon still
        //displays in the tray.
        //if( !Shell_NotifyIcon( NIM_ADD, &m_SystemTray ) )
		//		AfxMessageBox( "System tray error." );

	}
	DoTimer( TIMER_SPEED );
	m_bShowIconInTray = TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// 
void CDlgPropSheet::DoTimer( int iSpeed )
{
static BOOL bCreated = FALSE;

    if( !bCreated )
	{
		SetTimer( ICON_TIMER, iSpeed, NULL );		
		bCreated = TRUE;
	}
}

/////////////////////////////////////////////////////////////////////////////
// 
void CDlgPropSheet::DisplayCookieCopWindow( )
{
	m_theApp->m_pDlg->SetParent( NULL  );
	m_theApp->m_pDlg->ShowWindow( SW_RESTORE );
    EnableTimers( TRUE );

	GetSystemMenu( TRUE ); //revert the menu
	CMenu* pSysMenu = GetSystemMenu( FALSE ); //and grab the handle
	pSysMenu->DeleteMenu( SC_MAXIMIZE, MF_BYCOMMAND );	  
	pSysMenu->DeleteMenu( SC_SIZE, MF_BYCOMMAND );
	pSysMenu->EnableMenuItem( SC_RESTORE, MF_DISABLED | MF_GRAYED );

	if( m_theApp->m_bOnTop )
		m_theApp->m_pDlg->SetWindowPos( &wndTopMost,0,0,0,0,SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE );
	else 
		m_theApp->m_pDlg->SetWindowPos( &wndNoTopMost,0,0,0,0,SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);

	m_theApp->m_pDlg->SetForegroundWindow( );	

    //update the browser tab
    m_theApp->m_pBrowserDlg->SetProxyOptions( );
}

/////////////////////////////////////////////////////////////////////////////
//call before closing the CookieCop window 
//returns TRUE to allow close, false to cancel close
BOOL CDlgPropSheet::QuerySaveChanges( ) 
{
	if( GetDlgItem( ID_APPLY_NOW )->IsWindowEnabled( ) )
	{
		if( int i = ShowError( IDS_SAVECHANGES, MB_YESNOCANCEL | MB_ICONQUESTION  | MB_SETFOREGROUND  ) )
		{	
			if( i == IDCANCEL )
				return( FALSE );
			
			if( i == IDYES )
			    SaveChanges( );
		}		
	}
	return TRUE;	//shutdown

}

/////////////////////////////////////////////////////////////////////////////
//Display a warning message if the user tries to shutdown CookieCop
//Also, check to see if any changes need to be saved
//Return true if it is okay to shutdown
BOOL CDlgPropSheet::ShowWarningMessage( ) 
{
BOOL bShowWarning = TRUE;
	
//	if( m_theApp->m_nStartupMode == STARTUP_MANUAL )
//		bShowWarning = FALSE;

//	if( m_theApp->m_nStartupMode == STARTUP_BROWSER )
//		bShowWarning = FALSE;
		
	if( bShowWarning )
		if( ShowWarning( IDS_SHUTDOWN, IDYES ) == IDNO )
			return( FALSE );
    
    return( QuerySaveChanges( ) );
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CDlgPropSheet::OnQueryEndSession() 
{
	if (!CPropertySheet::OnQueryEndSession())
		return FALSE;

    return( QuerySaveChanges( ) );
}

/////////////////////////////////////////////////////////////////////////////
// 
int CDlgPropSheet::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CPropertySheet::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	
    ModifyStyle( DS_CONTEXTHELP | DS_MODALFRAME, WS_MINIMIZEBOX );    
	// Configure our system menu
	GetSystemMenu( TRUE ); //revert the menu
	CMenu* pSysMenu = GetSystemMenu( FALSE ); //and grab the handle
    pSysMenu->EnableMenuItem( SC_MINIMIZE, MF_ENABLED );
	pSysMenu->DeleteMenu( SC_MAXIMIZE, MF_BYCOMMAND );	  
	pSysMenu->DeleteMenu( SC_SIZE, MF_BYCOMMAND );
	pSysMenu->EnableMenuItem( SC_RESTORE, MF_DISABLED | MF_GRAYED );
	
	return 0;
}


/////////////////////////////////////////////////////////////////////////////
// 
void CDlgPropSheet::EnableTimers( BOOL bEnable )
{
    m_theApp->m_pPerformanceDlg->EnableTimer( bEnable );
    m_theApp->m_pStatsDlg->EnableTimer( bEnable ); 
}


/////////////////////////////////////////////////////////////////////////////
// 
void CDlgPropSheet::OnDestroy() 
{
    CPropertySheet::OnDestroy();
}

/////////////////////////////////////////////////////////////////////////////
// 
void CDlgPropSheet::SaveChanges( ) 
{
    ResetFileErrorWarnings( );

	if( GetDlgItem( ID_APPLY_NOW )->IsWindowEnabled( ) )
	{
		for( int i = 0; i < GetPageCount( ); i++ )
		{
			GetPage(i)->OnApply();
		}
	}		

}
