/*=========================================================================*/
/*                           Config.cpp                                    */
/*                                                                         */
/*                  Functions for configuring the browser                  */
/*                                                                         */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include "stdafx.h"
#include "CookieCop.h"
#include "globals.h"
#include "Registry.h"
#include "Config.h"


/////////////////////////////////////////////////////////////////////////////
//
BOOL GetBrowsersPaths( LPCSTR lpBrowser, LPSTR lpPath )
{
	static char szAppPath[]="Software\\Microsoft\\Windows\\CurrentVersion\\App Paths";
	char szKey[MAX_PATH];
	DWORD dwResult = MAX_PATH;
	CRegistry reg;
	
	*lpPath = 0;
	wsprintf( szKey, "%s\\%s", szAppPath, lpBrowser );

	if( reg.Open( HKEY_LOCAL_MACHINE, szKey, KEY_ALL_ACCESS ) == ERROR_SUCCESS )
	{
		reg.QueryValue( lpPath, "", &dwResult );
		reg.Close( );
	}
	return( *lpPath != 0 );	
}


/////////////////////////////////////////////////////////////////////////////
//
BOOL GetVersionNumber( LPTSTR szFile, LPTSTR szBuf, LPINT lpiBuf, int iNumOfFields )
{
	DWORD dwUseless = 0;
	UINT iBufSize = 0;
	VS_FIXEDFILEINFO* lpFFI = NULL;
	TCHAR s[MAX_PATH] = {0};

	DWORD dwLen = GetFileVersionInfoSize(szFile, &dwUseless);
	if(dwLen == 0)
		return( FALSE );

	HANDLE  hMem = GlobalAlloc(GMEM_MOVEABLE, dwLen);
	LPVOID lpVI  = GlobalLock(hMem);

	GetFileVersionInfo(szFile, NULL, dwLen, lpVI);
	VerQueryValue(lpVI, __TEXT("\\"), reinterpret_cast<LPVOID*>(&lpFFI), &iBufSize);
	DWORD dwVer1 = lpFFI->dwFileVersionMS;
	DWORD dwVer2 = lpFFI->dwFileVersionLS;
	GlobalUnlock( hMem );
	GlobalFree( hMem );

   // Fill return buffers
   if(szBuf != NULL)
   {
      wsprintf(s, __TEXT("%d.%d.%d.%d"), HIWORD(dwVer1), LOWORD(dwVer1), HIWORD(dwVer2), LOWORD(dwVer2));
      lstrcpy(szBuf, s);
   }

   if( lpiBuf != NULL )
   {
      for(int i = 0 ; i < iNumOfFields; i++)
      {
         if(i == 0)
            lpiBuf[i] = HIWORD(dwVer1);
         if(i == 1)
            lpiBuf[i] = LOWORD(dwVer1);
         if(i == 2)
            lpiBuf[i] = HIWORD(dwVer2);
         if(i == 3)
            lpiBuf[i] = LOWORD(dwVer2);
      }
   }

   return( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
//
BOOL ConfigSaved_IE( )
{
	CString sFileName;
	char szBuf[30];

	QualifyPathName( &sFileName, SZ_BROWSER_INI );
	return( GetPrivateProfileString( SZ_CONFIG_IE, SZ_SAVED, "", szBuf,sizeof(szBuf), sFileName ) );
}

/////////////////////////////////////////////////////////////////////////////
//
BOOL ConfigRestored_IE( )
{
	CString sFileName;
	char szBuf[30];

	QualifyPathName( &sFileName, SZ_BROWSER_INI );
	return( GetPrivateProfileString( SZ_CONFIG_IE, SZ_RESTORED, "", szBuf,sizeof(szBuf), sFileName ) );
}


/////////////////////////////////////////////////////////////////////////////
//
void SaveRegistrySettingsIE( )
{
	CString sFileName;
	CRegistry reg;
	char szValue[MAX_PATH];
	char szBuf[MAX_PATH];
	char szTmp[MAX_PATH];
	DWORD dwResult = MAX_PATH;

	QualifyPathName( &sFileName, SZ_BROWSER_INI );

	//if not already saved
	if( ConfigSaved_IE( ) == FALSE )
	{
		//Proxy Settings
		if( reg.Open( HKEY_CURRENT_USER, SZ_IE_REG_SETTINGS, KEY_ALL_ACCESS ) == ERROR_SUCCESS )
		{
			reg.QueryValue( szValue, "ProxyServer", &dwResult );
			WritePrivateProfileString( SZ_CONFIG_IE, SZ_PROXY, szValue, sFileName );

			//AllowCookies
			reg.QueryValue( dwResult, "AllowCookies" );
			wsprintf( szBuf, "%lu", dwResult );
			WritePrivateProfileString( SZ_CONFIG_IE, SZ_ALLOW_COOKIES, szBuf, sFileName );
			
			//ProxyEnable
			reg.QueryValueLength( dwResult,  "ProxyEnable" );
			HGLOBAL hByte = GlobalAlloc( GHND, dwResult );
			LPBYTE lpByte = (LPBYTE)GlobalLock( hByte );
			reg.QueryValue( lpByte, dwResult, "ProxyEnable" );
			szBuf[0] = 0;
			for( int i = 0; i < (int)dwResult; i++ )
			{
				BYTE ch = lpByte[i];
				wsprintf( szTmp, "%x ", ch );
				strcat( szBuf, szTmp );
			}	
			GlobalUnlock( hByte );
			GlobalFree( hByte );
			
			WritePrivateProfileString( SZ_CONFIG_IE, SZ_PROXY_ENABLE, szBuf, sFileName );
		}	

		WritePrivateProfileString( SZ_CONFIG_IE, SZ_SAVED, "1", sFileName );
		WritePrivateProfileString( SZ_CONFIG_IE, SZ_RESTORED, "1", sFileName );
	}

}

/////////////////////////////////////////////////////////////////////////////
//
BOOL RestoreConfiguration_IE( )
{
	BOOL bReturn = FALSE;
	char szPath[MAX_PATH];
	GetModuleFileName( NULL, szPath, sizeof( szPath ) );
	GetShortPathName( szPath, szPath, sizeof(szPath) );

	LPSTR p = strrchr( szPath, '\\' );
	if( p )
	{
		*p = 0;
		strcat( p, "\\CookieHlpr.dll" );
		HINSTANCE hLib = LoadLibrary( szPath );
		if( hLib > (HINSTANCE)HINSTANCE_ERROR )
		{
			FARPROC fp;
			fp = GetProcAddress( hLib, "RestoreIE" );
			if( fp )
				bReturn = fp();
			FreeLibrary( hLib );
		} else {
			ShowError( IDS_COOKIEHLPR_ERROR, MB_OK | MB_ICONEXCLAMATION );
		}

	}
	return( bReturn );	
}	
	
	
	
	
/////////////////////////////////////////////////////////////////////////////
//
void AutoConfigureIE( )
{

	CRegistry reg;

	char szKey[MAX_PATH];
	char szValue[MAX_PATH];
	DWORD dwResult = MAX_PATH;
	CString sFileName;

	QualifyPathName( &sFileName, SZ_BROWSER_INI  );
	
	szValue[0] = 0;
	wsprintf( szKey, "%s\\%s", SZ_IE_REG_SETTINGS, "ProxyServer" );

	SaveRegistrySettingsIE( );

	if( reg.Open( HKEY_CURRENT_USER, SZ_IE_REG_SETTINGS, KEY_ALL_ACCESS ) == ERROR_SUCCESS )
	{
		reg.QueryValue( szValue, "ProxyServer", &dwResult );
	
		int i,index = 0;
		CStringArray sArray;
		CString sTmp;
		LPSTR p = szValue;

		//originally this code would auto configure the browser even if they
		//had a proxy server such as a firewall already configured. But lack
		//of time prevented full testing of this function.
		//Currently, if they have anything set in the ProxyServer registry entry
		//we will bail out.

		if( *szValue )
		{
			ShowError( IDS_PROXY_CONFIG_ERR, MB_OK | MB_ICONEXCLAMATION );
			return;
		}


		if( *szValue )
		{
			//parse the components in the string
			while( *p )
			{
				sTmp += *p;
				
				if( *p == ';' )
				{
					sArray.Add( sTmp );
					sTmp = "";
				}
					
				p++;
			}
			if( sTmp.IsEmpty( ) == FALSE )
				sArray.Add( sTmp );
		} else {
			//reg entry was empty, fake some strings
			sArray.Add( "http=" );
			sArray.Add( "https=");
		}
			
		index = sArray.GetSize();
		//replace with our entries
		for( i = 0; i < index; i++ )
		{
			sTmp = sArray.GetAt( i );
			sTmp.MakeLower( );
			if( sTmp.Find( "http=" ) != -1 ) // SET THIS ENTRY AND ITS PORT TO THE GATEWAY
			{
				sTmp.Format( "http=%s:%u;", m_theApp->m_sCookieIP, m_theApp->m_nCookiePort);
				sArray.SetAt( i, sTmp );
			}
			if( sTmp.Find( "https=" ) != -1 ) 
			{
				sTmp.Format( "https=%s:%u;", m_theApp->m_sCookieIP,m_theApp->m_nCookiePort);
				sArray.SetAt( i, sTmp );
			}

		}
		//rebuild the string
		sTmp = "";
		for( i = 0; i < index; i++ )
		{
			sTmp += sArray.GetAt( i );
		}

		//update the registry
		reg.SetValue( sTmp, "ProxyServer" );

		//AllowCookies
		reg.SetValue( 1, "AllowCookies" );
		//ProxyEnable
		BYTE byEnable[]={1,0,0,0};
		reg.SetValueBinary( byEnable, sizeof(byEnable), "ProxyEnable" );	
		reg.Close( );
		WritePrivateProfileString( SZ_CONFIG_IE, SZ_RESTORED, "", sFileName );
		ShowError( IDS_AUTO_CONFIG_OK, MB_OK | MB_ICONEXCLAMATION );
	} else {
		ShowError( IDS_AUTO_CONFIG_ERROR, MB_OK | MB_ICONEXCLAMATION  );
	}

}
