/*=========================================================================*/
/*                           CookieCop.cpp                                 */
/*                                                                         */
/*                   Startup module for CookieCop 2                        */
/*                                                                         */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*=========================================================================*/
#include "stdafx.h"
#include "CookieCop.h"
#include "Globals.h"
#include "CookieCopDlg.h"
#include "DlgPropSheet.h"
#include "Log.h"
#include "EventData.h"
#include "CookieListDlg.h"
#include "BlockedListDlg.h"
#include "Stats.h"
#include "About.h"
#include "WarningDlgs.h"
#include "BrowserDlg.h"
#include "PopupDlg.h"
#include "Config.h"
#include "PerformanceDlg.h"
#include "Registry.h"
#include "hlp/helpids.h"
#include <Atlbase.h>

//requires the platform sdk
#include <wininet.h>


#pragma comment( lib, "Ws2_32.lib" )

extern CCookieCopApp* m_theApp;

void SaveRegistrySettingsIE( );


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCookieCopApp

BEGIN_MESSAGE_MAP(CCookieCopApp, CWinApp)
	//{{AFX_MSG_MAP(CCookieCopApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCookieCopApp construction

CCookieCopApp::CCookieCopApp()
{
	m_bDefaultAction = ACCEPT_COOKIE;
	m_bShowOpeningWindow = FALSE;
	m_pViewerDlg = NULL;
	m_nSessionID =0;
	m_bDisableCookieCop = FALSE;
	m_nSessionCount = 0;
    m_nWinsockBufSize = DEFAULT_BUFSIZE;
    m_nRecvBufSize = 0;
    m_nSendBufSize = 0;
    m_nLargestHostRecv = 0;
    m_bUseDefaultBufferSizes = TRUE;
    ZeroMemory( &m_OptimizeStates, sizeof(m_OptimizeStates) );
    
	#ifdef _DEBUGFILE // For debugging
	m_bDebugFile = FALSE;
	#endif

    WSADATA   wsaData;
    WORD wVersionRequested = WINSOCK_VERSION;
    if( WSAStartup( wVersionRequested, &wsaData) != 0 )
    {
        ShowError( "Winsock failed" );
    } else {
        if( wsaData.wHighVersion < 0x0200 )
        {
            ShowError( "Winsock 2 or higher is required." );

        }
    }                

}

/////////////////////////////////////////////////////////////////////////////
// 
CCookieCopApp::~CCookieCopApp()
{
	WSACleanup( );
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CCookieCopApp object

CCookieCopApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CCookieCopApp initialization

BOOL CCookieCopApp::InitInstance()
{

	//
    //Single Instance -- check for our invisable window, if present show
	//the window and exit
    //
	#define SZ_WINDOWNAME "CookieCopWindow"
	HWND hwnd = FindWindow( NULL, SZ_WINDOWNAME ); 
 
	if( hwnd )
	{
		if(::GetWindow( hwnd, GW_CHILD ) )		
			hwnd = ::GetWindow( hwnd, GW_CHILD );	
		else
			hwnd = FindWindow(NULL, "CookieCop" );
		::SetParent( hwnd, NULL );
		::ShowWindow( hwnd, SW_SHOWNORMAL );
		::SetForegroundWindow( hwnd );
		return( FALSE );
	}



	AfxOleInit();	//for DropTarget 

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.


#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	AfxInitRichEdit( );         //for viewer dialog 1.3

	//for shared data
	m_theApp = this;

	//check for Active Desktop for the Start with IE option.
	m_IsActiveDesktop = IsActiveDesktop( );
	
	//check hosts file
	if( WriteHostsFile( "127.0.0.1", SZ_APPNAME ) == FALSE )
		ShowWarning( IDS_HOSTS_FILE_ERROR, IDOK );

	ReadDataFiles( );
	ResetStats( );
	if( m_theApp->m_bSaveStats )
		ReadStatsFile( );
	m_dwStats[STATS_BYTES_RECEIVED] = 0;
	m_dwStats[STATS_BYTES_SENT] = 0;


    //get current proxy setting (IE only)
    m_bDisableProxy = QueryProxySetting( );

	//Create our property sheet and property page dialogs
	CDlgPropSheet	propdlg( "CookieCop 2", AfxGetMainWnd() );
	CCookieCopDlg	cookieDlg;
	CLog			logDlg;
	CCookieListDlg  cookielistDlg;
	CStats			statsDlg;
	CAbout			aboutDlg;
	CBrowserDlg		browserDlg;
	CBlockedListDlg blockedlistDlg;
    CPopupDlg       popupDlg;
    CPerformanceDlg performanceDlg;
	
	m_pDlg            = &propdlg;
	m_pLog            = &logDlg;
	m_pCookieListDlg  = &cookielistDlg;
	m_pBlockedListDlg = &blockedlistDlg;	
    m_pCookieDlg      = &cookieDlg;
    m_pPopupDlg       = &popupDlg;
    m_pPerformanceDlg = &performanceDlg;
    m_pStatsDlg       = &statsDlg;
    m_pBrowserDlg     = &browserDlg;

	propdlg.m_psh.dwFlags |= PSH_MODELESS;	//must be modeless for SC_CLOSE to work

	propdlg.AddPage( &cookieDlg );
	propdlg.AddPage( &browserDlg );
	propdlg.AddPage( &logDlg );
	propdlg.AddPage( &cookielistDlg );
	propdlg.AddPage( &blockedlistDlg );
    propdlg.AddPage( &popupDlg );
    propdlg.AddPage( &performanceDlg );
	propdlg.AddPage( &statsDlg );
	propdlg.AddPage( &aboutDlg );

	CWnd cwnd;
	CRect rc;

	rc.SetRectEmpty( );
	
	//create a hidden window
	LPCTSTR pClass = AfxRegisterWndClass( 0 ); 
	if( cwnd.CreateEx( WS_EX_TOOLWINDOW, pClass, SZ_WINDOWNAME, WS_OVERLAPPED, rc, NULL, 0) == 0 )
	{
		ShowError( IDS_CREATEWINDOWERROR, MB_OK | MB_ICONHAND );
		return FALSE;	//bail out
	}

	m_pcwnd = &cwnd;	
	SaveRegistrySettingsIE( );      //save proxy settings 
    popupDlg.VerifyPopupFiles( );   //show missing files errors
	propdlg.DoModal( );             //Show the cookiecop dialog
	WriteStatsFile( );              //CC has been closed, save the stats

	cwnd.DestroyWindow( );
   ::UnregisterClass( pClass, AfxGetInstanceHandle() );

	// Since the dialog has been closed, return FALSE so that we exit the
	// application, rather than start the application's message pump.

 	return FALSE;
}

void CCookieCopApp::ResetStats( )
{
	for( int i = 0; i < STATS_MAX; i++ )
		m_dwStats[i] = 0;

    InterlockedExchange( &m_dwDNSCacheMisses, 0 );
    InterlockedExchange( &m_dwDNSCacheHits, 0 );

}

/////////////////////////////////////////////////////////////////////////////
//  returns TRUE if proxy is disabled
BOOL CCookieCopApp::QueryProxySetting( )
{
    CRegKey reg;
    DWORD dwEnable = 0;
    if( reg.Open( HKEY_CURRENT_USER, SZ_INTERNET_KEY ) == ERROR_SUCCESS )
    {
        reg.QueryValue( dwEnable, SZ_PROXY_KEY );

    }
    return( (BOOL)dwEnable == 0 );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopApp::EnableProxy( BOOL bEnable ) 
{
    CRegKey reg;
    if( reg.Create( HKEY_CURRENT_USER, SZ_INTERNET_KEY ) == ERROR_SUCCESS )
    {
        reg.SetValue( (DWORD)bEnable, SZ_PROXY_KEY );
        InternetSetOption(NULL, INTERNET_OPTION_SETTINGS_CHANGED, NULL, 0);
	    InternetSetOption(NULL, INTERNET_OPTION_REFRESH, NULL, 0);
    }
}

/////////////////////////////////////////////////////////////////////////////
// examines the registry for IE proxy settings.  if none have been entered 
// then don't allow the Enable IE proxy option on the systray menu or the
// browser tab.   The Enable IE proxy option only works for IE Lan, it does
// not work with dial-up connections or other browsers.  
BOOL CCookieCopApp::AllowEnableIEProxy( ) 
{
    CRegistry reg;

    if( m_theApp->m_bEnableGateway )
        return FALSE;

    //examine IE proxy settings
    if( reg.Open( HKEY_CURRENT_USER, SZ_IE_REG_SETTINGS, KEY_ALL_ACCESS ) == ERROR_SUCCESS )
    {
        char szValue[MAX_PATH];
        DWORD dwResult = MAX_PATH;
        if( reg.QueryValue( szValue, "ProxyServer", &dwResult ) == ERROR_SUCCESS )
        {
            if( dwResult > 0 )
                return( TRUE );
        }
    }
    return( FALSE );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopApp::ReadDataFiles( )
{
	//init data files
	ReadBlockedFile( );
	ReadRejectFile( );
	ReadAcceptFile( );
	ReadCookieCopFile( );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopApp::WinHelp(DWORD dwData, UINT nCmd) 
{
	int iPage = m_pDlg->GetActiveIndex();

	switch( iPage )
	{
		case 0: dwData = IDH_setup;			    break;
		case 1: dwData = IDH_browser_dialog;    break;
		case 2:	dwData = IDH_event_log;		    break;
		case 3:	dwData = IDH_cookie_list;	    break;
		case 4:	dwData = IDH_blocked_site_list; break;
        case 5: dwData = IDH_navigation;        break;
        case 6: dwData = IDH_performance;       break;
		case 7:	dwData = IDH_statistics;        break;	
		default:
			dwData = IDH_cookiecop;	
	}
	CWinApp::WinHelp(dwData, nCmd);
}

/////////////////////////////////////////////////////////////////////////////
// Check for WDU and Active Desktop -- see MSDN Online article Q216840
//
BOOL CCookieCopApp::IsActiveDesktop( )
{
HKEY hKey;
DWORD dwSize;
BYTE  szValue[MAX_PATH];
#define ACTIVE_DESKTOP	"SOFTWARE\\Classes\\CLSID\\{AEB6717E-7E19-11d0-97EE-00C04FD91972}\\InProcserver32"
 
	if( RegOpenKeyEx( HKEY_LOCAL_MACHINE, ACTIVE_DESKTOP, 0, KEY_ALL_ACCESS, &hKey ) == ERROR_SUCCESS )
	{
		dwSize = sizeof( szValue );
		RegQueryValueEx( hKey,"", 0, NULL, szValue, &dwSize );
		RegCloseKey( hKey );
		
		szValue[sizeof(szValue)-1] = 0;
		strlwr( (LPSTR)szValue );
		if( strstr( (LPSTR)szValue, "shell32.dll" ) )
			return( TRUE );
	}

	return( FALSE );
}


