// CookieCop.h : main header file for the COOKIECOP application
//

#if !defined(AFX_COOKIECOP_H__13A9D909_704C_11D3_A180_00A0C90AE44B__INCLUDED_)
#define AFX_COOKIECOP_H__13A9D909_704C_11D3_A180_00A0C90AE44B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"		// main symbols
#include "Globals.h"
#include "EventData.h"
#include "DropTarget.h"
#include "Listen.h"
#include "Viewer.h"
#include "CookieCopDlg.h"
#include "Stats.h"

//cookie options
typedef enum
{
	REJECT_COOKIE,
	ACCEPT_COOKIE,
	SESSION_COOKIE,
} DEFAULT_ACTION;

//performance tab
typedef struct
{
    BOOL bBufferError;
    BOOL bTimeout;                
} OPTIMIZE_STRUCT;


/////////////////////////////////////////////////////////////////////////////
// CCookieCopApp:
// See CookieCop.cpp for the implementation of this class
//
class CDlgPropSheet;
class CListen;
class CLog;
class CEventData;
class CBlockedListDlg;
class CCookieListDlg;
class CPopupDlg;
class CPerformanceDlg;
class CBrowserDlg;
      
class CCookieCopApp : public CWinApp
{
public:
	CCookieCopApp();
	~CCookieCopApp();
	void ResetStats( );
	void ReadDataFiles( );
	BOOL IsActiveDesktop( );
    BOOL QueryProxySetting( );
    void EnableProxy( BOOL bEnable );
    BOOL AllowEnableIEProxy(  );



// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CCookieCopApp)
	public:
	virtual BOOL InitInstance();
	virtual void WinHelp(DWORD dwData, UINT nCmd = HELP_CONTEXT);
	//}}AFX_VIRTUAL


// Implementation
	CWnd*				m_pcwnd;	
	CDlgPropSheet*		m_pDlg;		//for task bar notification
	CLog*				m_pLog;
	CListen				m_Listen;
	CCookieListDlg*		m_pCookieListDlg;
	CBlockedListDlg* 	m_pBlockedListDlg;	
	CViewer*			m_pViewerDlg;
    CCookieCopDlg*      m_pCookieDlg;
    CPopupDlg*          m_pPopupDlg;
    CPerformanceDlg*    m_pPerformanceDlg;
    CStats*             m_pStatsDlg;
    CBrowserDlg*        m_pBrowserDlg;

	//Data Files
	CStringArray		m_BlockedImageList;	//blocked images
	CStringArray		m_BlockedSiteList;	//blocked sites
	CStringArray		m_RejectList;		//reject cookies from 
	CStringArray		m_AcceptList;		//accept cookies from 

    CByteArray          m_BlockedSiteFileData;
    CByteArray          m_BlockedImageFileData;
    CByteArray          m_PopupFileData;	
    CByteArray          m_PopupFileDataJS;

	//Default action for receiving cookies 
	DEFAULT_ACTION		m_bDefaultAction;
	LONG m_Head;
	LONG m_Tail;

    int             m_nRecvBufSize;
    int             m_nSendBufSize;
    BOOL            m_bUseDefaultBufferSizes;
	BOOL			m_IsActiveDesktop;
		
	//Event list array 
	CEventList		m_EventList;
	WORD			m_wEventMask;	//mask for m_EventList 

	//Cookiecop.ini entries
	CString			m_sCookieIP;
	int				m_nCookiePort;
	CString			m_sGatewayIP;
	int				m_nGatewayPort;
	BOOL			m_bEnableGateway;
	int				m_nMaxEvents;
	BOOL			m_bSearchJavaScript;
	BOOL			m_bOnTop;				//display our window on top ( for drag drop )
	CString			m_sBlockedSiteFile;		//path to replace blocked sites
	CString			m_sBlockedImageFile;	//path to replace blocked images
    CString         m_sPopupFile;
    CString         m_sPopupFileJS;
	BOOL			m_bDisableCookieList;
	BOOL			m_bDisableBlockedList;
	BOOL			m_bDisableCookieCop;		//1.3 disable all CC options
	BOOL			m_bDisableLog;
	BOOL			m_bSaveStats;	
	CString			m_sCookieFile_Nav;  //navigator cookie.txt file location
	CString			m_sCookieFile_IE;	//IE cookie folder location
	BOOL			m_bScanFolder_IE;
	BOOL			m_bScanFolder_Nav;
	BOOL			m_bConfirmDelete;	//display recycle bin confirmation dlg
	BOOL			m_bShowOpeningWindow;
    BOOL            m_bCheckReferer;
    int             m_nWinsockBufSize;
    BOOL            m_bViewerVisible;   //redisplay viewer on startup
    int             m_nConnectionTimeout;   //timeout
    BOOL            m_bUseDefaultTimeOut;
    //dns cache
    int             m_nDNS_CacheSize;
    BOOL            m_bDNSCacheDisable;
    LONG            m_dwDNSCacheMisses;
    LONG            m_dwDNSCacheHits;
    
    //performance tab
    OPTIMIZE_STRUCT m_OptimizeStates;
    int             m_nLargestHostRecv;

    //proxy
    BOOL            m_bDisableProxy;
    
	//stats dlg
	DWORD	        m_dwStats[STATS_MAX];

	// If this _DEBUGFILE is defined then a [ ] Debug file option will be displayed
	// in the About box.  All data transmitted through cookiecop is then written to
	// a file named C:\DEBUG.TXT
	#ifdef _DEBUGFILE  
	CFile			m_DebugFile;
	BOOL			m_bDebugFile; 
	#endif

    int             m_nPopupMode;   //0 for prompts, 1 for no prompts
    BOOL            m_bPopupCheckDisabled;
	STARTUP_MODE    m_nStartupMode;

	//for icon display, see COOKIE_COP_STATE 
	int             m_CookieState;

	CCriticalSection m_criticalSectionLog;
	CCriticalSection m_criticalSectionDebug;
	CCriticalSection m_criticalSectionViewer;
    CCriticalSection m_criticalSectionDNS;
	LONG m_nSessionID;			//for viewer
	LONG m_nSessionCount;		//	''
	LONG m_dwSessionStartTime;	//	''
	LONG m_dwSessionBytes;	   	//	''

	//{{AFX_MSG(CCookieCopApp)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_COOKIECOP_H__13A9D909_704C_11D3_A180_00A0C90AE44B__INCLUDED_)
