/*=========================================================================*/
/*                          CookieCopDlg.cpp                               */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include "stdafx.h"
#include "CookieCop.h"
#include "CookieCopDlg.h"
#include "Listen.h"
#include "Globals.h"
#include "Log.h"
#include "DlgPropSheet.h"
#include "Registry.h"
#include "WarningDlgs.h"
#include "Config.h"
#include "Viewer.h"
#include "hlp/helpids.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CCookieCopDlg dialog
CCookieCopDlg::CCookieCopDlg( ) : CPropertyPage( CCookieCopDlg::IDD )
{
	//{{AFX_DATA_INIT(CCookieCopDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDI_ICON_IDLE);
}

/////////////////////////////////////////////////////////////////////////////
// Unload the Fast User Switching notification
CCookieCopDlg::~CCookieCopDlg( )
{
 	HMODULE hModule = LoadLibrary( "wtsapi32" );
	if( (int)hModule > 32 )
	{
		typedef BOOL ( WINAPI *WTS)(HWND );
		WTS fpWTSUnRegisterSession = (WTS)GetProcAddress( hModule, "WTSUnRegisterSessionNotification" );
		if( fpWTSUnRegisterSession )
		{
			fpWTSUnRegisterSession( GetSafeHwnd( ) );
		}
		FreeLibrary( hModule );
	}	
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCookieCopDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CCookieCopDlg, CPropertyPage)
	//{{AFX_MSG_MAP(CCookieCopDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_DESTROY()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_GATEWAY_ENABLE, OnGatewayEnable)
	ON_BN_CLICKED(IDC_DEFAULTS, OnDefaults)
	ON_BN_CLICKED(IDC_ONTOP, OnOntop)
	ON_CBN_EDITCHANGE(IDC_COOKIECOP_IP, OnEditchangeCookiecopIp)
	ON_EN_CHANGE(IDC_COOKIECOP_PORT, OnChangeCookiecopPort)
	ON_CBN_EDITCHANGE(IDC_GATEWAY_IP, OnEditchangeGatewayIp)
	ON_CBN_SELCHANGE(IDC_GATEWAY_IP, OnSelchangeGatewayIp)
	ON_CBN_SELCHANGE(IDC_COOKIECOP_IP, OnSelchangeCookiecopIp)
	ON_EN_CHANGE(IDC_GATEWAY_PORT, OnChangeGatewayPort)
	ON_BN_CLICKED(IDC_STARTUP_BROWSER, OnStartupBrowser)
	ON_BN_CLICKED(IDC_STARTUP_MANUAL, OnStartupManual)
	ON_BN_CLICKED(IDC_STARTUP_WINDOWS, OnStartupWindows)
	ON_BN_CLICKED(IDC_EXIT, OnExit)
	ON_BN_CLICKED(IDC_RESET_WARNINGS, OnResetWarnings)
	ON_BN_CLICKED(IDC_SHOW_VIEWER, OnShowViewer)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_WTSSESSION_CHANGE, OnSessionChange ) 
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCookieCopDlg message handlers

BOOL CCookieCopDlg::PreTranslateMessage(MSG* pMsg) 
{
	m_ToolTip.RelayEvent( pMsg );	
	return CPropertyPage::PreTranslateMessage(pMsg);
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CCookieCopDlg::OnInitDialog()
{
	CPropertyPage::OnInitDialog();

	//check version of ie
	char szPath[MAX_PATH];
	if( GetBrowsersPaths( "Iexplore.exe", szPath ) )
	{
		char szVersion[MAX_PATH];
		int  nVersionNumbers[4];
		GetVersionNumber( szPath, szVersion, nVersionNumbers, 4 );
	
		BOOL bEnable = TRUE;
	
		if( nVersionNumbers[0] < 4 )
			bEnable = FALSE;

		//check for IE 5.0, if so don't allow BHO -- IE has a bug
		//if IE 5.01 or higher then allow the BHO
		if( nVersionNumbers[0] == 5 )
		{
			if( nVersionNumbers[1] == 0 )
			{
				if( nVersionNumbers[2] == 2919 && nVersionNumbers[3] < 6300 )
					bEnable = FALSE;
			} 
		}
		
		GetDlgItem( IDC_STARTUP_BROWSER )->EnableWindow( bEnable );
		if( !bEnable )
		{
			if( m_theApp->m_nStartupMode ==	STARTUP_BROWSER )
				m_theApp->m_nStartupMode = STARTUP_WINDOWS;
		}

	}

    
    //Active desktop
	if( m_theApp->m_IsActiveDesktop == FALSE )
	{
		if( m_theApp->m_nStartupMode == STARTUP_BROWSER )
			m_theApp->m_nStartupMode = STARTUP_WINDOWS;

		GetDlgItem( IDC_STARTUP_BROWSER )->EnableWindow( FALSE );
	}


	// Set the icon for this dialog.  The framework does this automatically
	// when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	UpdateControls( );
	BOOL bOK = FALSE;
	
	//first time? force the dialog to display
	if( m_theApp->m_nCookiePort == 0 )
	{
		OnDefaults();
		OnApply( );
		bOK = StartListen( );
		m_theApp->m_bShowOpeningWindow = TRUE;
	} else {
		m_theApp->m_bShowOpeningWindow = FALSE;
		bOK = StartListen( );
		SetModified( FALSE );
	}

	//tooltips
	m_ToolTip.Create( this );
	m_ToolTip.AddTool( GetDlgItem( IDC_DEFAULTS ), IDS_DEFAULTS );
	m_ToolTip.AddTool( GetDlgItem( IDC_RESET_WARNINGS ), IDS_RESET_WARNINGS );
	m_ToolTip.AddTool( GetDlgItem( IDC_EXIT ), IDS_EXIT );
	
	// check for 256 colors
	CWindowDC dc(this); // device context for painting
	WORD wID;
	if( dc.GetDeviceCaps(NUMCOLORS) != -1 )
		wID = IDB_BANNER_16;
	else
		wID = IDB_BANNER;

	//v1.2
	HANDLE hBitmap = LoadImage( AfxGetInstanceHandle( ), MAKEINTRESOURCE( wID ),IMAGE_BITMAP,0,0,LR_LOADTRANSPARENT|LR_LOADMAP3DCOLORS);
	
    BITMAP bm;
    CRect rc;
    GetObject( hBitmap, sizeof(bm), &bm );
    GetClientRect( rc );
    int center = ( rc.Width( ) - bm.bmWidth ) / 2;	
    
    
    CRect rcbanner;
    GetDlgItem( IDC_BANNER )->GetWindowRect( rcbanner );
    ScreenToClient( rcbanner );
    GetDlgItem( IDC_BANNER )->SetWindowPos( NULL, center, rcbanner.top,bm.bmWidth, bm.bmHeight, SWP_NOSIZE | SWP_NOZORDER   );  

    
    hBitmap = (HBITMAP)GetDlgItem( IDC_BANNER )->SendMessage( STM_SETIMAGE, IMAGE_BITMAP, (LPARAM)(HBITMAP)hBitmap );  
	if( hBitmap )
		::DeleteObject( hBitmap );	


    //load the data files - popups and blocked sites/images
    LoadDataFiles( );

    //Check for Fast User Switching
	HMODULE hModule = LoadLibrary( "wtsapi32" );
	if( (int)hModule > 32 )
	{
		typedef BOOL ( WINAPI *WTS)(HWND, DWORD );
		WTS fpWTSRegisterSession = (WTS)GetProcAddress( hModule, "WTSRegisterSessionNotification" );
		if( fpWTSRegisterSession )
		{
			fpWTSRegisterSession( GetSafeHwnd( ), NOTIFY_FOR_THIS_SESSION );
		}

		FreeLibrary( hModule );
	}	

	
	
	return bOK;  // return TRUE  unless you set the focus to a control
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	CDialog::OnSysCommand(nID, lParam);
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnDestroy()
{
	HBITMAP hBitmap = (HBITMAP)GetDlgItem( IDC_BANNER )->SendMessage( STM_GETIMAGE, IMAGE_BITMAP, 0 );  
	if( hBitmap )
		::DeleteObject( hBitmap );

	WinHelp(0L, HELP_QUIT);
	CDialog::OnDestroy();
}

/////////////////////////////////////////////////////////////////////////////
// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.
void CCookieCopDlg::OnPaint() 
{

	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
		return;	  
	}
}

/////////////////////////////////////////////////////////////////////////////
// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CCookieCopDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::CheckEnableGateway( )
{	
	m_theApp->m_bEnableGateway = IsDlgButtonChecked( IDC_GATEWAY_ENABLE );
	GetDlgItem(	IDC_GATEWAY_IP )->EnableWindow(	m_theApp->m_bEnableGateway );
	GetDlgItem(	IDC_GATEWAY_PORT )->EnableWindow( m_theApp->m_bEnableGateway );
	SetModified( TRUE );
}	

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::UpdateControls() 
{
	char sz[256];

	ParseHostsFile( GetDlgItem( IDC_COOKIECOP_IP), m_theApp->m_sCookieIP );
	ParseHostsFile( GetDlgItem( IDC_GATEWAY_IP	), m_theApp->m_sGatewayIP  );
	
	SetDlgItemText( IDC_COOKIECOP_IP, m_theApp->m_sCookieIP );
	SetDlgItemText( IDC_GATEWAY_IP, m_theApp->m_sGatewayIP );

	SetDlgItemText( IDC_COOKIECOP_PORT, itoa( m_theApp->m_nCookiePort, sz, 10 ) );
	SetDlgItemText( IDC_GATEWAY_PORT, itoa( m_theApp->m_nGatewayPort, sz, 10 ) );
	
	CheckDlgButton( IDC_GATEWAY_ENABLE, m_theApp->m_bEnableGateway );
	CheckDlgButton( IDC_ONTOP, m_theApp->m_bOnTop );
					
	CheckEnableGateway( );

	WORD nID;
	switch( m_theApp->m_nStartupMode )
	{
		case STARTUP_MANUAL	: nID = IDC_STARTUP_MANUAL; break;
		case STARTUP_BROWSER: nID = IDC_STARTUP_BROWSER; break;
		default:
			nID = IDC_STARTUP_WINDOWS;
	}
	CheckRadioButton( IDC_STARTUP_WINDOWS, IDC_STARTUP_MANUAL, nID );
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CCookieCopDlg::OnSetActive() 
{
	return CPropertyPage::OnSetActive();
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnGatewayEnable() 
{
	CheckEnableGateway( );
}


/////////////////////////////////////////////////////////////////////////////
//save the data to ini file
BOOL CCookieCopDlg::OnApply() 
{
char sz[256];
	
	GetDlgItemText( IDC_COOKIECOP_PORT, sz, sizeof(sz) );	
	int nOldPort = m_theApp->m_nCookiePort;
	int nNewPort = atoi( sz );

	if( nOldPort !=	nNewPort )
	{
		if( ShowWarning( IDS_PORT_WARNING, IDYES ) != IDYES )
		{
			SetDlgItemText( IDC_COOKIECOP_PORT, itoa( m_theApp->m_nCookiePort, sz, 10 ) );
			return( FALSE );
		}
	}
	
	m_theApp->m_nCookiePort = nNewPort;

	GetDlgItemText( IDC_COOKIECOP_IP, sz, sizeof(sz) );	
	m_theApp->m_sCookieIP = sz;

	GetDlgItemText( IDC_GATEWAY_IP, sz, sizeof(sz) );	
	m_theApp->m_sGatewayIP = sz;

	GetDlgItemText( IDC_GATEWAY_PORT, sz, sizeof(sz) );	
	m_theApp->m_nGatewayPort = atoi( sz );
	
    WriteCookieCopFile( );

	//if the port number changes restart the listen socket
	if( nOldPort !=	m_theApp->m_nCookiePort ) 
		StartListen( );	
	
	m_theApp->m_nStartupMode = STARTUP_WINDOWS;
	if( IsDlgButtonChecked( IDC_STARTUP_MANUAL  ) ) m_theApp->m_nStartupMode = STARTUP_MANUAL;
	if( IsDlgButtonChecked( IDC_STARTUP_BROWSER ) ) m_theApp->m_nStartupMode = STARTUP_BROWSER;

	CRegistry key;
	#define SZ_KEY "Software\\Microsoft\\Windows\\CurrentVersion\\Run"
	
	char szPath[MAX_PATH];
	GetModuleFileName( NULL, szPath, sizeof( szPath ) );
	GetShortPathName( szPath, szPath, sizeof( szPath ) );

	//note: this setting must be made for all users
    if( m_theApp->m_nStartupMode == STARTUP_WINDOWS )
	{
		if( key.Open( HKEY_LOCAL_MACHINE, SZ_KEY ) == ERROR_SUCCESS )
			key.SetValue( szPath, SZ_APPNAME );
	} else {
		if( key.Open( HKEY_LOCAL_MACHINE, SZ_KEY ) == ERROR_SUCCESS )
			key.DeleteValue( SZ_APPNAME );
	}
	
	LPSTR p = strrchr( szPath, '\\' );
	if( p )
	{
		*p = 0;
		strcat( p, "\\CookieHlpr.dll" );
		HINSTANCE hLib = LoadLibrary( szPath );
		if( hLib > (HINSTANCE)HINSTANCE_ERROR )
		{
			FARPROC fp;
			if( m_theApp->m_nStartupMode == STARTUP_BROWSER )
				fp = GetProcAddress( hLib, "DllRegisterServer" );
			else
				fp = GetProcAddress( hLib, "DllUnregisterServer" );			

			if( fp )
				fp();
			FreeLibrary( hLib );
		} else {
			ShowError( IDS_COOKIEHLPR_ERROR, MB_OK | MB_ICONEXCLAMATION );
		}

	}

	SetModified( FALSE );
	return CPropertyPage::OnApply();
}



/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnReset() 
{
	m_theApp->ReadDataFiles();
	UpdateControls( );
	SetModified( FALSE );
	CPropertyPage::OnReset();
}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnDefaults() 
{
	m_theApp->m_sCookieIP = SZ_APPNAME;
	m_theApp->m_nCookiePort = DEFAULT_PORT;
	m_theApp->m_bEnableGateway = FALSE;
	m_theApp->m_nStartupMode = STARTUP_WINDOWS;
	UpdateControls( );
	SetModified( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnOntop() 
{
	m_theApp->m_bOnTop = IsDlgButtonChecked( IDC_ONTOP );
	m_theApp->m_pDlg->DisplayCookieCopWindow( );

	SetModified( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
BOOL CCookieCopDlg::StartListen( )
{
	//startup
	BOOL bOK = m_theApp->m_Listen.StartListen( );

	if( !bOK )
	{
		GetDlgItem( IDC_COOKIECOP_PORT )->SetFocus( );
	}

	return( bOK );
}



/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnEditchangeCookiecopIp() 
{
	SetModified( TRUE );	
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnChangeCookiecopPort() 
{
		SetModified( TRUE );	
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnEditchangeGatewayIp() 
{
	SetModified( TRUE );	
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnSelchangeGatewayIp() 
{
	SetModified( TRUE );		
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnSelchangeCookiecopIp() 
{
	SetModified( TRUE );		
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnChangeGatewayPort() 
{
	SetModified( TRUE );		
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnStartupBrowser() 
{
	SetModified( TRUE );		
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnStartupManual() 
{
	SetModified( TRUE );		
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnStartupWindows() 
{
	SetModified( TRUE );		
}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnExit() 
{
    //This is the Exit button on CC's setup tab
	if( m_theApp->m_pDlg->ShowWarningMessage( )  )
    {
        if( m_theApp->m_pViewerDlg )
            m_theApp->m_pViewerDlg->EndDialog( TRUE );
   
        m_theApp->m_Listen.StopListen( );
        PostQuitMessage( 0 );
    }

}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnResetWarnings() 
{
	CString sFileName;
	QualifyPathName( &sFileName, SZ_COOKIECOP_INI );

	WritePrivateProfileString( SZ_WARNINGS, NULL, NULL,sFileName  );
	AfxMessageBox( IDS_WARNINGS_RESET, MB_OK );	
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::WinHelp(DWORD dwData, UINT nCmd) 
{
	CPropertyPage::WinHelp(dwData, nCmd);
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::OnShowViewer() 
{
	if( m_theApp->m_pViewerDlg == NULL )
	{
		m_theApp->m_pViewerDlg = new CViewer;
		m_theApp->m_pViewerDlg->Create( IDD_VIEWER_DLG,GetDesktopWindow( ) );
	}
    m_theApp->m_bViewerVisible = TRUE;
    WritePrivateProfile( SZ_VIEWER_OPEN, m_theApp->m_bViewerVisible );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieCopDlg::PostNcDestroy() 
{
	//TEST THIS
    if( m_theApp->m_pViewerDlg )
        m_theApp->m_pViewerDlg->EndDialog( TRUE );

    
    m_theApp->m_pDlg->RemoveTrayIcon( );
    m_theApp->m_Listen.StopListen( );
   
    if( m_theApp->m_pViewerDlg )
    {
        CRect rc;
        m_theApp->m_pViewerDlg->GetWindowRect( rc );
        SaveViewerPosition( &rc );

       	delete m_theApp->m_pViewerDlg;

    }
	m_theApp->m_pViewerDlg = NULL;

	CPropertyPage::PostNcDestroy();
}



/////////////////////////////////////////////////////////////////////////////
//  Fast User Switching under WinXP
LRESULT CCookieCopDlg::OnSessionChange( WPARAM wParam, LPARAM lParam)	
{
    UNREFERENCED_PARAMETER( lParam );

	switch( wParam )
	{
		case WTS_CONSOLE_CONNECT:
        m_theApp->ReadDataFiles( ); //load changes
		m_theApp->m_Listen.StartListen( );
		break;
	
		case WTS_CONSOLE_DISCONNECT:
        m_theApp->m_pDlg->SaveChanges( );//save changes if necessary
		m_theApp->m_Listen.StopListen( );
		break;
	}
	return 0;
}
