/*=========================================================================*/
/*                           CookieHlpr.cpp                                */
/*                                                                         */
/*                                                                         */
/*                   CookieCop 1.2 Copyright (c) 2000                      */
/*                         Ziff Davis, Inc.							       */
/*                       All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/

// Note: Proxy/Stub Information
//		To build a separate proxy/stub DLL, 
//		run nmake -f CookieHlprps.mk in the project directory.

#include "stdafx.h"
#include "resource.h"
#include "initguid.h"
#include "CookieHlpr.h"
#include "..\config.h"
#include "..\registry.h"
#include "CookieHlpr_i.c"
#include "CookieHlprObj.h"


//Shared data section for this dll
#pragma data_seg(".Shared")
int nInstanceCount = 0;
#pragma data_seg()
#pragma comment(linker, "/SECTION:.Shared,RWS")


static char SZ_IE_REG_SETTINGS[]="Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings";

BOOL RestoreIE( );

CComModule _Module;
HINSTANCE _hInstance;

BEGIN_OBJECT_MAP(ObjectMap)
	OBJECT_ENTRY(CLSID_CCookieHlprObj, CCookieHlprObj)
END_OBJECT_MAP()

/////////////////////////////////////////////////////////////////////////////
//
extern "C"
HWND FindCookieCopHwnd( )
{
	#define SZ_WINDOWNAME "CookieCopWindow"

	HWND hwnd = FindWindow( NULL, "CookieCop" );  
	if( hwnd == 0 )
	{
		hwnd = FindWindow( NULL, SZ_WINDOWNAME );
		if( hwnd )
			hwnd = GetWindow( hwnd, GW_CHILD );	//1.2
	}

	return( hwnd );


}


/////////////////////////////////////////////////////////////////////////////
// DLL Entry Point
//
// This is a browser helper object that is notified of IE and Explorer events. 
// If IE is loading us, search for the CookieCop window.  If not found, start
// up CookieCop.exe.   If Explorer is loading us, then just return FALSE to
// cancel the load.
//
//
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /*lpReserved*/)
{

	_hInstance = hInstance;
	
	if (dwReason == DLL_PROCESS_ATTACH)
	{
		TCHAR pszLoader[MAX_PATH];
		GetModuleFileName(NULL, pszLoader, MAX_PATH);
		_tcslwr(pszLoader);

		if (_tcsstr(pszLoader, _T("explorer.exe"))) 
			return FALSE;
		 

		if (_tcsstr(pszLoader, _T("iexplore.exe"))) 		
		{
			if( FindCookieCopHwnd( ) == 0  )
			{
				GetModuleFileName(hInstance, pszLoader, MAX_PATH);
				LPSTR p = strrchr( pszLoader, '\\' );

				if( p )
				{
					*p = 0;
					strcat( p, "\\cookiecop.exe" );
					WinExec( pszLoader ,SW_SHOW );
					
				}
			}
			nInstanceCount++;
		}
		
		_Module.Init(ObjectMap, hInstance);
		DisableThreadLibraryCalls(hInstance);
	}
	
	if (dwReason == DLL_PROCESS_DETACH)
	{

		TCHAR pszLoader[MAX_PATH];
		GetModuleFileName(NULL, pszLoader, MAX_PATH);
		_tcslwr(pszLoader);
		if (_tcsstr(pszLoader, _T("iexplore.exe"))) 		
		{
			nInstanceCount--;

			//DebugOut( "DLL_PROCESS_DETACH count = %u", nInstanceCount );

			if( nInstanceCount <1 )
			{	
				if( HWND hwnd = FindCookieCopHwnd( ) )
				{
					PostMessage( hwnd,WM_SYSCOMMAND, SC_CLOSE,  0 );
				}
			}

		}

		_Module.Term();
	}
	
	return TRUE;    // ok
}

/////////////////////////////////////////////////////////////////////////////
// Used to determine whether the DLL can be unloaded by OLE

STDAPI DllCanUnloadNow(void)
{
	//DebugOut( "DllCanUnloadNow count=%u", _Module.GetLockCount() );

	return (_Module.GetLockCount()==0) ? S_OK : S_FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// Returns a class factory to create an object of the requested type

STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID* ppv)
{
	return _Module.GetClassObject(rclsid, riid, ppv);
}

/////////////////////////////////////////////////////////////////////////////
// DllRegisterServer - Adds entries to the system registry

STDAPI DllRegisterServer(void)
{
	// registers object, typelib and all interfaces in typelib
	return _Module.RegisterServer(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// DllUnregisterServer - Removes entries from the system registry

STDAPI DllUnregisterServer(void)
{
	_Module.UnregisterServer();
	return S_OK;
}


//=====================================================================
//  FUNCTION:		UninstInitialize
//
//  PURPOSE:		checks to see if the desktop is locked 
//					
//  PARAMETERS:     
//
//  RETURN VALUE:	-1 to terminate the uninstall
//
//  COMMENTS:		called by InstallShield
//=====================================================================
STDAPI UninstInitialize( HWND hDlg, HINSTANCE hInst,LONG lReserved )
{
	char szWarning[] = { 
	"\nIMPORTANT: You will need to remove CookieCop's proxy settings\n"
	"from your browser's configuration after CookieCop has been uninstalled.\n"
	"You will not be able to access the Internet with your browser until it\n" 
	"has been properly reconfigured.\n\n"
	"If you are not sure how to reconfigure your browser, press the Cancel\n"
	"button now to stop the uninstall and review CookieCop's online help section\n"
	"on 'Configuring your Browser'.\n\n\n"
	"Press the OK button to continue the uninstall or Cancel to exit.\n\n"
	};

	if( MessageBox( hDlg, szWarning, "CookieCop 2", MB_OKCANCEL |MB_ICONWARNING | MB_SETFOREGROUND ) != IDOK )
		return -1;	//cancel
	
	//Restore IE configuration
	RestoreIE( );

	#define SZ_KEY "Software\\Microsoft\\Windows\\CurrentVersion\\Run"
	#define SZ_APPNAME "CookieCop"
	
	CRegistry key;
	if( key.Open( HKEY_LOCAL_MACHINE, SZ_KEY ) == ERROR_SUCCESS )
			key.DeleteValue( SZ_APPNAME );

	//Remove BHO
	DllUnregisterServer( );

	return 0;
}

//=====================================================================
//  FUNCTION:		UninstUnInitialize
//
//  PURPOSE:		
//					
//  PARAMETERS:     
//
//  RETURN VALUE:	
//
//  COMMENTS:		called by InstallShield
//=====================================================================
int UninstUnInitialize( HWND hDlg, HINSTANCE hInst,LONG lReserved )
{
	return 0;
}


void QualifyPathName( LPSTR pFile, LPCSTR pIni )
{
char szName[MAX_PATH];
LPSTR p;

	//Qualify the INI file to the same location as our dll
	GetModuleFileName( _hInstance, szName, sizeof( szName ) );
	p = strrchr( szName, '\\' );

	if( p )
		*(++p) = 0;
	else 
		p = szName;

	strcat( p, pIni );
	
	strcpy( pFile, szName );
	
	
}


BOOL ConfigSaved_IE( )
{
	char szFileName[MAX_PATH];
	char szBuf[30];

	QualifyPathName( szFileName, SZ_BROWSER_INI );
	return( GetPrivateProfileString( SZ_CONFIG_IE, SZ_SAVED, "", szBuf,sizeof(szBuf), szFileName ) );
}

BOOL ConfigRestored_IE( )
{
	char szFileName[MAX_PATH];
	char szBuf[30];

	QualifyPathName( szFileName, SZ_BROWSER_INI );
	return( GetPrivateProfileString( SZ_CONFIG_IE, SZ_RESTORED, "", szBuf,sizeof(szBuf), szFileName ) );
}

//called from CookieCop and during the uninstall
BOOL RestoreIE( )
{	
	
	CRegistry reg;
	char szBuf[MAX_PATH];
	BOOL bReturn = FALSE;

	DWORD dwResult = MAX_PATH;
	char szFileName[MAX_PATH];

	QualifyPathName( szFileName, SZ_BROWSER_INI  );

	if( ConfigRestored_IE( ) )
		return TRUE;

	if( ConfigSaved_IE( ) )
	{
		if( reg.Open( HKEY_CURRENT_USER, SZ_IE_REG_SETTINGS, KEY_ALL_ACCESS ) == ERROR_SUCCESS )
		{
			GetPrivateProfileString( SZ_CONFIG_IE, SZ_PROXY, "", szBuf, sizeof(szBuf), szFileName );
			reg.SetValue(szBuf, "ProxyServer" );

			int nLen = GetPrivateProfileString( SZ_CONFIG_IE, SZ_PROXY_ENABLE, "", szBuf, sizeof(szBuf), szFileName );
			HGLOBAL hBuf = GlobalAlloc( GHND, nLen + 1 );
			LPBYTE  pBuf = (LPBYTE)GlobalLock( hBuf );
			
			LPSTR p = (LPSTR)szBuf;
			int index = 0;

			char *stop;
			while( p && *p)
			{
				
				BYTE ch = (BYTE)strtol( p, &stop, 16 );
				pBuf[index++] = ch;
				p = stop;
				if( p && *p)
					p++;
			}
			
			//DWORD dwValue = atol( szBuf );
			reg.SetValueBinary(pBuf, index, "ProxyEnable" );
			GlobalUnlock( hBuf );
			GlobalFree( hBuf );

			WritePrivateProfileString( SZ_CONFIG_IE, SZ_RESTORED, "1", szFileName );
			bReturn = TRUE;
		}

	} else {
		bReturn = FALSE;
	}
	return( bReturn );
}


void _cdecl DebugOut_CompiledOutOfReleaseBuild( TCHAR *format,... )
{
	#ifdef _DEBUG
	
    TCHAR szBuf[8096];
    TCHAR sz[8096];
	va_list argptr;

	va_start( argptr, format );
	wvsprintf( szBuf, format, argptr );
	va_end( argptr);
	wsprintf(sz, _T("CookieHlpr -- %s\n"), szBuf  );
	OutputDebugString( sz );
	#endif
}






