/*=========================================================================*/
/*                        CookieHlprObj.cpp                                */
/*                                                                         */
/*                                                                         */
/*                   CookieCop 1.2 Copyright (c) 2000                      */
/*                         Ziff Davis, Inc.							       */
/*                       All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include "stdafx.h"
#include "CookieHlpr.h"
#include "CookieHlprObj.h"
#include "ExDispID.h"
#include <strstrea.h>


const char* const pszAppName = "CookieHlpr";

/////////////////////////////////////////////////////////////////////////////
// CCookieHlprObj

//
// CCookieHlprObj Methods
//

CCookieHlprObj::~CCookieHlprObj()
{
}

BOOL CCookieHlprObj::ManageConnection(enum ConnectType eConnectType)
{
   //_ASSERT(m_spWebBrowser2);

   if (!m_spWebBrowser2)
      return S_OK;


   HRESULT hr = 0;
	
   //
   // If eConnectType is Advise then we are advising IE that we
   // want to handle events.  If eConnectType is Unadvise, we are
   // telling IE that we no longer want to handle events.
   //
   CComQIPtr<IConnectionPointContainer,
             &IID_IConnectionPointContainer> spCPContainer(m_spWebBrowser2);

   if (spCPContainer != NULL)
   {
      CComPtr<IConnectionPoint> spConnectionPoint;

      hr = spCPContainer->FindConnectionPoint(DIID_DWebBrowserEvents2, &spConnectionPoint);
      if (SUCCEEDED(hr))
      {
         if (eConnectType == Advise)
         {
            //
            // Advise the client site of our desire to be handle events
            //
            hr = spConnectionPoint->Advise((IDispatch*)this, &m_dwCookie);
            if (FAILED(hr))
               ATLTRACE("\n%s: ManageConnection(): Failed to Advise\n\n", pszAppName);
         }
         else
         {
            // Remove us from the list of people interested...
            hr = spConnectionPoint->Unadvise(m_dwCookie);
            if (FAILED(hr))
               ATLTRACE("\npszAppName: ManageConnection(): Failed to Unadvise\n\n", pszAppName);
         }
      }
   }

   return (SUCCEEDED(hr));
}

//
// IOleObjectWithSite Methods
//
STDMETHODIMP CCookieHlprObj::SetSite(IUnknown *pUnkSite)
{
   USES_CONVERSION;

   if (!pUnkSite)
      ATLTRACE("\nSetSite(): pUnkSite is NULL\n\n");
   else
   {
      // Query pUnkSite for the IWebBrowser2 interface.
      m_spWebBrowser2 = pUnkSite;

      if (m_spWebBrowser2)
      {
      
         // Connect to the browser in order to handle events.
         if (!ManageConnection(Advise))
         {
            ATLTRACE("\n%s: %s\n\n", 
                     "SetSite(): Failure connecting to Browser",
                     pszAppName);
         }
      }
   }
   return S_OK;
}


//
// IDispatch Methods
//
STDMETHODIMP CCookieHlprObj::Invoke(DISPID dispidMember, REFIID riid, LCID lcid, WORD wFlags,
                                DISPPARAMS* pDispParams, VARIANT* pvarResult,
                                EXCEPINFO*  pExcepInfo,  UINT* puArgErr)
{
  // USES_CONVERSION;

   if (!pDispParams)
      return E_INVALIDARG;

   switch (dispidMember)
   {
	  case DISPID_ONQUIT:
      case DISPID_QUIT:
         ManageConnection(Unadvise);
         break;
   }

   return S_OK;
}

