/*=========================================================================*/
/*                         CookieListDlg.cpp                               */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include "stdafx.h"
#include "cookiecop.h"
#include "CookieListDlg.h"
#include "EditUrlDlg.h"
#include "DlgPropSheet.h"
#include "WarningDlgs.h"
#include "DeleteCookies.h"	
#include "hlp/helpids.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//cookie options
enum {REJECT, ACCEPT};

/////////////////////////////////////////////////////////////////////////////
// CCookieListDlg dialog

CCookieListDlg::CCookieListDlg() : CPropertyPage(CCookieListDlg::IDD)
{
	//{{AFX_DATA_INIT(CCookieListDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_bDirty = FALSE;
}

void CCookieListDlg::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCookieListDlg)
	DDX_Control(pDX, IDC_COOKIE_LIST, m_CookieListCtrl);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CCookieListDlg, CPropertyPage)
	//{{AFX_MSG_MAP(CCookieListDlg)
	ON_NOTIFY(NM_DBLCLK, IDC_COOKIE_LIST, OnDblclkCookieList)
	ON_BN_CLICKED(IDC_DELETE, OnDelete)
	ON_BN_CLICKED(IDC_NEW, OnNew)
	ON_BN_CLICKED(IDC_EDIT, OnEdit)
	ON_BN_CLICKED(IDC_SEARCH_JAVASCRIPT, OnSearchJavascript)
	ON_BN_CLICKED(IDC_REJECT, OnReject)
	ON_BN_CLICKED(IDC_ACCEPT, OnAccept)
	ON_MESSAGE( MSG_DRAGDROP, OnDrop)
	ON_NOTIFY(NM_SETFOCUS, IDC_COOKIE_LIST, OnSetfocusCookieList)
	ON_BN_CLICKED(IDC_DISABLECOOKIELIST, OnDisablecookielist)
	ON_BN_CLICKED(IDC_SCAN_FILES, OnScanFiles)
	ON_NOTIFY(NM_CLICK, IDC_COOKIE_LIST, OnClickCookieList)
	ON_BN_CLICKED(IDC_SESSION_ONLY, OnSessionOnly)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_SCAN_ALL, OnScanAll)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCookieListDlg message handlers

BOOL CCookieListDlg::PreTranslateMessage(MSG* pMsg) 
{
	m_ToolTip.RelayEvent( pMsg );	
	return CPropertyPage::PreTranslateMessage(pMsg);
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::PopulateCookieList() 
{
	//populate
	m_CookieListCtrl.DeleteAllItems( );
	
	int i,iCount;
	LV_ITEM lv;
	CString s;
	memset( &lv, 0, sizeof( lv ) );

	iCount = m_theApp->m_RejectList.GetSize( );
	for( i = 0; i < iCount; i++ )
	{
		int index = m_CookieListCtrl.GetItemCount( );
		lv.mask =  LVIF_IMAGE | LVIF_TEXT;
		lv.iItem = index;
		s = m_theApp->m_RejectList.GetAt( i );
		lv.pszText = (LPSTR)(LPCSTR)s;
		lv.iImage = REJECT;
		index = m_CookieListCtrl.InsertItem( &lv );
	}
	

	iCount = m_theApp->m_AcceptList.GetSize( );
	for( i = 0; i < iCount; i++ )
	{
		int index = m_CookieListCtrl.GetItemCount( );
		lv.mask = LVIF_IMAGE | LVIF_TEXT;
		lv.iItem = index;
		s = m_theApp->m_AcceptList.GetAt( i );
		lv.pszText = (LPSTR)(LPCSTR)s;
		lv.iImage = ACCEPT;
		m_CookieListCtrl.InsertItem( &lv );
	}

	//always force a selection
	if( m_CookieListCtrl.GetNextItem(-1, LVNI_SELECTED) == -1 )
		HighlightItem( 0 );

	CheckDlgButton( IDC_DISABLECOOKIELIST, m_theApp->m_bDisableCookieList );
	CheckDlgButton( IDC_SEARCH_JAVASCRIPT, m_theApp->m_bSearchJavaScript );
	WORD nID;

	switch( m_theApp->m_bDefaultAction )
	{
		case REJECT_COOKIE:
			nID = IDC_REJECT;
			break;
		case SESSION_COOKIE:
			nID = IDC_SESSION_ONLY;
			break;
		default:
			nID = IDC_ACCEPT;		
	}

	
	CheckRadioButton( IDC_ACCEPT, IDC_SESSION_ONLY, nID );

	DoModified( FALSE );
}

/////////////////////////////////////////////////////////////////////////////
// 
BOOL CCookieListDlg::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	m_ToolTip.Create( this );
	m_ToolTip.AddTool( GetDlgItem( IDC_COOKIE_LIST ), IDS_COOKIE_LIST );
	m_ToolTip.AddTool( GetDlgItem( IDC_DELETE ), IDS_DELETE );
	m_ToolTip.AddTool( GetDlgItem( IDC_NEW ), IDS_NEW );
	m_ToolTip.AddTool( GetDlgItem( IDC_EDIT ),IDS_EDIT );
	m_ToolTip.AddTool( GetDlgItem( IDC_ACCEPT ),IDS_ACCEPT_COOKIE );
	m_ToolTip.AddTool( GetDlgItem( IDC_REJECT ),IDS_REJECT_COOKIE );
	m_ToolTip.AddTool( GetDlgItem( IDC_REJECT ),IDS_SESSION_COOKIE );
	m_ToolTip.AddTool( GetDlgItem( IDC_SEARCH_JAVASCRIPT ),IDS_SEARCH_JAVASCRIPT );
	m_ToolTip.AddTool( GetDlgItem( IDC_SCAN_FILES ),IDS_SCAN_FILES );
	m_ToolTip.AddTool( GetDlgItem( IDC_DISABLECOOKIELIST ),IDS_DISABLECOOKIELIST );
	
	COLORREF crBackground = RGB(192,192,192);	//v1.2
	CSize size;
	BITMAP bm;
	CBitmap bmp;
	bmp.LoadBitmap( IDB_ACCEPT_COOKIES );  //order is important see enum{ACCEPT,REJECT}
	bmp.GetBitmap( &bm );	
	m_ImageList.Create(	IDB_REJECT_COOKIES, bm.bmWidth, 1, crBackground );
	m_ImageList.Add( &bmp, crBackground );

	m_CookieListCtrl.SetImageList( &m_ImageList, LVSIL_SMALL );

	//v1.2
	HANDLE hBitmap = LoadImage( AfxGetInstanceHandle( ), MAKEINTRESOURCE( IDB_JAVA ),IMAGE_BITMAP,0,0,LR_LOADMAP3DCOLORS );
	hBitmap = (HBITMAP)GetDlgItem( IDC_JAVA )->SendMessage( STM_SETIMAGE, IMAGE_BITMAP, (LPARAM)(HBITMAP)hBitmap );
	if( hBitmap )
		::DeleteObject( hBitmap );

	//force report style
	HWND hwndLV = m_CookieListCtrl.m_hWnd;
    DWORD dwStyle = GetWindowLong(hwndLV, GWL_STYLE);  
    SetWindowLong( hwndLV, GWL_STYLE, dwStyle | LVS_REPORT ); 
	
	CRect rc;
	m_CookieListCtrl.GetClientRect( &rc );            
	
	LV_COLUMN lv;
	lv.mask = LVCF_TEXT;
	lv.cx = rc.Width( );
	lv.pszText = "URL";
	m_CookieListCtrl.InsertColumn( 0, &lv );
	m_CookieListCtrl.SetColumnWidth( 0, rc.Width( ) );

	//register the listctrl as a drop target -- destructor calls Revoke()
	m_DropTarget.Register( GetDlgItem( IDC_COOKIE_LIST ) );	
	
	PopulateCookieList();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

/////////////////////////////////////////////////////////////////////////////
// 
int CCookieListDlg::FindString( LPCSTR pItem )
{
	LVFINDINFO lv;

	lv.flags = LVFI_STRING;
	lv.psz = pItem;
	return( m_CookieListCtrl.FindItem( &lv, -1 ) );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::HighlightItem( int iIndex )
{
	LVITEM lv;
	memset( &lv, 0, sizeof( lv ) );

	lv.mask = LVIF_STATE;
	lv.iItem = iIndex;
	lv.stateMask = LVIS_SELECTED | LVIS_FOCUSED; 
	lv.state = LVIS_SELECTED | LVIS_FOCUSED; 
	m_CookieListCtrl.EnsureVisible( iIndex, FALSE );
	m_CookieListCtrl.SetItem( &lv );

}

/////////////////////////////////////////////////////////////////////////////
//if index is -1 then we are adding a new item
void CCookieListDlg::EditItem( int iIndex, CString* pItem /* = NULL */ )
{
	CString sUrl;
	
	if( pItem )
		sUrl = *pItem;

	BOOL bAccept = (BOOL)REJECT;
	if( iIndex >= 0 )
	{
		sUrl = m_CookieListCtrl.GetItemText( iIndex, 0 );
		LV_ITEM lv;
		memset( &lv, 0, sizeof( lv ) );
		lv.iItem = iIndex;
		lv.mask = LVIF_IMAGE ;
		m_CookieListCtrl.GetItem( &lv );
		bAccept = (lv.iImage == ACCEPT);
	}


	CEditUrlDlg dlg;
	dlg.m_sUrl = sUrl;
	dlg.m_ListType = COOKIE_LIST;
	dlg.m_bAcceptCookie = bAccept;
	
	if( dlg.DoModal( ) == IDOK )
	{
		if( iIndex >= 0 ) {
			CString s = m_CookieListCtrl.GetItemText( iIndex, 0 );
			if( s.Compare( dlg.m_sUrl ) == 0 && dlg.m_bAcceptCookie == bAccept )
				return;
			m_CookieListCtrl.DeleteItem( iIndex );
		}

		//check for duplicate entries		
		int i = FindString( dlg.m_sUrl );
		if( i >= 0 ) 
		{
			HighlightItem( i );
			ShowError( IDS_ENTRYEXISTS, MB_OK | MB_ICONEXCLAMATION );
		} else {
			InsertUrl( dlg.m_sUrl, dlg.m_bAcceptCookie ); 
		}
		UpdateCookieLists( );
	}
} 


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::InsertUrl( LPCSTR pUrl, BOOL bAcceptFlag )
{
	LV_ITEM lv;
	memset( &lv, 0, sizeof( lv ) );
		
	int iIndex = m_CookieListCtrl.GetItemCount( );
	lv.mask =  LVIF_IMAGE | LVIF_TEXT;
	lv.iItem = iIndex;
	lv.pszText = (LPSTR)pUrl;
	
	if( bAcceptFlag )
	{
		m_theApp->m_AcceptList.Add( pUrl ); 
		lv.iImage = ACCEPT;
	} else {
		m_theApp->m_RejectList.Add( pUrl ); 
		lv.iImage = REJECT;
	}

	iIndex = m_CookieListCtrl.InsertItem( &lv );

	//delete files from the cookies directory
	if( !bAcceptFlag )
	{
		CDeleteCookies *pDC = new CDeleteCookies;
		pDC->m_sUrl = pUrl;
		//FormatUrl( &pDC->m_sUrl );	1.1 removed
		pDC->DoModal( );
		delete pDC;	//~1
	}

	HighlightItem( iIndex );
	m_CookieListCtrl.SetFocus( );
	DoModified( TRUE );
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CCookieListDlg::OnSetActive() 
{
	return CPropertyPage::OnSetActive();
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CCookieListDlg::OnKillActive() 
{
	return CPropertyPage::OnKillActive();
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnDblclkCookieList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UNREFERENCED_PARAMETER( pNMHDR );

	*pResult = 0;
	OnEdit( );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnEdit() 
{
	int nItem = m_CookieListCtrl.GetNextItem(-1, LVNI_SELECTED); 
	if( nItem >= 0 )
		EditItem( nItem );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnDelete() 
{
	int nItem = m_CookieListCtrl.GetNextItem(-1, LVNI_SELECTED); 
	if( nItem >= 0 ) 
	{
		m_CookieListCtrl.DeleteItem( nItem );
		while( nItem >= m_CookieListCtrl.GetItemCount( ) )
			nItem--;
		HighlightItem( nItem );
		UpdateCookieLists( );
		DoModified( TRUE );
	}
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnNew() 
{
	EditItem( -1 );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnReset() 
{
	m_theApp->ReadDataFiles();
	PopulateCookieList(); 
	SetModified( FALSE );	
	CPropertyPage::OnReset();
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::UpdateCookieLists( )
{
	char sz[512];
	LV_ITEM lv;
	
	m_theApp->m_RejectList.RemoveAll( );
	m_theApp->m_AcceptList.RemoveAll( );

	int iCount = m_CookieListCtrl.GetItemCount( );
	for( int i = 0; i < iCount; i++ )
	{
		memset( &lv, 0, sizeof( lv ) );
		lv.mask =  LVIF_IMAGE | LVIF_TEXT;
		lv.iItem = i;
		lv.cchTextMax = sizeof( sz );
		lv.pszText = sz;

		m_CookieListCtrl.GetItem( &lv );
		if( lv.iImage == ACCEPT )
			m_theApp->m_AcceptList.Add( sz );
		else
			m_theApp->m_RejectList.Add( sz );
	}
}

/////////////////////////////////////////////////////////////////////////////
// 
BOOL CCookieListDlg::OnApply() 
{
	UpdateCookieLists( );
	WriteRejectFile( );	
	WriteAcceptFile( );
	WriteCookieCopFile( );
	DoModified( FALSE );
	return CPropertyPage::OnApply();
}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnSearchJavascript() 
{
	m_theApp->m_bSearchJavaScript = IsDlgButtonChecked( IDC_SEARCH_JAVASCRIPT );
	DoModified( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnSessionOnly() 
{
	UpdateData( FALSE );
	if( ShowWarning( IDS_SESSION_ONLY_COOKIES, IDYES ) != IDYES )
	{	
		m_theApp->m_bDefaultAction = ACCEPT_COOKIE;
		CheckRadioButton( IDC_ACCEPT, IDC_SESSION_ONLY, IDC_ACCEPT );
	} else {
		m_theApp->m_bDefaultAction = SESSION_COOKIE;
		SetModified( TRUE );
	}
	
	
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnReject() 
{
	UpdateData( FALSE );
	if( ShowWarning( IDS_REJECT_ALL_COOKIES, IDYES ) != IDYES )
	{	
		m_theApp->m_bDefaultAction = ACCEPT_COOKIE;
		CheckRadioButton( IDC_ACCEPT, IDC_SESSION_ONLY, IDC_ACCEPT );
	} else {
		m_theApp->m_bDefaultAction = REJECT_COOKIE;
		SetModified( TRUE );
	}
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnAccept() 
{
	m_theApp->m_bDefaultAction = ACCEPT_COOKIE;
	SetModified( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::DoModified( BOOL bDirty )
{
	m_bDirty = bDirty;
	SetModified( bDirty );
	EnableDisableControls( );
}

/////////////////////////////////////////////////////////////////////////////
// 
LRESULT CCookieListDlg::OnDrop(WPARAM wParam, LPARAM lParam )
{
	UNREFERENCED_PARAMETER( wParam );

	if( !m_theApp->m_bDisableCookieList )
	{
		m_theApp->m_pDlg->SetForegroundWindow( );	

		CString s = (LPSTR)lParam;
		if( ExtractUrl( &s, FALSE ) )
		{
			FormatUrl( &s );	//1.1 change host to wildcard
			EditItem( -1, &s );
		} else {
			ShowError( IDS_BADURL, MB_OK | MB_ICONHAND );
		}
	}
	return 0;
}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnSetfocusCookieList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UNREFERENCED_PARAMETER( pNMHDR );

	DoModified( m_bDirty );
	EnableDisableControls( );
	*pResult = 0;
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnDisablecookielist() 
{
	m_theApp->m_bDisableCookieList = IsDlgButtonChecked( IDC_DISABLECOOKIELIST );
	DoModified( TRUE );
	EnableDisableControls( );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::EnableDisableControls( )
{
	BOOL bEnable = m_theApp->m_bDisableCookieList == FALSE;
	BOOL bEnableSelected = FALSE;
	BOOL bEnableUpdated = FALSE;
	
	if( bEnable )
	{
		bEnableSelected = ( m_CookieListCtrl.GetNextItem(-1, LVNI_SELECTED) != -1 );
		bEnableUpdated = m_bDirty;
	}
	
	m_CookieListCtrl.EnableWindow( bEnable );
    GetDlgItem( IDC_SCAN_ALL)->EnableWindow( bEnable );

	GetDlgItem( IDC_NEW )->EnableWindow( bEnable );

	//if nothing is selected disable appropriate buttons
	GetDlgItem( IDC_DELETE )->EnableWindow( bEnableSelected );
	GetDlgItem( IDC_EDIT )->EnableWindow( bEnableSelected );

	//1.1
	if( bEnableSelected == FALSE )
		GetDlgItem( IDC_SCAN_FILES )->EnableWindow( bEnableSelected );
	
	///check to see if the item is to reject cookies
	if( bEnableSelected )
	{
		int nItem = m_CookieListCtrl.GetNextItem(-1, LVNI_SELECTED ); 
		if( nItem >= 0 )
		{
			LV_ITEM lv;
			memset( &lv, 0, sizeof( lv ) );
			lv.iItem = nItem;
			lv.mask = LVIF_IMAGE ;
			m_CookieListCtrl.GetItem( &lv );
			bEnableSelected  = (lv.iImage == REJECT);
		}
		if( m_theApp->m_bScanFolder_IE == FALSE && m_theApp->m_bScanFolder_Nav == FALSE )
			bEnableSelected = FALSE;

		GetDlgItem( IDC_SCAN_FILES )->EnableWindow( bEnableSelected );
	}		

}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnScanAll() 
{
	CDeleteCookies *pDC = new CDeleteCookies;
	pDC->m_sUrl.Empty( );
	pDC->DoModal( );
	delete pDC;	
	
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnScanFiles() 
{
	int nItem = m_CookieListCtrl.GetNextItem(-1, LVNI_SELECTED ); 
	if( nItem >= 0 )
	{
		CString sUrl = m_CookieListCtrl.GetItemText( nItem, 0 );
		CDeleteCookies *pDC = new CDeleteCookies;
		pDC->m_sUrl = sUrl;
		pDC->DoModal( );
		delete pDC;	
	}
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnClickCookieList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UNREFERENCED_PARAMETER( pNMHDR );

	EnableDisableControls( );
	*pResult = 0;
}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieListDlg::OnDestroy() 
{

	HBITMAP hBitmap = (HBITMAP)GetDlgItem( IDC_JAVA )->SendMessage( STM_GETIMAGE, IMAGE_BITMAP, 0 );
	if( hBitmap )
		::DeleteObject( hBitmap );

	CPropertyPage::OnDestroy();

}

