/*=========================================================================*/
/*                        CookieRecvDlg.cpp                                */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include "stdafx.h"
#include "cookiecop.h"
#include "Globals.h"
#include "CookieRecvDlg.h"
#include "CookieListDlg.h"
#include "hlp/helpids.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCookieRecvDlg dialog


CCookieRecvDlg::CCookieRecvDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CCookieRecvDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CCookieRecvDlg)
	m_sTime = _T("");
	m_sURL = _T("");
	m_sContents = _T("");
	//}}AFX_DATA_INIT
}


void CCookieRecvDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCookieRecvDlg)
	DDX_Control(pDX, IDC_COOKIE_PATTERN, m_PatternEdit);
	DDX_Control(pDX, IDC_REJECT, m_btnReject);
	DDX_Control(pDX, IDC_ACCEPT, m_btnAccept);
	DDX_Text(pDX, IDD_TIME, m_sTime);
	DDX_Text(pDX, IDD_URL, m_sURL);
	DDX_Text(pDX, IDD_CONTENTS, m_sContents);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CCookieRecvDlg, CDialog)
	//{{AFX_MSG_MAP(CCookieRecvDlg)
	ON_BN_CLICKED(IDC_ACCEPT, OnAccept)
	ON_BN_CLICKED(IDC_REJECT, OnReject)
	ON_BN_CLICKED(IDC_SAVE, OnSave)
	ON_WM_CTLCOLOR()
	ON_BN_CLICKED(IDD_COOKIE_ACCEPT_TEXT, OnCookieAcceptText)
	ON_BN_CLICKED(IDC_REJECT_TEXT, OnRejectText)
	ON_BN_CLICKED(IDC_NEXT, OnNext)
	ON_BN_CLICKED(IDC_PREV, OnPrev)
	ON_BN_CLICKED(IDHELP, OnHelp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCookieRecvDlg message handlers

BOOL CCookieRecvDlg::OnInitDialog() 
{
	//save the initial event
	m_eventtype = m_pEventData->event;

	CDialog::OnInitDialog();

	//v1.2 -- corrected background colors
	HANDLE hBitmap = LoadImage( AfxGetInstanceHandle( ), MAKEINTRESOURCE( IDB_ACCEPT_COOKIES ),IMAGE_BITMAP,0,0,LR_LOADTRANSPARENT|LR_LOADMAP3DCOLORS);
	hBitmap = (HBITMAP)m_btnAccept.SendMessage( BM_SETIMAGE, IMAGE_BITMAP, (LPARAM)(HBITMAP)hBitmap );  
	if( hBitmap )
		::DeleteObject( hBitmap );

 	hBitmap = LoadImage( AfxGetInstanceHandle( ), MAKEINTRESOURCE( IDB_REJECT_COOKIES ),IMAGE_BITMAP,0,0,LR_LOADTRANSPARENT|LR_LOADMAP3DCOLORS);
	hBitmap = (HBITMAP)m_btnReject.SendMessage( BM_SETIMAGE, IMAGE_BITMAP, (LPARAM)(HBITMAP)hBitmap );  
	if( hBitmap )
		::DeleteObject( hBitmap );

	hBitmap = LoadImage( AfxGetInstanceHandle( ), MAKEINTRESOURCE( IDB_SMALL_COP ),IMAGE_BITMAP,0,0,LR_LOADTRANSPARENT|LR_LOADMAP3DCOLORS);
	hBitmap = (HBITMAP)GetDlgItem( IDC_COP )->SendMessage( STM_SETIMAGE, IMAGE_BITMAP, (LPARAM)(HBITMAP)hBitmap );  
	if( hBitmap )
		::DeleteObject( hBitmap );

	CString s;
	switch( m_pEventData->event )
	{
		case EVENT_COOKIE_JS:
			s = "A Javascript cookie has been blocked.";
			break;
		case EVENT_COOKIE_REJECTED: 
			s = "An attempt to download a cookie has been blocked.";
			break;												 
		case EVENT_SESSION_COOKIE:
			s = "A cookie has been converted into session-only.";
			break;
		case EVENT_COOKIE_ACCEPTED: 
			s = "A cookie has been downloaded to your computer.";
			break;
		case EVENT_COOKIE_SENT:
			s = "A cookie has been sent from your computer to the host.";
			break;
		case EVENT_COOKIE_BLOCKED:
			s = "An attempt upload a cookie has been blocked.";
			break;
		case EVENT_COOKIE_JS_ACCEPTED:
			s = "A JavaScript cookie may have been downloaded to your PC.";
			break;
        case EVENT_REFERER_REMOVED:
            s = "The referer field has been removed.";
            break;
        case EVENT_REFERRER_REMOVED_JS:
            s = "JavaScript referrer fields has been removed.";
            break;
		default:
			s = "Unknown event";	//error -- should not happen
	}

	SetDlgItemText( IDC_EVENT, s );

	UpdateData( TRUE );	
	UpdateControls( );
	
	GetDlgItem( IDCANCEL )->SetFocus( );
	return FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieRecvDlg::UpdateControls( )
{
	int index = (int)m_pListCtrl->GetItemData( m_nItem );
	
	ASSERT (index <= m_theApp->m_EventList.GetUpperBound() );
	m_EventData = m_theApp->m_EventList.GetAt( index );

	m_sURL			= m_EventData.sUrl;
	m_sContents		= m_EventData.sCommand;
	m_sTime.Format( "%s  entry %u", m_EventData.sTime, m_nItem + 1 );

	
	//sPattern is the entry in the list that rejected or accepted the cookie
	//such as *.doubleclick.com.   If sPattern is empty then no action was 
	//taken to block or accept the cookie.  In this case format the URL 
	//with the host as a wildcard and insert it into the Web Site entry in the dialog.
	if( m_EventData.sPattern.IsEmpty( ) )
	{
		//replace host portion with wildcard 1.1
		CString sTmp = m_EventData.sUrl;
		FormatUrl( &sTmp );
		m_EventData.sPattern = sTmp;
	}

	SetDlgItemText( IDC_COOKIE_PATTERN, m_EventData.sPattern );
	
	UpdateData( FALSE );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieRecvDlg::OnCookieAcceptText() 
{
	OnAccept( );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieRecvDlg::OnRejectText() 
{
	OnReject( );	
}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieRecvDlg::OnAccept() 
{
	GetDlgItem(IDC_SAVE)->EnableWindow( TRUE );
	CheckDlgButton( IDC_REJECT,  0 );
	CheckDlgButton( IDC_ACCEPT,  1 );
	GetDlgItem( IDC_ACCEPT )->SetFocus( );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieRecvDlg::OnReject() 
{
	GetDlgItem(IDC_SAVE)->EnableWindow( TRUE );
	CheckDlgButton( IDC_ACCEPT,  0 );
	CheckDlgButton( IDC_REJECT,  1 );
	GetDlgItem( IDC_REJECT )->SetFocus( );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CCookieRecvDlg::OnSave() 
{
	GetDlgItemText( IDC_COOKIE_PATTERN, m_sURL );
	if( m_sURL.GetLength( ) )
	{
   		RemoveDups( &m_theApp->m_AcceptList, m_sURL );
		RemoveDups( &m_theApp->m_RejectList, m_sURL );
		m_theApp->m_pCookieListDlg->PopulateCookieList();
		m_theApp->m_pCookieListDlg->InsertUrl( m_sURL, IsDlgButtonChecked( IDC_ACCEPT ) );
	}
	EndDialog( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
HBRUSH CCookieRecvDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
	COLORREF crColor;
	
	if( nCtlColor == CTLCOLOR_STATIC )
	{
		int iID = pWnd->GetDlgCtrlID( );
		switch( iID )
		{
			case IDC_SITE          :
			case IDC_EVENT_TEXT	   :
			case IDC_TIME_TEXT     :
			case IDC_URL_TEXT	   :
			case IDC_CONTENTS_TEXT :
				 crColor = COLOR_DIALOG_EVENT;
				 break;
			default:
				return hbr;
		}
		pDC->SetTextColor( crColor );
	}
	return hbr;
}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieRecvDlg::RemoveDups( CStringArray* pList, LPCSTR pEntry)
{
	CString s;

	for( int i=0; i <= pList->GetUpperBound( ); i++ )
	{
		s = pList->GetAt( i );
		if( s.Compare( pEntry ) == 0 )
			pList->RemoveAt( i );
	}

}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieRecvDlg::OnNext() 
{
int nSave = m_nItem;

	if( m_pListCtrl->GetItemCount( ) == 0 )
		return;

	while( TRUE )
	{
		if( ( m_nItem + 1) >= m_pListCtrl->GetItemCount( ) )
		{
			if( ShowError( IDS_LASTITEM, MB_YESNO | MB_ICONQUESTION, this ) == IDYES )
			{
				m_nItem = -1;
			} else {
				m_nItem = nSave;
				return;	
			}
		}
		//check to see if the same event type
		int index = (int)m_pListCtrl->GetItemData( ++m_nItem );
		m_EventData = m_theApp->m_EventList.GetAt( index );
		if( m_EventData.event == m_eventtype )
			break;
	}	

	UpdateControls( );
}



/////////////////////////////////////////////////////////////////////////////
// 
void CCookieRecvDlg::OnPrev() 
{
int nSave = m_nItem;
	
	if( m_pListCtrl->GetItemCount( ) == 0 )
		return;

	while( TRUE )
	{
		if( (m_nItem-1)  < 0  )
		{
			if( ShowError( IDS_FIRSTITEM, MB_YESNO | MB_ICONQUESTION, this ) == IDYES )
			{
				m_nItem = m_pListCtrl->GetItemCount( );
			} else {
				m_nItem = nSave;
				return;	
			}	
		}
		//check to see if the same event type
		int index = (int)m_pListCtrl->GetItemData( --m_nItem );
		m_EventData = m_theApp->m_EventList.GetAt( index );
		if( m_EventData.event == m_eventtype )
			break;
	}	

	UpdateControls( );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CCookieRecvDlg::OnHelp() 
{
	m_theApp->CWinApp::WinHelp(IDH_n3_edit_cookie_list);
}
