/*=========================================================================*/
/*                         Globals.cpp                                     */
/*              Shared functions between all modules                       */
/*                                                                         */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include "stdafx.h"
#include "CookieCop.h"

#define DWORD_PTR DWORD*
#include <C:\Program Files\Microsoft Platform SDK\Include\wininet.h>

#include "Globals.h"
#include "Log.h"
#include <String.h>
#include "DlgPropSheet.h"
#include "Registry.h"

#pragma warning( disable : 4706 ) // assignment within conditional expression

#define MAX_BUF_LEN MAX_PATH*2 

CCookieCopApp* m_theApp = 0;

BOOL bErrorDisplayed_Ini = 0;
BOOL bErrorDisplayed_Data = 0;

/////////////////////////////////////////////////////////////////////////////
//
struct hostent* GetHostByAddr ( LPCSTR lpszIP ) 
{
      // translate dotted string format into integer
      int uPeer[4] ;
      sscanf ( lpszIP, "%d.%d.%d.%d",
             &uPeer[0], &uPeer[1], &uPeer[2], &uPeer[3] ) ;

      // move it into a char array for ::gethostbyaddr()
      char cPeer[4] ;
      cPeer[0] = (char)uPeer[0] ;
      cPeer[1] = (char)uPeer[1] ;
      cPeer[2] = (char)uPeer[2] ;
      cPeer[3] = (char)uPeer[3] ;
      return( ::gethostbyaddr ( cPeer, 4, PF_INET ) ) ;
}


/////////////////////////////////////////////////////////////////////////////
//
int ShowError( LPCSTR pErr )
{
	HWND hwnd = m_theApp->m_pDlg->GetSafeHwnd();

	if( IsWindowVisible( hwnd ) == FALSE )
		hwnd = GetDesktopWindow( );

	return( MessageBox( hwnd, pErr, SZ_APPNAME, MB_ICONHAND | MB_SETFOREGROUND | MB_APPLMODAL ) );
}

/////////////////////////////////////////////////////////////////////////////
//
int ShowError( int nID, int nMode, HWND hwnd )
{
	char szMsg[256];
	LoadString( AfxGetInstanceHandle( ), nID, szMsg, sizeof( szMsg ) );
	return( MessageBox( hwnd, szMsg, SZ_APPNAME, nMode | MB_SETFOREGROUND | MB_APPLMODAL ) );
}

/////////////////////////////////////////////////////////////////////////////
//
int ShowError( int nID, int nMode )
{
	HWND hwnd = m_theApp->m_pDlg->GetSafeHwnd();

	if( IsWindowVisible( hwnd ) == FALSE )
		hwnd = GetDesktopWindow( );

	return( ShowError( nID, nMode, hwnd ) );
}

/////////////////////////////////////////////////////////////////////////////
//
int ShowError( int nID, int nMode, CWnd* pWnd )
{
	HWND hwnd = pWnd->GetSafeHwnd( );
	return( ShowError( nID, nMode, hwnd ) );	
}

/////////////////////////////////////////////////////////////////////////////
//
void GetWinsockError( LPSTR pErrMsg )
{
	ASSERT( pErrMsg );

	if( pErrMsg )
	{
		strcpy( pErrMsg, "" );
		ShowWinsockError( 0,0,pErrMsg );
	}

}

/////////////////////////////////////////////////////////////////////////////
//
void ShowWinsockError( SOCKET s, LONG nSessionID, LPSTR pErrMsg )
{
	LPSTR p = 0;
	char szError[80];

	int error = WSAGetLastError( );
	if( error )
	{
		switch( error )
		{
			case WSAENETDOWN:		p = "The Windows Sockets implementation detected that the network subsystem failed.";break;
			case WSAEACCES:			p = "The requested address is a broadcast address, but the appropriate flag was not set.";break;
			case WSAEINPROGRESS:	p = "A blocking Windows Sockets operation is in progress.";break;
			case WSAEFAULT:			p = "The lpBuf argument is not in a valid part of the user address space."; break;
			case WSAENETRESET:		p = "The connection must be reset because the Windows Sockets implementation dropped it."; break;
			case WSAENOBUFS:		p = "The Windows Sockets implementation reports a buffer deadlock."; break;
			case WSAENOTCONN:		p = "The socket is not connected."; break;
			case WSAENOTSOCK:		p = "The descriptor is not a socket."; break;
			case WSAEOPNOTSUPP:		p = "MSG_OOB was specified, but the socket is not of type SOCK_STREAM."; break;
			case WSAESHUTDOWN:		p = "The socket has been shut down."; break;
			case WSAEWOULDBLOCK:	p = "The socket is marked as nonblocking and the requested operation would block."; break;
			case WSAEMSGSIZE:		p = "The socket is of type SOCK_DGRAM, and the datagram is larger than the maximum supported by the Windows Sockets implementation."; break;
			case WSAEINVAL:			p = "The socket has not been bound with Bind."; break;
			case WSAECONNABORTED:	p = "The virtual circuit was aborted due to timeout or other failure."; break;
			case WSAECONNRESET:		p = "Connection reset";break;	//this will happen when the user clicks the stop button
			default:
				{
					wsprintf( szError, "Winsock error = (%u)", error );
					p = szError;
				}
		}

		//optional, copy error message to a buffer
		if( p )
		{
			if( pErrMsg )
				strcpy( pErrMsg, p );
		}

		
		//let the browser display it's error
		#ifdef _DEBUG 
		if( p )	 
		{
			if( s != INVALID_SOCKET )
				DebugOut( "Socket %u Session = %u -- %s", s, nSessionID, p );
			else
				DebugOut( p );
		
		}
		#endif 
	}

}

/////////////////////////////////////////////////////////////////////////////
//formats a url for accept, reject, cookie lists 
//1.1 change host portion to wildcard
void FormatUrl( CString *pUrl )
{

	//first check for numeric url, if so leave it alone
	if( isalpha( pUrl->GetAt(0) ) == 0 )
		return;

	//remove the first portion, leaving the domain name
	//check for 3 or more dots
	CStringArray sArray;	
	int nElements;
	
	if( nElements =  ParseString( (LPCSTR)*pUrl, ".", &sArray )  )
	{
		if( nElements > 2 )
		{
			
			//build the rest of the url
			*pUrl ="*.";
			for( int i = 1; i < nElements; i++ )
			{
				*pUrl += sArray[i];
				if( i < nElements - 1  )
					*pUrl += ".";
			}
		}
	}

}


/////////////////////////////////////////////////////////////////////////////
//
BOOL ExtractUrl( CString *pUrl, BOOL bUseWildCards )
{
	int start = pUrl->Find( "//" );
	if( start >  0 )
	{
		start += 2;
		int end = pUrl->Find(  "/" , start);

		if( end > 0 )
		{
			CString sUrl = pUrl->Mid( start, end-start );
			
			//use wild cards for banners
			if( bUseWildCards )
				pUrl->Format( "*/%s/*", sUrl );
			else
				*pUrl = sUrl;
			return( TRUE );		

		}
	}
	return( FALSE );
}

/////////////////////////////////////////////////////////////////////////////
//
void ReadDataFile( LPCSTR pFileName, CStringArray* pList )
{
	CString s;
	CStdioFile file;
	CString sFileName;
	
	QualifyPathName( &sFileName, pFileName );

	pList->RemoveAll( );
	
	if( file.Open( sFileName, CFile::modeRead  ) )  
	{
		while( file.ReadString( s ) )
			if( s.GetLength() && s.GetAt(0) >= ' ' && s.GetAt(0) != ';' )
				pList->Add( s );

		file.Close( );
	}

}

/////////////////////////////////////////////////////////////////////////////
//
void WriteDataFile( LPCSTR pFileName, CStringArray* pList )
{
	CString s;
	CStdioFile file;
	CString sFileName;
	
	QualifyPathName( &sFileName, pFileName );
	
	if( file.Open( sFileName, CFile::modeCreate | CFile::modeWrite  ) )  
	{
		int iLen = pList->GetUpperBound( );
		for( int i=0; i <= iLen; i++ )
		{
			s = pList->GetAt( i ) + '\n';
			file.WriteString( s );
		}

		file.Close( );
	}  else {
        CString sErr;
        sErr.Format( "Error writing to file:\n%s", sFileName );
        ShowError( sErr );
        bErrorDisplayed_Data = TRUE;
    }        

}

/////////////////////////////////////////////////////////////////////////////
//
void ReadBlockedFile( )
{
	ReadDataFile( SZ_BLOCKED_SITE_INI, &m_theApp->m_BlockedSiteList );
	ReadDataFile( SZ_BLOCKED_IMAGE_INI, &m_theApp->m_BlockedImageList );
}

/////////////////////////////////////////////////////////////////////////////
//
void ReadRejectFile( )
{
	ReadDataFile( SZ_REJECT_INI, &m_theApp->m_RejectList );
}

/////////////////////////////////////////////////////////////////////////////
//
void ReadAcceptFile( )
{
	ReadDataFile( SZ_ACCEPT_INI, &m_theApp->m_AcceptList );
}

/////////////////////////////////////////////////////////////////////////////
//
void WriteBlockedFile( )
{
	WriteDataFile( SZ_BLOCKED_SITE_INI, &m_theApp->m_BlockedSiteList );
	WriteDataFile( SZ_BLOCKED_IMAGE_INI, &m_theApp->m_BlockedImageList );
}

/////////////////////////////////////////////////////////////////////////////
//
void WriteRejectFile( )
{
	WriteDataFile( SZ_REJECT_INI, &m_theApp->m_RejectList );
}

/////////////////////////////////////////////////////////////////////////////
//
void WriteAcceptFile( )
{
	WriteDataFile( SZ_ACCEPT_INI, &m_theApp->m_AcceptList );
}


/////////////////////////////////////////////////////////////////////////////
//
void WriteCookieCopFile( )
{
	WritePrivateProfile( SZ_COOKIE_IP, m_theApp->m_sCookieIP );
	
	
	WritePrivateProfile( SZ_COOKIE_PORT, m_theApp->m_nCookiePort );

	WritePrivateProfile( SZ_GATEWAY_IP, m_theApp->m_sGatewayIP );
	
	WritePrivateProfile( SZ_GATEWAY_PORT, m_theApp->m_nGatewayPort );

	WritePrivateProfile(  SZ_GATEWAY_ENABLE, m_theApp->m_bEnableGateway );

	//startup mode
	WritePrivateProfile( SZ_STARTUPMODE, m_theApp->m_nStartupMode );

	//display on top
	WritePrivateProfile( SZ_ONTOP, m_theApp->m_bOnTop );
	
	WritePrivateProfile( SZ_DEFAULTACTION, m_theApp->m_bDefaultAction );

	WritePrivateProfile( SZ_JAVASCRIPT, m_theApp->m_bSearchJavaScript );
	
	WritePrivateProfile( SZ_DISABLECOOKIELIST,m_theApp->m_bDisableCookieList );

	WritePrivateProfile( SZ_MAX_EVENTS, m_theApp->m_nMaxEvents );
 	
	WritePrivateProfile( SZ_DISABLELOG, m_theApp->m_bDisableLog );
	
	WritePrivateProfile(  SZ_EVENTMASK, m_theApp->m_wEventMask);

	WritePrivateProfile(  SZ_RESET_STATS, m_theApp->m_bSaveStats);

	WritePrivateProfile(  SZ_CONFIRM_DELETE, m_theApp->m_bConfirmDelete );

	WritePrivateProfile(  SZ_DISABLEBLOCKEDLIST, m_theApp->m_bDisableBlockedList );

	WritePrivateProfile(  SZ_CHECK_REFERER, m_theApp->m_bCheckReferer );

    WritePrivateProfile(  SZ_BLOCKED_SITE_FILE, m_theApp->m_sBlockedSiteFile );
	WritePrivateProfile(  SZ_BLOCKED_IMAGE_FILE, m_theApp->m_sBlockedImageFile );

    WritePrivateProfile(  SZ_USE_DEFAULT_SIZES, m_theApp->m_bUseDefaultBufferSizes );
    WritePrivateProfile( SZ_DISABLED, m_theApp->m_bDisableCookieCop );    
    WritePrivateProfile( SZ_DISABLE_PROXY, m_theApp->m_bDisableProxy );
    WritePrivateProfile( SZ_BUFFER_SIZE, m_theApp->m_nWinsockBufSize );

    WritePrivateProfile( SZ_DNS_CACHE_SIZE, m_theApp->m_nDNS_CacheSize );
    WritePrivateProfile( SZ_DNS_CACHE_DISABLE,m_theApp->m_bDNSCacheDisable );

    WritePrivateProfile( SZ_POPUP_CHECK_DISABLED, m_theApp->m_bPopupCheckDisabled  );        
    WritePrivateProfile( SZ_POPUP_MODE, m_theApp->m_nPopupMode );        

    WritePrivateProfile( SZ_TIMEOUT, m_theApp->m_nConnectionTimeout );        
    WritePrivateProfile( SZ_DEFAULT_TIMEOUT, m_theApp->m_bUseDefaultTimeOut );        

}


		
/////////////////////////////////////////////////////////////////////////////
//
void ReadCookieCopFile( )
{
    
	GetPrivateProfile( SZ_COOKIE_IP,  &m_theApp->m_sCookieIP, "CookieCop" );
	GetPrivateProfile( SZ_COOKIE_PORT, &m_theApp->m_nCookiePort );
	GetPrivateProfile( SZ_GATEWAY_IP, &m_theApp->m_sGatewayIP );
	GetPrivateProfile( SZ_GATEWAY_PORT, &m_theApp->m_nGatewayPort, 80 );
	GetPrivateProfile( SZ_GATEWAY_ENABLE, &m_theApp->m_bEnableGateway );

	//for log.cpp
	GetPrivateProfile( SZ_MAX_EVENTS, &m_theApp->m_nMaxEvents, 50 );	//v1.2 default to 50

	//search javascript
	GetPrivateProfile( SZ_JAVASCRIPT, &m_theApp->m_bSearchJavaScript );

	GetPrivateProfile( SZ_DEFAULTACTION, (int*)&m_theApp->m_bDefaultAction, (int)ACCEPT_COOKIE);

	//startup mode
	GetPrivateProfile( SZ_STARTUPMODE, (int*)&m_theApp->m_nStartupMode, (int)STARTUP_WINDOWS );

	//event mask
    GetPrivateProfile( SZ_EVENTMASK, (int*)&m_theApp->m_wEventMask, -1 );
	
	//display on top
	GetPrivateProfile( SZ_ONTOP, &m_theApp->m_bOnTop, 1 );

	//disable options
	GetPrivateProfile( SZ_DISABLECOOKIELIST, &m_theApp->m_bDisableCookieList );

    GetPrivateProfile( SZ_DISABLELOG, &m_theApp->m_bDisableLog );

	GetPrivateProfile( SZ_RESET_STATS, &m_theApp->m_bSaveStats );

	GetPrivateProfile( SZ_CONFIRM_DELETE, &m_theApp->m_bConfirmDelete, 1 );

	//html file for block
	CString sTmp;
	QualifyPathName( &sTmp, "CookieCop.htm" );

	GetPrivateProfile( SZ_BLOCKED_SITE_FILE, &m_theApp->m_sBlockedSiteFile, sTmp );
	 
	
	QualifyPathName( &sTmp, "CookieCop.gif" );
	GetPrivateProfile( SZ_BLOCKED_IMAGE_FILE, &m_theApp->m_sBlockedImageFile, sTmp );
	
	GetPrivateProfile( SZ_DISABLEBLOCKEDLIST, &m_theApp->m_bDisableBlockedList );

    GetPrivateProfile( SZ_CHECK_REFERER, &m_theApp->m_bCheckReferer );

    GetPrivateProfile( SZ_VIEWER_OPEN, &m_theApp->m_bViewerVisible );

    GetPrivateProfile( SZ_DISABLED, &m_theApp->m_bDisableCookieCop );
    
    GetPrivateProfile( SZ_USE_DEFAULT_SIZES, &m_theApp->m_bUseDefaultBufferSizes, TRUE );

    GetPrivateProfile( SZ_BUFFER_SIZE, &m_theApp->m_nWinsockBufSize, 8192 );

    GetPrivateProfile( SZ_DNS_CACHE_SIZE, &m_theApp->m_nDNS_CacheSize, 10 );
    GetPrivateProfile( SZ_DNS_CACHE_DISABLE, &m_theApp->m_bDNSCacheDisable, TRUE );

    GetPrivateProfile( SZ_DISABLE_PROXY, &m_theApp->m_bDisableProxy );    

    GetPrivateProfile( SZ_POPUP_CHECK_DISABLED, &m_theApp->m_bPopupCheckDisabled, 1 );        
    GetPrivateProfile( SZ_POPUP_MODE, &m_theApp->m_nPopupMode );        
    
    GetPrivateProfile( SZ_TIMEOUT, &m_theApp->m_nConnectionTimeout, 60 );//seconds            
    GetPrivateProfile( SZ_DEFAULT_TIMEOUT, &m_theApp->m_bUseDefaultTimeOut, TRUE );        
    
}

/////////////////////////////////////////////////////////////////////////////
//
void ReadStatsFile( )
{
	CString s;
	CStdioFile file;
	char *pend;

	CString sFileName;
	QualifyPathName( &sFileName, SZ_STATS_FILE );

	if( file.Open( sFileName, CFile::modeRead  ) )  
	{
		for( int i = 0; i < STATS_MAX; i++ )
		{
			if( file.ReadString( s ) )
			{
				m_theApp->m_dwStats[i] = strtoul( s, &pend, 10 );
				if( m_theApp->m_dwStats[i] == ULONG_MAX )
					m_theApp->m_dwStats[i] = 0;					
			} else {
				m_theApp->m_dwStats[i] = 0;
			}
		}
		file.Close( );
	}
}

/////////////////////////////////////////////////////////////////////////////
//
void WriteStatsFile( )
{
	CString s;
	CStdioFile file;

	CString sFileName;
	QualifyPathName( &sFileName, SZ_STATS_FILE );

	if( file.Open( sFileName, CFile::modeCreate | CFile::modeWrite ) )  
	{
		for( int i = 0; i < STATS_MAX; i++ )
		{
			s.Format( "%lu\n", m_theApp->m_dwStats[i] );
			file.WriteString( s );
		}
		file.Close( );
	}
}



/////////////////////////////////////////////////////////////////////////////
//
BOOL MatchPattern( LPCSTR pString, LPCSTR pPattern ) 
{ 
    char c, p; 
 
    for (; ;) 
	{ 
        switch( p = *pPattern++ ) 
		{ 
            //end of pattern, return true if end of string
			case 0:                             
                return *pString ? FALSE : TRUE; 
 
            //match any number of characters
			case '*': 
                while (*pString) 
				{               
                    if( MatchPattern( pString++, pPattern ) ) 
                        return TRUE; 
                } 
                return MatchPattern (pString, pPattern); 
 
            //match a single character
		    case '?': 
                if (*pString++ == 0)          
                    return FALSE;             
                break; 
 
 
            //exact match -- not case sensitive
			default: 
                c = *pString++; 
                if( toupper(c) != toupper(p) )  
                    return FALSE;               
 
                break; 
        } 
    } 
} 


/////////////////////////////////////////////////////////////////////////////
//
LPSTR GetHostLine( LPSTR p, LPSTR pBuf )
{
int i = 0;
BYTE ch;
    
    while( *p )
    {
        ch = *p++;
            
        if( ch == 0 )
            break;  

        if( ch == 0x0a )
            break;

        if( ch != 0x0d )
            pBuf[i++] = ch;

        if( i >= MAX_BUF_LEN )
            break;

    }

    pBuf[i] = 0;
    return( p );

}


/////////////////////////////////////////////////////////////////////////////
//
void GetTheHostName( LPSTR p )
{
LPSTR pSave = p;
char sz[256];
int i = 0;
BYTE ch;
    
    if( p == 0 )
        return;

    
    //remove leading white space
    while( *p )
        if( isspace( *p ) )
            p++;
        else
            break;
    //skip the IP address
    while( *p )
        if( !isspace( *p ) && *p !='#' )
            p++;
        else
            break;
        
    
    //remove leading white space
    while( *p )
        if( isspace( *p ) )
            p++;
        else
            break;
    
    if( *p == '#' )
        goto QUICK_OUT;
        
    //find next space
    while( *p )
        if( isspace( *p ) )
            p++;
        else
            break;

    //find next non-space
    while( *p )
        if( isspace( *p ) )
            p++;
        else
            break;



    while( *p )
    {
        ch = *p++;
                
        if( ch <= ' '  )
            break;
    
        sz[i++] = ch;

        if( i >= MAX_BUF_LEN )
            break;
    }



QUICK_OUT:
    sz[i] = 0;
    lstrcpy( pSave, sz );
    
}

/////////////////////////////////////////////////////////////////////////////
//
BOOL IsWinNT( )
{

	OSVERSIONINFO  osvi; 
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO); 
	GetVersionEx(&osvi); 
 
	return( VER_PLATFORM_WIN32_NT == osvi.dwPlatformId );
}

/////////////////////////////////////////////////////////////////////////////
//returns path to HOSTS file
void GetHostsFile( CString *pHost )
{
char szBuf[MAX_PATH];

    GetWindowsDirectory( szBuf, sizeof( szBuf ) );

	if( IsWinNT( ) )
		strcat( szBuf, "\\system32\\drivers\\etc\\hosts" );
	else
		strcat( szBuf, "\\hosts" );

	*pHost = szBuf;
}

/////////////////////////////////////////////////////////////////////////////
//
BOOL WriteHostsFile( LPSTR lpAddr, LPCSTR lpName )
{
	BOOL bFound = ParseHostsFile( NULL, lpName );
	
	if( bFound == FALSE )
	{
		CString sFileName;
		GetHostsFile( &sFileName );

		CStdioFile file;
		if( file.Open( sFileName, CFile::modeWrite | CFile::modeNoTruncate | CFile::modeCreate ) )  
		{
			char sz[512];
			wsprintf( sz, "%s\t%s\n", lpAddr, lpName );

			file.SeekToEnd( );
			file.WriteString( sz );
			file.Close( );
		}
		bFound = ParseHostsFile( NULL, lpName ); 	
	}
	return( bFound );
}


/////////////////////////////////////////////////////////////////////////////
//parse the entries contained in the HOSTS file and populate a list control
//returns TRUE if lpCurrent was found in the list
BOOL ParseHostsFile( CWnd* pCtrl, LPCSTR lpCurrent )
{
HANDLE hFile;
OFSTRUCT of;
DWORD dwSize;
char szBuf[MAX_BUF_LEN];
CString sHostFile;
LPSTR p;
HANDLE h;
int i;
BOOL bFound = FALSE;
LONG wPos = 0;
DWORD dwExtra;

	if( pCtrl )
		pCtrl->SendMessage( CB_RESETCONTENT, 0, 0 );   

	GetHostsFile( &sHostFile );
	hFile = (HANDLE)OpenFile( sHostFile, &of, OF_READ | OF_SHARE_COMPAT );

    if( hFile != (HANDLE)HFILE_ERROR )
    {   
        dwSize = GetFileSize( hFile, &dwExtra ); 
        if( dwSize )
        {
            h = GlobalAlloc( GHND, dwSize + 1 );
            if( h == 0 )
                return FALSE; //show error!
            p = (LPSTR)GlobalLock( h );
            ReadFile( hFile, p, dwSize, &dwExtra, 0 );
            
            i = 0;
            while( *p )
            {
                p = GetHostLine( p, szBuf );
                GetTheHostName( szBuf );
                if( szBuf[0] && szBuf[0] != '#' )
				{
					if( lpCurrent && !bFound )
					{
						char b1[256], b2[256];
						strncpy( b1, szBuf, sizeof(b1) );
						strncpy( b2, lpCurrent, sizeof(b2) );
						strlwr( b1 );
						strlwr( b2 );
						if( strstr( b1, b2 ) )
							bFound = TRUE;
					}
					
					if( pCtrl )
						pCtrl->SendMessage(  CB_ADDSTRING, 0, (LPARAM)(LPSTR)szBuf );

				 }
       
            }
            GlobalUnlock( h );
            GlobalFree( h );
        }

        _lclose( (HFILE)hFile );
    }

	if( pCtrl )
	{
		if( lpCurrent && *lpCurrent )
		{
			wPos = pCtrl->SendMessage( CB_FINDSTRING, 0, (LPARAM)lpCurrent );
			if( wPos == CB_ERR )
				wPos = pCtrl->SendMessage( CB_ADDSTRING, 0, (LPARAM)lpCurrent );
		}

		//select the first entry 
		if( wPos == CB_ERR )
			wPos = 0;

		pCtrl->SendMessage( CB_SETCURSEL, (WPARAM)wPos, 0 );
	}

	return( bFound );
}

///////////////////////////////////////////////////////////////////////////////////////////
//
LPSTR FormatNumber( DWORD N )
{
	  #define BUF_SIZE 128
	  static char buf[BUF_SIZE+1];
	  
	  
	  int len = 1, posn = 1, sign = 1;
      char *ptr = buf + BUF_SIZE - 1;
      *ptr-- = NULL;
 

      for ( ; len <= BUF_SIZE; ++len, ++posn)
      {
            *ptr-- = (char)((N % 10L) + '0');
            if (0L == (N /= 10L))
                  break;
      
	        if (0 == (posn % 3))
            {
                  *ptr-- = ',';
                  ++len;
            }
      
	        if (len >= BUF_SIZE)
                  return( "" );
      }

      if (0 > sign)
      {
            if (len >= BUF_SIZE)
			{
				return( "" );
			}
                 
            *ptr-- = '-';
            ++len;
      }

      memmove(buf, ++ptr, len + 1);
      return( buf );
}

 	

/////////////////////////////////////////////////////////////////////////////
//
void QualifyPathName( CString *pFile, LPCSTR pIni )
{
char szName[MAX_PATH];
LPSTR p;

	//Qualify the INI file to the same location as our exe
	GetModuleFileName( AfxGetInstanceHandle( ), szName, sizeof( szName ) );
	p = strrchr( szName, '\\' );

	if( p )
		*(++p) = 0;
	else 
		p = szName;

	strcat( p, pIni );
	*pFile = szName;
	
}

/////////////////////////////////////////////////////////////////////////////
//
BOOL IsValidDir( LPCTSTR pPath )
{
    DWORD dwAttrib = GetFileAttributes( pPath ); 
    
	if (dwAttrib == -1) 
        return( FALSE ); 
    
	if (dwAttrib & FILE_ATTRIBUTE_DIRECTORY)  
        return( TRUE ); 
     
    return( FALSE ); 
}

///////////////////////////////////////////////////////////////////////////////////////////
//
void SaveViewerPosition( CRect *pRect )
{
    WritePrivateProfile( SZ_WINPOS_TOP, pRect->top );
    WritePrivateProfile( SZ_WINPOS_LEFT, pRect->left );
    WritePrivateProfile( SZ_WINPOS_BOTTOM, pRect->bottom );
    WritePrivateProfile( SZ_WINPOS_RIGHT, pRect->right );

}

///////////////////////////////////////////////////////////////////////////////////////////
//
void LoadViewerPosition( CRect *pRect )
{
    int t,l,r,b;

    GetPrivateProfile( SZ_WINPOS_TOP, &t, -1 );
    GetPrivateProfile( SZ_WINPOS_LEFT, &l, -1 );
    GetPrivateProfile( SZ_WINPOS_BOTTOM, &b, -1 );
    GetPrivateProfile( SZ_WINPOS_RIGHT, &r, -1 );

    pRect->top = t;
    pRect->left = l;
    pRect->bottom = b;
    pRect->right = r;

}


/////////////////////////////////////////////////////////////////////////////
//
void GetPrivateProfile( LPCSTR pSection, CString* pString, LPCSTR pDefault )
{
	CString sFileName;
	QualifyPathName( &sFileName, SZ_COOKIECOP_INI );

    char buf[MAX_PATH];

    GetPrivateProfileString( SZ_APPNAME, pSection, pDefault, buf, sizeof(buf),sFileName );

    *pString = buf;

}

/////////////////////////////////////////////////////////////////////////////
//
void GetPrivateProfile( LPCSTR pSection, int* pValue, int nDefault )
{
	CString sFileName;

	QualifyPathName( &sFileName, SZ_COOKIECOP_INI );
    *pValue = GetPrivateProfileInt( SZ_APPNAME, pSection, nDefault, sFileName );

}


/////////////////////////////////////////////////////////////////////////////
//
void WritePrivateProfile( LPCSTR pSection, LPCSTR pValue )
{
	CString sFileName;

	QualifyPathName( &sFileName, SZ_COOKIECOP_INI );
	BOOL bResult = WritePrivateProfileString( SZ_APPNAME, pSection, pValue, sFileName );
    if( !bResult && !bErrorDisplayed_Ini )
    {
        CString sErr;
        sErr.Format( "Error writing to file:\n%s", sFileName );
        ShowError( sErr );
        bErrorDisplayed_Ini = TRUE;
    }

}


/////////////////////////////////////////////////////////////////////////////
//
void WritePrivateProfile( LPCSTR pSection, int nValue )
{

	CString sFileName;
	char buf[256];

	QualifyPathName( &sFileName, SZ_COOKIECOP_INI );
	wsprintf( buf, "%u", nValue );
	BOOL bResult = WritePrivateProfileString( SZ_APPNAME, pSection, buf, sFileName );

    if( !bResult && !bErrorDisplayed_Ini )
    {
        CString sErr;
        sErr.Format( "Error writing to file:\n%s", sFileName );
        ShowError( sErr );
        bErrorDisplayed_Ini = TRUE;
    }

}


/////////////////////////////////////////////////////////////////////////////
//
void VerifyFile( LPCSTR pDesc, LPCSTR pFile )
{
	CFile file;

	if( file.Open( pFile, CFile::modeRead | CFile::typeBinary ) )  
	{
		file.Close( );
	} else {

		CString s;
		s.Format( "Warning: The %s file does not exist.\n\nFile name: '%s'", pDesc,pFile );

		HWND hwnd = m_theApp->m_pDlg->GetSafeHwnd();

		if( IsWindowVisible( hwnd ) == FALSE )
			hwnd = GetDesktopWindow( );

		MessageBox( hwnd, s, SZ_APPNAME, MB_SETFOREGROUND | MB_ICONWARNING | MB_OK );
	}

}

/////////////////////////////////////////////////////////////////////////////
//
void LoadDataFiles( )
{
    ReadFileData( &m_theApp->m_BlockedSiteFileData, m_theApp->m_sBlockedSiteFile );
    ReadFileData( &m_theApp->m_BlockedImageFileData, m_theApp->m_sBlockedImageFile );
    ReadFileData( &m_theApp->m_PopupFileData, m_theApp->m_sPopupFile );	
    ReadFileData( &m_theApp->m_PopupFileDataJS, m_theApp->m_sPopupFileJS );	
}

/////////////////////////////////////////////////////////////////////////////
//
BOOL ReadFileData( CByteArray* pArray, LPCSTR pFile )
{
   CFile file;
   DWORD dwSize = 0;
   if( file.Open( pFile, CFile::modeRead | CFile::typeBinary ) )  
   {
		dwSize = file.GetLength( );
        pArray->SetSize( dwSize );
        file.Read( pArray->GetData( ), dwSize );
		file.Close( );		
    }
    return( dwSize != 0 );
}

/////////////////////////////////////////////////////////////////////////////
//
void ResetFileErrorWarnings( )
{
    bErrorDisplayed_Ini = 0;
    bErrorDisplayed_Data = 0;

}



/////////////////////////////////////////////////////////////////////////////
// Debug messages -- disabled in release build
void _cdecl DebugOut_CompiledOutOfReleaseBuild( TCHAR *format,... )
{
    return;
	
    #ifdef _DEBUG

    TCHAR szBuf[DEFAULT_BUFSIZE+100];
    TCHAR sz[DEFAULT_BUFSIZE+200];
	va_list argptr;

	va_start( argptr, format );
	wvsprintf( szBuf, format, argptr );
	va_end( argptr);
	wsprintf(sz, _T("CookieCop -- %s\n"), szBuf  );
	OutputDebugString( sz );
	#endif
}

