/*=========================================================================*/
/*                          Listen.cpp                                     */
/*                                                                         */
/*                   CookieCop's listening socket                          */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/

#include "stdafx.h"
#include "cookiecop.h"
#include "Globals.h"
#include "Listen.h"
#include "Proxy.h"
#include "DlgPropSheet.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


UINT ProxyThread(LPVOID lpParam);
UINT AcceptThread(LPVOID lpParam);
HANDLE hAcceptThread;
HANDLE  m_hExit;


BOOL bAttachThread;

#ifdef _DEBUG
static int sem;
#endif



/////////////////////////////////////////////////////////////////////////////
// CListen

CListen::CListen()
{
	m_hListenSocket = INVALID_SOCKET;
}


/////////////////////////////////////////////////////////////////////////////
// 
CListen::~CListen()
{
    try
    {
	    CloseSocket( );

        if( bAttachThread )
        {
            DebugOut( "Waiting for attachthread" );
            Sleep( 1000 );
        }

    }
    catch(...)
    {
        DebugOut( "Exception in CListen::~CListen()" );
    }
    
}


/////////////////////////////////////////////////////////////////////////////
// CListen member functions


/////////////////////////////////////////////////////////////////////////////
// 
void CListen::CloseSocket( )
{
	if( m_hListenSocket != INVALID_SOCKET )
		closesocket( m_hListenSocket );		
	
	m_hListenSocket = INVALID_SOCKET;
    Sleep( 500 );
}

/////////////////////////////////////////////////////////////////////////////
// 
BOOL CListen::StartListen( )
{
	extern UINT SOCKET_MSG;	
    char buf[256],errmsg[512];
	SOCKADDR_IN sockAddr;
	
    if( m_hListenSocket != INVALID_SOCKET )	
        CloseSocket( );

	memset( &sockAddr,0,sizeof(sockAddr) );
	
	sockAddr.sin_family = AF_INET;
	sockAddr.sin_port = htons( (u_short)m_theApp->m_nCookiePort );
	sockAddr.sin_addr.s_addr = htonl( INADDR_LOOPBACK );

    m_hListenSocket = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );

    if( m_hListenSocket == INVALID_SOCKET )
	{
		char buf[256],errmsg[512];
		GetWinsockError( buf );
		wsprintf( errmsg, "Unable to create listen socket.\n\n%s", buf );
		ShowError( errmsg );
		return( FALSE );
	}

    //get the default socket buffer sizes
    int e;
    int nSize = sizeof(int);
    e = getsockopt( m_hListenSocket, SOL_SOCKET, SO_RCVBUF, (char*)&m_theApp->m_nRecvBufSize, &nSize );
    e = getsockopt( m_hListenSocket, SOL_SOCKET, SO_SNDBUF, (char*)&m_theApp->m_nSendBufSize, &nSize );


	int option = 1;
    int err;

    //resuse should be called before bind
	err = setsockopt( m_hListenSocket, SOL_SOCKET, SO_REUSEADDR, (char*)&option, sizeof(option) );
    err = bind( m_hListenSocket, (PSOCKADDR)&sockAddr, sizeof(sockAddr) );
    err = WSAAsyncSelect( m_hListenSocket, m_theApp->m_pDlg->GetSafeHwnd( ), 0, 0 );
    
    ULONG block = 0;
	int ret = ioctlsocket( m_hListenSocket, FIONBIO,(unsigned long*) &block );
	if( ret )
		ShowWinsockError( m_hListenSocket );

	if( err == SOCKET_ERROR )
	{
		GetWinsockError( buf );
		wsprintf( errmsg, "Error creating blocking socket.\n\n%s", buf );
		ShowError( errmsg );
		return( FALSE );
	}

	err = listen( m_hListenSocket, SOMAXCONN );

	if( err == SOCKET_ERROR )
	{
		GetWinsockError( buf );
		wsprintf( errmsg, "Error creating listen socket.\n\n%s", buf );
		ShowError( errmsg );
		return( FALSE );
	}


	//
	// Create the exit signal event object
	//
	m_hExit = CreateEvent(NULL,		// Security
						 TRUE,		// Manual reset
						 FALSE,		// Initial State
						 NULL);		// Name


    SECURITY_ATTRIBUTES sa;
	ZeroMemory( &sa, sizeof( sa ) );
	sa.nLength = sizeof( sa );
	
	DWORD dw;
	m_hAcceptThread = ::CreateThread( &sa, NULL, (LPTHREAD_START_ROUTINE)AcceptThread, (LPVOID)this, 0, &dw );

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CListen::IsSocketValid( )
{
	return( m_hListenSocket != INVALID_SOCKET );
}


/////////////////////////////////////////////////////////////////////////////
// 
UINT ProxyThread( LPVOID lpParam )
{

    HANDLE hDone = CreateEvent(NULL,	// Security
				    		 TRUE,		// Manual reset
					    	 FALSE,		// Initial State
						     NULL);		// Name

    HANDLE hCloseDone = CreateEvent(NULL,	// Security
				    		 TRUE,		// Manual reset
					    	 FALSE,		// Initial State
						     NULL);		// Name


		SECURITY_ATTRIBUTES sa;
		ZeroMemory( &sa, sizeof( sa ) );
		sa.nLength = sizeof( sa );
    
    #ifdef _DEBUG
    sem++;
    DebugOut( "Starting proxy %u", sem );
    #endif

    try {
	    CProxy* pProxy = new CProxy( );	//start a new proxy session

	    if( pProxy )
	    {

		    LPINT s = (LPINT)lpParam; //get socket
		    pProxy->m_hBrowserSocket = (SOCKET)s;
            pProxy->StartSession( );
		    delete pProxy;	
	    }
    } catch(...)
    {
        DebugOut("Exception in ProxyThread" );
    }

    
    #ifdef _DEBUG
    DebugOut( "Closing proxy %u", sem );
    sem--;
    #endif

    CloseHandle( hDone );    
    CloseHandle( hCloseDone );
    ExitThread( 0 );
    return 0;
}



/////////////////////////////////////////////////////////////////////////////
// 
UINT AcceptThread( LPVOID lpParam )
{
	SOCKET s;
	struct sockaddr sockAddr;  

    try
    {	
	CListen* pThis = (CListen*)lpParam;            

	while( pThis->m_hListenSocket != INVALID_SOCKET )
	{
		memset(&sockAddr,0,sizeof(sockAddr));
		int len = sizeof( sockAddr );

		s = accept( pThis->m_hListenSocket, (PSOCKADDR)&sockAddr, &len );

    	BOOL option = 1;
        int err;
        err = setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char*)&option, sizeof(option) );

		if( s != INVALID_SOCKET )
		{
			ULONG block = 0;
			int ret = ioctlsocket( s, FIONBIO,(unsigned long*) &block );
			if( ret )
				ShowWinsockError( s );


			m_theApp->m_pDlg->ShowTrayIcon( );
            bAttachThread = TRUE;

		    SECURITY_ATTRIBUTES sa;
		    ZeroMemory( &sa, sizeof( sa ) );
		    sa.nLength = sizeof( sa );
		    DWORD dw;
		    HANDLE hThread = ::CreateThread( &sa, NULL, (LPTHREAD_START_ROUTINE)ProxyThread, (LPVOID)s, 0, &dw );
			if (!hThread)
			{
				closesocket( s );
				TRACE("Could not create thread\n");
				return 0;
			}
		    CloseHandle( hThread );	

		}

	}
    

	//
	// Wait for exit event
	//
	WaitForSingleObject( m_hExit, INFINITE );

	//
	// Wait for ALL clients to exit
	//
    extern LONG nProxyCount;
    BOOL bDone = FALSE;
    int nLoop = nProxyCount * 10;
    for( int i = 0; i < nLoop && bDone == FALSE; i++ )
    {
    	DebugOut("Waiting for proxy threads %u", nProxyCount );
        if( nProxyCount )
            Sleep( 1000 );
        else 
            bDone = TRUE;
    }    

    bAttachThread = FALSE;

    }
    catch(...)
    {
        DebugOut( "Exception in AttachThread" );
    }

    hAcceptThread = 0;
	return 0;
}



/////////////////////////////////////////////////////////////////////////////
// 
BOOL CListen::StopListen( )
{

    if(	m_hListenSocket != INVALID_SOCKET )
    {
	    //
	    // Signal the exit event
	    //
        SetEvent( m_hExit );
	    CloseSocket( );

   	    //
	    // Give the proxy thread a chance to clean up
	    //
	    int nRet = WaitForSingleObject( m_hAcceptThread, 5000 );
	    if (nRet == WAIT_TIMEOUT)
		    DebugOut( "TIMEOUT waiting for ListenThread to close");

	    CloseHandle( m_hAcceptThread );
	    CloseHandle( m_hExit );
    }
	return 1;
}


