/*=========================================================================*/
/*                        Log.cpp                                          */
/*                   Event log functions                                   */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/

#include "stdafx.h"
#include "cookiecop.h"
#include "globals.h"
#include "Log.h"
#include "CookieRecvDlg.h"
#include "BlockedSiteDlg.h"
#include "hlp/helpids.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//static int iNumEvents;
int CALLBACK ListViewCompareProc(LPARAM, LPARAM, LPARAM);
BOOL bDontDeleteEventData ;


//return index from image list
int GetEventImage( EVENT_TYPE event )
{
	switch ( event )
	{
		case EVENT_COOKIE_BLOCKED:      return 6;
		case EVENT_CONNECT_SSL:		    return 5;
		case EVENT_CONNECT:			    return 3;		
		case EVENT_COOKIE_ACCEPTED:     return 1;	
		case EVENT_COOKIE_JS_ACCEPTED:  return 1;
		case EVENT_SITE_BLOCKED:	    return 2;
		case EVENT_IMAGE_BLOCKED:       return 7;
		case EVENT_COOKIE_SENT:		    return 4;
		case EVENT_SESSION_COOKIE:	    return 8;
        case EVENT_REFERER_REMOVED:     return 9;
        case EVENT_REFERRER_REMOVED_JS: return 9;
		default:
		case EVENT_COOKIE_REJECTED:	    return 0;
	}

}


/////////////////////////////////////////////////////////////////////////////
// CLog dialog
IMPLEMENT_DYNCREATE(CLog, CPropertyPage)


/////////////////////////////////////////////////////////////////////////////
// 
CLog::CLog() : CPropertyPage(CLog::IDD)
{
	m_CurrentSortColumn = -1;
//	m_ImageList = NULL;

	//{{AFX_DATA_INIT(CLog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


/////////////////////////////////////////////////////////////////////////////
// 
CLog::~CLog() 
{
}


/////////////////////////////////////////////////////////////////////////////
// 
void CLog::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLog)
	DDX_Control(pDX, IDC_LOGLIST, m_ListCtrl);
	//}}AFX_DATA_MAP
}


/////////////////////////////////////////////////////////////////////////////
// 
BEGIN_MESSAGE_MAP(CLog, CPropertyPage)
	//{{AFX_MSG_MAP(CLog)
	ON_BN_CLICKED(IDC_CLEAREVENTS, OnClearevents)
	ON_NOTIFY(NM_DBLCLK, IDC_LOGLIST, OnDblclkLoglist)
	ON_WM_CTLCOLOR()
	ON_BN_CLICKED(IDC_COOKIE_ACCEPTED, OnCookieAccepted)
	ON_BN_CLICKED(IDC_COOKIE_REJECTED, OnCookieRejected)
	ON_BN_CLICKED(IDC_CONNECT, OnConnect)
	ON_BN_CLICKED(IDC_SITE_BLOCKED, OnSiteBlocked)
	ON_EN_CHANGE(IDC_MAX_EVENTS, OnChangeMaxEvents)
	ON_BN_CLICKED(IDC_COOKIE_SENT, OnCookieSent)
	ON_BN_CLICKED(IDC_DISABLELOG, OnDisablelog)
	//}}AFX_MSG_MAP
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_LOGLIST, OnCustomDrawList)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLog message handlers

BOOL CLog::PreTranslateMessage(MSG* pMsg) 
{
	m_ToolTip.RelayEvent( pMsg );	
	return CPropertyPage::PreTranslateMessage(pMsg);
}

/////////////////////////////////////////////////////////////////////////////
// 
void CLog::UpdateControls( )
{
	char sz[33];

	CheckDlgButton( IDC_SITE_BLOCKED, m_theApp->m_wEventMask & EVENT_SITE_BLOCKED );
	CheckDlgButton( IDC_CONNECT, m_theApp->m_wEventMask & EVENT_CONNECT );
	CheckDlgButton( IDC_COOKIE_ACCEPTED, m_theApp->m_wEventMask & EVENT_COOKIE_ACCEPTED );
	CheckDlgButton( IDC_COOKIE_REJECTED, m_theApp->m_wEventMask & EVENT_COOKIE_REJECTED );
	CheckDlgButton( IDC_COOKIE_SENT, m_theApp->m_wEventMask & EVENT_COOKIE_SENT );
	CheckDlgButton( IDC_DISABLELOG,	m_theApp->m_bDisableLog );
	SetDlgItemText( IDC_MAX_EVENTS, itoa( m_theApp->m_nMaxEvents, sz, 10 ) ); 	
	m_ListCtrl.EnableWindow( m_theApp->m_bDisableLog == FALSE );
}

void CLog::AddToImageList( int nID )
{
	//v1.2  corrected background color
	COLORREF crBackground = RGB(192,192,192);	
	CBitmap bmp;
	
	if( bmp.LoadBitmap( nID ) )
	{  
		if( m_ImageList == NULL )
		{
			BITMAP bm;
			bmp.GetBitmap( &bm );	
			m_ImageList.Create(	nID, bm.bmWidth, 1, crBackground );
		} else {
			m_ImageList.Add( &bmp, crBackground );
		}
	}

}

/////////////////////////////////////////////////////////////////////////////
// 
BOOL CLog::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();


	//force report style
	HWND hwndLV = m_ListCtrl.m_hWnd;
    DWORD dwStyle = GetWindowLong( hwndLV, GWL_STYLE );  
    SetWindowLong( hwndLV, GWL_STYLE, dwStyle | LVS_REPORT ); 

	//CListCtrl
	LV_COLUMN lv;
	lv.mask = LVCF_TEXT;
	
	lv.pszText = "Time";
	m_ListCtrl.InsertColumn( COL_TIME, &lv );

	lv.pszText = "Site";
	m_ListCtrl.InsertColumn( COL_URL, &lv );

	lv.pszText = "Event";
	m_ListCtrl.InsertColumn( COL_EVENT, &lv );

	//make sure we have full-row select
	m_ListCtrl.SetExtendedStyle(LVS_EX_FULLROWSELECT);
	
	SetColumnWidths( );

	m_ToolTip.Create( this );
	m_ToolTip.AddTool( GetDlgItem( IDC_LOGLIST ), IDS_LOGTIP );
	m_ToolTip.AddTool( GetDlgItem( IDC_CLEAREVENTS ), IDS_CLEAREVENTS );
	m_ToolTip.AddTool( GetDlgItem( IDC_MAX_EVENTS ), IDS_MAX_EVENTS );


	AddToImageList( IDB_REJECT_COOKIES );
	AddToImageList( IDB_ACCEPT_COOKIES );
	AddToImageList( IDB_BLOCKED );  
	AddToImageList( IDB_URL );  
	AddToImageList( IDB_COOKIE_UPLOAD  );  
	AddToImageList( IDB_CONNECT_SSL );  
	AddToImageList( IDB_COOKIE_BLOCKED );  
	AddToImageList( IDB_BLOCKED_IMAGE );  
	AddToImageList( IDB_SESSION_COOKIE );
    AddToImageList( IDB_REFERRER_REMOVED );
							 
	m_ListCtrl.SetImageList( &m_ImageList, LVSIL_SMALL );

	SendDlgItemMessage( IDC_SPIN1, UDM_SETRANGE, 0,(LPARAM) MAKELONG ( 200, 1));
	SendDlgItemMessage( IDC_MAX_EVENTS, EM_SETLIMITTEXT, 3, 0 );
	
	HWND hwnd = GetDlgItem( IDC_MAX_EVENTS )->GetSafeHwnd( );
	SendDlgItemMessage( IDC_SPIN1, UDM_SETBUDDY, (WPARAM)hwnd , 0 );	
	
	//image list for header control
	COLORREF crBackground = RGB(192,192,192);	

	m_pHeaderCtrl = m_ListCtrl.GetHeaderCtrl( );
	m_HeaderImageList.Create( IDB_ARROWS, 16, 2, crBackground );
	m_pHeaderCtrl->SetImageList( &m_HeaderImageList );

	//for v1.3, if the EVENT_COOKIE_ACCEPTED is in the event mask then add EVENT_SESSION_COOKIE and JAVA COOKIES
	if( m_theApp->m_wEventMask & EVENT_COOKIE_ACCEPTED )
	{
		m_theApp->m_wEventMask |= EVENT_SESSION_COOKIE;
		m_theApp->m_wEventMask |= EVENT_COOKIE_JS_ACCEPTED;
	}

    //ditto 1.3
    if( m_theApp->m_wEventMask & EVENT_CONNECT )
    {
        m_theApp->m_wEventMask |= EVENT_REFERER_REMOVED;    
        m_theApp->m_wEventMask |= EVENT_REFERRER_REMOVED_JS;
    }

	SetEventListSize( );
	UpdateControls( );
	SetSortColumn( 0 );
	SetModified( FALSE );

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

/////////////////////////////////////////////////////////////////////////////
// 
void CLog::SaveColumnWidth( int index, int width )
{
	CString sKey,sWidth;

	sWidth.Format( "%u", width );
	sKey.Format( "%s%u", SZ_COLWIDTH, index );

	CString sFileName;
	QualifyPathName( &sFileName, SZ_COOKIECOP_INI  );

	WritePrivateProfileString( SZ_APPNAME, sKey, sWidth, sFileName );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CLog::SetColumnWidths( )
{
	int i, width;
	CRect rc;
	CString sKey;
	

	m_ListCtrl.GetClientRect( &rc );            

	LV_COLUMN lv;
	lv.mask = LVCF_WIDTH;

	for( i = 0; i < COL_ENDOFLIST; i++ )
		m_ListCtrl.SetColumnWidth( i, 0 );

	CString sFileName;
	QualifyPathName( &sFileName, SZ_COOKIECOP_INI  );

	int count = COL_ENDOFLIST;		
	for( i = 0; i < COL_ENDOFLIST; i++ )
	{
		sKey.Format( "%s%u", SZ_COLWIDTH, i );
		width = GetPrivateProfileInt( SZ_APPNAME, sKey, 0, sFileName );
		if( width < 1 )
		{
			width = rc.Width( ) / count;
		}
		m_ListCtrl.SetColumnWidth( i, width );
	}

}

int CLog::AddEventToListCtrl( CEventData* pEvent, int nEventIndex, int nIndex )
{

	LV_ITEM lv;	
	int index;

	memset( &lv, 0, sizeof( lv ) );

	//send -1 to append
	if( nIndex == -1 )
		index = m_ListCtrl.GetItemCount( );
	else
		index = nIndex;

	lv.mask		= LVIF_PARAM | LVIF_IMAGE | LVIF_TEXT;
	lv.iItem	= index;
	lv.iImage	= GetEventImage( pEvent->event );
	lv.lParam   = nEventIndex;	 //store the index in the CListCtrl's data
	lv.pszText	= (LPTSTR)(LPCSTR)pEvent->sTime;
	nIndex = m_ListCtrl.InsertItem( &lv );


	lv.mask = LVIF_TEXT ;
	lv.iSubItem = COL_URL;
	lv.pszText = (LPTSTR)(LPCSTR)pEvent->sUrl;	
	m_ListCtrl.SetItem( &lv );

	lv.iSubItem = COL_EVENT;
	lv.pszText = (LPTSTR)(LPCSTR)pEvent->sCommand;
	m_ListCtrl.SetItem( &lv );

	return( nIndex );
}

/////////////////////////////////////////////////////////////////////////////
//AddEvent -- add an entry to our Event Log
void CLog::AddEvent( EVENT_TYPE event, LPCSTR pTime, LPCSTR pURL, LPCSTR pCommand, LPCSTR pPattern )
{
	if( m_theApp->m_bDisableLog || m_theApp->m_bDisableCookieCop )
		return;

	//Remove control characters from the HTTP header string 
	char pMsg[400];
	strncpy( pMsg, pCommand, sizeof( pMsg )-1 );
	pMsg[399] = 0;

	//remove any cr/lf at the end
	int iLen = strlen( pMsg );
	while( iLen )
	{
		if( pMsg[iLen] < 32 )
			pMsg[iLen] = 0;
		else
			break;
		iLen--;
	}

	m_ListCtrl.SetRedraw( FALSE );

	CEventData data;
	data.event = event;
	data.sTime = pTime;
	data.sCommand = pMsg;
	data.sUrl = pURL;
	data.sPattern = pPattern;

	//if we have reached the max number of events, restart at the beginning -- circular buffer
	if( ( m_theApp->m_Head ) >= m_theApp->m_EventList.GetSize( ) )
		m_theApp->m_Head = 0;


	//add entry to event list
	int index;
	m_theApp->m_EventList[m_theApp->m_Head] = data;
	index = m_theApp->m_Head;
	
	InterlockedIncrement((LPLONG) &m_theApp->m_Head );
	InterlockedExchange((LPLONG)  &m_theApp->m_Tail,( m_theApp->m_Tail, m_theApp->m_Head ) );

	if( m_theApp->m_wEventMask & event )
	{
		//if we have reached the max number of events remove the entries from the beginning
		while( m_ListCtrl.GetItemCount( ) && m_ListCtrl.GetItemCount( ) >= m_theApp->m_nMaxEvents )
			m_ListCtrl.DeleteItem( 0 );

		int i = AddEventToListCtrl( &data, index, -1 );
		if( i >= 0 )  
			m_ListCtrl.EnsureVisible( i, TRUE );

	}

	//update stats
	switch( event )
	{
		//1.3
		case EVENT_COOKIE_SENT:
			m_theApp->m_dwStats[STATS_COOKIES_SENT]++ ; 
			break;
		
		case EVENT_CONNECT_SSL:
		case EVENT_CONNECT:				
			m_theApp->m_dwStats[STATS_CONNECTIONS]++ ; 
			break;

		case EVENT_SITE_BLOCKED:	
		case EVENT_IMAGE_BLOCKED:
			m_theApp->m_dwStats[STATS_BLOCKED]++; 
			break;

		case EVENT_COOKIE_BLOCKED: 
		case EVENT_COOKIE_REJECTED:	
		case EVENT_COOKIE_JS:
			m_theApp->m_dwStats[STATS_REJECTED]++; 
			m_theApp->m_CookieState |= STATE_COOKIE_REJECTED;   //for traffic light icon
			break;
		
		case EVENT_SESSION_COOKIE:
		case EVENT_COOKIE_ACCEPTED:	
		case EVENT_COOKIE_JS_ACCEPTED:
			m_theApp->m_dwStats[STATS_ACCEPTED]++;  
			m_theApp->m_CookieState |= STATE_COOKIE_RECEIVED;   //for traffic light icon
			break;

        case EVENT_REFERRER_REMOVED_JS:
        case EVENT_REFERER_REMOVED:
            m_theApp->m_dwStats[STATS_REFERRER]++;  
            break;

	}

	m_ListCtrl.SetRedraw( TRUE );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CLog::AddEvent( EVENT_TYPE event, LPCSTR pURL, LPCSTR pMsg, LPCSTR pPattern )
{
	CString s;
	CTime t = CTime::GetCurrentTime( );
	s = t.Format( "%I:%M:%S" );
	
	AddEvent( event, (LPCSTR)s, pURL, pMsg, pPattern );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CLog::AddEvent( EVENT_TYPE event, LPCSTR pURL, LPCSTR pMsg )
{
	AddEvent( event, pURL, pMsg, ""	);
}


/////////////////////////////////////////////////////////////////////////////
// 
void CLog::OnClearevents() 
{
	m_ListCtrl.DeleteAllItems( );	
	InterlockedExchange((LPLONG)&m_theApp->m_Head, 0 );
	InterlockedExchange((LPLONG)&m_theApp->m_Tail, 0 );

	m_theApp->m_EventList.RemoveAll();	
	SetEventListSize( );
	ReDrawLog( );
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CLog::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult) 
{
	NM_LISTVIEW *pnmlv = (NM_LISTVIEW *)lParam;
	
	switch( pnmlv->hdr.code )   
	{
		case LVN_COLUMNCLICK:
		{
			LPNMLISTVIEW pnmv = (LPNMLISTVIEW) lParam; 
			SetSortColumn( pnmv->iSubItem );
		}
		break;


		case HDN_ENDTRACK:
		{
			LPNMHEADER phdr = (LPNMHEADER)lParam;
			LPHDITEM pitem = phdr->pitem;
			SaveColumnWidth( phdr->iItem, pitem->cxy );
			break;
		}


	}
	
	return CPropertyPage::OnNotify(wParam, lParam, pResult);
}


/////////////////////////////////////////////////////////////////////////////
// 
void CLog::OnDblclkLoglist(NMHDR* pNMHDR, LRESULT* pResult) 
{
	UNREFERENCED_PARAMETER( pNMHDR );

	*pResult = 0;

	int nItem = m_ListCtrl.GetNextItem( -1, LVNI_SELECTED ); 
	if( nItem >= 0 )
	{
		
		int index = (int)m_ListCtrl.GetItemData( nItem );
		CEventData data = m_theApp->m_EventList.GetAt( index );
		CEventData event_data = data;		

		switch( data.event )
		{
			case EVENT_COOKIE_SENT:
			case EVENT_COOKIE_JS: 
			case EVENT_COOKIE_REJECTED: 
			case EVENT_COOKIE_ACCEPTED:
			case EVENT_COOKIE_BLOCKED: 
			case EVENT_SESSION_COOKIE:
			case EVENT_COOKIE_JS_ACCEPTED:
			{
				CCookieRecvDlg* pDlg = new CCookieRecvDlg; 
				pDlg->m_pEventData = &event_data;
				pDlg->m_pListCtrl = &m_ListCtrl;
				pDlg->m_nItem = nItem;

				pDlg->DoModal( );	
				delete pDlg;
				break;
			}

			
			case EVENT_CONNECT:
			case EVENT_CONNECT_SSL:
			case EVENT_SITE_BLOCKED:
			case EVENT_IMAGE_BLOCKED:
            case EVENT_REFERER_REMOVED:
            case EVENT_REFERRER_REMOVED_JS:  
			{
				CBlockedSiteDlg* pDlg = new CBlockedSiteDlg; 
				pDlg->m_pEventData = &event_data;
				pDlg->m_pListCtrl = &m_ListCtrl;
				pDlg->m_nItem = nItem;
				pDlg->DoModal( );	
				delete pDlg;
				break;
			}
			
			default:
				MessageBox( "Unknown event type" );
				break;
		}		

	}

	return;
}



/////////////////////////////////////////////////////////////////////////////
//  Set the colors for the check boxes
HBRUSH CLog::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	HBRUSH hbr = CPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);
	COLORREF crColor;
	
	if( nCtlColor == CTLCOLOR_STATIC )
	{
		int iID = pWnd->GetDlgCtrlID( );
		switch( iID )
		{
			case IDC_COOKIE_REJECTED: crColor = COLOR_REJECTED; break;
			case IDC_COOKIE_ACCEPTED: crColor = COLOR_ACCEPTED; break;
			case IDC_SITE_BLOCKED   : crColor = COLOR_BLOCKED ; break;
			case IDC_COOKIE_SENT	: crColor = COLOR_SENT    ; break;
			default:
				return hbr;
		}
		pDC->SetTextColor( crColor );
	}
	
	// TODO: Return a different brush if the default is not desired
	return hbr;
}

/////////////////////////////////////////////////////////////////////////////
// 
void CLog::ReDrawLog( )
{
	m_theApp->m_wEventMask = 0;
	
	if( IsDlgButtonChecked( IDC_COOKIE_SENT) )
		m_theApp->m_wEventMask |= EVENT_COOKIE_SENT ;

	if( IsDlgButtonChecked( IDC_COOKIE_ACCEPTED ) )
		m_theApp->m_wEventMask |= EVENT_COOKIE_ACCEPTED | EVENT_SESSION_COOKIE | EVENT_COOKIE_JS_ACCEPTED;

	if( IsDlgButtonChecked( IDC_COOKIE_REJECTED) )
		m_theApp->m_wEventMask |= EVENT_COOKIE_REJECTED | EVENT_COOKIE_BLOCKED | EVENT_COOKIE_JS;

	if( IsDlgButtonChecked( IDC_CONNECT ))
		m_theApp->m_wEventMask |= EVENT_CONNECT | EVENT_CONNECT_SSL | EVENT_REFERRER_REMOVED_JS | EVENT_REFERER_REMOVED;

	if( IsDlgButtonChecked( IDC_SITE_BLOCKED ) ) 
		m_theApp->m_wEventMask |= EVENT_SITE_BLOCKED | EVENT_IMAGE_BLOCKED;


	//Reset our Event Log list, but don't delete the saved events!
	bDontDeleteEventData = TRUE;
	m_ListCtrl.DeleteAllItems( );	
	bDontDeleteEventData = FALSE; 


	int nArraySize = m_theApp->m_EventList.GetSize( );	
	int iNumEvents = min( m_theApp->m_Tail, m_theApp->m_nMaxEvents );
	int nEnd   = m_theApp->m_Tail -1;
	int nStart = m_theApp->m_Head -1;	//zero base
	int i = nStart;
	int nCount = 0;
	int nLoop = 0;

	ASSERT( iNumEvents < m_theApp->m_EventList.GetSize( ) );
	ASSERT( nStart     < m_theApp->m_EventList.GetSize( ) );
	ASSERT( nEnd       < m_theApp->m_EventList.GetSize( ) );

	//~3
	if( ( iNumEvents > m_theApp->m_EventList.GetSize( ) ) || 
		( nStart     > m_theApp->m_EventList.GetSize( ) ) || 
		( nEnd       > m_theApp->m_EventList.GetSize( ) ) )
			return;

	if( nStart >= 0 && nEnd >= 0 )
	{
		while( nCount < iNumEvents )
		{		
			//just in case
			if( ++nLoop > nArraySize )
				break;

			CEventData data = m_theApp->m_EventList.GetAt( i );	

			if( data.event & m_theApp->m_wEventMask )
			{
				AddEventToListCtrl( &data, i, 0 );
				nCount++;
			}
			
			if( --i < 0 )
				i = nEnd;


			//we have gone through the list
			if( i == nStart )
				break;
		}	
	}

}

/////////////////////////////////////////////////////////////////////////////
// 
void CLog::OnCookieSent() 
{
	ReDrawLog( );	
	SetModified( TRUE );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CLog::OnCookieAccepted() 
{
	ReDrawLog( );	
	SetModified( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CLog::OnCookieRejected() 
{
	ReDrawLog( );
	SetModified( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CLog::OnConnect() 
{
	ReDrawLog( );	
	SetModified( TRUE );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CLog::OnSiteBlocked() 
{
	ReDrawLog( );
	SetModified( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
BOOL CLog::OnSetActive() 
{
	return CPropertyPage::OnSetActive();
}

/////////////////////////////////////////////////////////////////////////////
// 
void CLog::OnChangeMaxEvents() 
{
	SetModified( TRUE );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CLog::SetSortColumn( int iColumn )
{
	HDITEM hditem;

	//get the current sort state
	hditem.mask = HDI_LPARAM;
	m_pHeaderCtrl->GetItem( iColumn, &hditem ); 
	int nSort = hditem.lParam;	
	
	
	if( m_CurrentSortColumn != -1 )
	{	
		if( m_CurrentSortColumn == iColumn )
			nSort = !nSort;
	} else {
		//first time being displayed
		m_CurrentSortColumn = 0;
	}

	//turn off the old
	hditem.mask = HDI_FORMAT | HDI_IMAGE;
	hditem.fmt  = HDF_STRING;
	m_pHeaderCtrl->SetItem( m_CurrentSortColumn, &hditem );

	hditem.mask = HDI_IMAGE | HDI_FORMAT | HDI_LPARAM;
	hditem.fmt  = HDF_IMAGE | HDF_STRING | HDF_LEFT | HDF_BITMAP_ON_RIGHT;
	hditem.lParam = nSort;
	hditem.iImage = nSort;
	m_pHeaderCtrl->SetItem( iColumn, &hditem );
	m_CurrentSortColumn = iColumn;
	m_SortAccending = nSort;
	m_ListCtrl.SortItems( ListViewCompareProc, (LPARAM)this );
}


/////////////////////////////////////////////////////////////////////////////
// 
int CALLBACK ListViewCompareProc(LPARAM lparam1, 
                                 LPARAM lparam2,
                                 LPARAM lparamSort)
{

	CLog* pLog = (CLog*)lparamSort;

	CEventData data1, data2;
	 
	if( pLog->m_SortAccending )
	{
		data1 = m_theApp->m_EventList.GetAt( (int)lparam1 );
		data2 = m_theApp->m_EventList.GetAt( (int)lparam2 );
	} else {
		data2 = m_theApp->m_EventList.GetAt( (int)lparam1 );
		data1 = m_theApp->m_EventList.GetAt( (int)lparam2 );
	}

	switch( pLog->m_CurrentSortColumn )
	{
		case COL_TIME: 
			return( strcmp( data1.sTime, data2.sTime ) );
		
		case COL_URL:
			return( strcmp( data1.sUrl, data2.sUrl ) );
			
		case COL_EVENT:
			return( data1.event > data2.event );
		
		default:
			return 0;
	}

}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CLog::OnKillActive() 
{
	return CPropertyPage::OnKillActive();
}

/////////////////////////////////////////////////////////////////////////////
// 
void CLog::OnDisablelog() 
{
	m_theApp->m_bDisableLog = IsDlgButtonChecked( IDC_DISABLELOG );
	SetModified( TRUE );

	m_ListCtrl.EnableWindow( m_theApp->m_bDisableLog == FALSE );

	if( m_theApp->m_bDisableLog )
		m_ListCtrl.DeleteAllItems( );	

	ReDrawLog( );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CLog::OnReset() 
{
	m_theApp->ReadDataFiles();	
	UpdateControls( );
	ReDrawLog( );
	SetModified( FALSE );	
	CPropertyPage::OnReset();
}

/////////////////////////////////////////////////////////////////////////////
// 
void CLog::SetEventListSize( )
{

	if( (m_theApp->m_nMaxEvents * NUMBER_OF_EVENTS) > m_theApp->m_EventList.GetSize() )
		m_theApp->m_EventList.SetSize( m_theApp->m_nMaxEvents * NUMBER_OF_EVENTS );

	InterlockedExchange((LPLONG)&m_theApp->m_Tail,
						 min( m_theApp->m_Tail, m_theApp->m_nMaxEvents ) );	//largest valid entry
	InterlockedExchange((LPLONG)&m_theApp->m_Head,
						 min( m_theApp->m_Head, m_theApp->m_nMaxEvents ) );	//largest valid entry

}



/////////////////////////////////////////////////////////////////////////////
// 
BOOL CLog::OnApply() 
{
	char sz[256];
	GetDlgItemText(	IDC_MAX_EVENTS, sz, sizeof(sz) );
	int iNum = atoi( sz );
	if( iNum < 1 )
		iNum = 1;
	m_theApp->m_nMaxEvents = iNum; 

	SetEventListSize( );
	SetModified( FALSE );	
	return CPropertyPage::OnApply();

}

/////////////////////////////////////////////////////////////////////////////
// 
void CLog::OnCustomDrawList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LPNMLVCUSTOMDRAW  lplvcd = (LPNMLVCUSTOMDRAW)pNMHDR;

	switch( lplvcd->nmcd.dwDrawStage )
	{
		case CDDS_PREPAINT:
			*pResult = CDRF_NOTIFYITEMDRAW;
			break;

		case CDDS_ITEMPREPAINT: 
		{
			
			int index = lplvcd->nmcd.lItemlParam ;
			CEventData data = m_theApp->m_EventList.GetAt( index );

			COLORREF cr;
			switch( data.event )
			{
				case EVENT_COOKIE_JS    :
				case EVENT_COOKIE_BLOCKED :
				case EVENT_COOKIE_REJECTED: cr = COLOR_REJECTED ; break;
				case EVENT_COOKIE_ACCEPTED: cr = COLOR_ACCEPTED; break;
				case EVENT_SESSION_COOKIE : cr = COLOR_ACCEPTED; break;
				case EVENT_COOKIE_JS_ACCEPTED: cr = COLOR_ACCEPTED; break;
				case EVENT_COOKIE_SENT    :	cr = COLOR_SENT; break;
				case EVENT_IMAGE_BLOCKED  :
				case EVENT_SITE_BLOCKED   : cr = COLOR_BLOCKED; break;
				default:
					cr = 0;
			}
				lplvcd->clrText = cr;
				*pResult = CDRF_NEWFONT;
		}
		break;
		
		
		default:
		// it wasn't a notification that was interesting to us, just return zero.
		*pResult = 0;
	}
		
}

