/*=========================================================================*/
/*                       PerformanceDlg.cpp                                */
/*                                                                         */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/

#include "stdafx.h"
#include "cookiecop.h"
#include "PerformanceDlg.h"
#include "DlgPropSheet.h"
#include <wininet.h>
#include <atlbase.h>


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define SLIDER_CACHE_INCREMENT 10
#define SLIDER_BUFFER_INCREMENT 8192
#define SLIDER_TIMER_INCREMENT 5

static DWORD dwLastByteCount  = 0;


/////////////////////////////////////////////////////////////////////////////
// CPerformanceDlg property page

IMPLEMENT_DYNCREATE(CPerformanceDlg, CPropertyPage)

CPerformanceDlg::CPerformanceDlg() : CPropertyPage(CPerformanceDlg::IDD)
{
	//{{AFX_DATA_INIT(CPerformanceDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CPerformanceDlg::~CPerformanceDlg()
{
}

void CPerformanceDlg::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPerformanceDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPerformanceDlg, CPropertyPage)
	//{{AFX_MSG_MAP(CPerformanceDlg)
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()
    ON_BN_CLICKED(IDC_USE_DEFAULT_SIZES, OnUseDefaultSizes)
	ON_BN_CLICKED(IDC_DNS_CACHE_DISABLE, OnDnsCacheEnable)
	ON_BN_CLICKED(IDC_USE_DEFAULT_TIMEOUT, OnUseDefaultTimeout)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_OPTIMIZE, OnOptimize)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPerformanceDlg message handlers

BOOL CPerformanceDlg::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
 
    InitDisplay( );
    m_bResetCache = FALSE;
    SetModified( FALSE );

    return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


/////////////////////////////////////////////////////////////////////////////
// 
void CPerformanceDlg::InitDisplay( )
{
	CSliderCtrl* pCtrl; 

    //buffer sizes
    pCtrl = (CSliderCtrl*)GetDlgItem( IDC_BUFFER_SLIDER );
    pCtrl->SetRange( 1, 20 );
    pCtrl->SetTicFreq( 1 );
	pCtrl->SetPos( m_theApp->m_nWinsockBufSize / SLIDER_BUFFER_INCREMENT );
    
    CheckDlgButton( IDC_USE_DEFAULT_SIZES, m_theApp->m_bUseDefaultBufferSizes );
    SetDlgItemInt( IDC_RECV_DEFAULT_TEXT, m_theApp->m_nRecvBufSize );
    SetDlgItemInt( IDC_SEND_DEFAULT_TEXT, m_theApp->m_nSendBufSize );
  
    //dns cache
    pCtrl = (CSliderCtrl*)GetDlgItem( IDC_DNS_SLIDER );
    pCtrl->SetRange( 1, MAX_CACHE / SLIDER_CACHE_INCREMENT );
    pCtrl->SetTicFreq( 1 );
	pCtrl->SetPos( m_theApp->m_nDNS_CacheSize / SLIDER_CACHE_INCREMENT );
    
    CheckDlgButton( IDC_DNS_CACHE_DISABLE, m_theApp->m_bDNSCacheDisable );

    //timeout 
    CheckDlgButton( IDC_USE_DEFAULT_TIMEOUT, m_theApp->m_bUseDefaultTimeOut );
    pCtrl = (CSliderCtrl*)GetDlgItem( IDC_TIMEOUT_SLIDER );
    pCtrl->SetRange( 60 / SLIDER_TIMER_INCREMENT, 240 / SLIDER_TIMER_INCREMENT  );
    pCtrl->SetTicFreq( SLIDER_TIMER_INCREMENT  );
	pCtrl->SetPos( m_theApp->m_nConnectionTimeout / SLIDER_TIMER_INCREMENT );
	
    OnUseDefaultTimeout( );
    UpdateSliders( );
    UpdateBufferSettings( );
    UpdateDNSSettings( );	
    EnableOptions( );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CPerformanceDlg::UpdateSliders( )    
{
    char buf[80];

    CSliderCtrl* pCtrl = (CSliderCtrl*)GetDlgItem(IDC_BUFFER_SLIDER);
    wsprintf( buf, "%s KB", FormatNumber( m_theApp->m_nWinsockBufSize / 1024 ) );
    SetDlgItemText( IDC_BUFFER_TEXT, buf );

    pCtrl = (CSliderCtrl*)GetDlgItem(IDC_DNS_SLIDER);
    wsprintf( buf, "%u entries", m_theApp->m_nDNS_CacheSize );
    SetDlgItemText( IDC_DNS_ENTRIES, buf );

    wsprintf( buf, "%u seconds", m_theApp->m_nConnectionTimeout );
    SetDlgItemText( IDC_TIMEOUT_TEXT, buf );

}

/////////////////////////////////////////////////////////////////////////////
// 
void CPerformanceDlg::HandleSliders( CScrollBar* pScrollBar )
{
    int nControl = pScrollBar->GetDlgCtrlID( );
    CSliderCtrl* pCtrl = (CSliderCtrl*)GetDlgItem(nControl);
    ASSERT( pCtrl != NULL );
    
    SetModified( TRUE );

    int nPos = pCtrl->GetPos( );

    switch( nControl )
    {
        case IDC_BUFFER_SLIDER:
            m_theApp->m_nWinsockBufSize = max( 1, nPos * SLIDER_BUFFER_INCREMENT );
            if( m_theApp->m_nWinsockBufSize < m_theApp->m_nLargestHostRecv )
                m_theApp->m_nLargestHostRecv = 0;
            break;
        
        case IDC_DNS_SLIDER:
            m_theApp->m_nDNS_CacheSize = nPos * SLIDER_CACHE_INCREMENT;
            m_bResetCache = TRUE;
            break;

        case IDC_TIMEOUT_SLIDER:
            m_theApp->m_nConnectionTimeout = nPos * SLIDER_TIMER_INCREMENT;
            break;                
    }

    UpdateSliders( );    
}


/////////////////////////////////////////////////////////////////////////////
// 
void CPerformanceDlg::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	HandleSliders( pScrollBar );
    CPropertyPage::OnHScroll(nSBCode, nPos, pScrollBar);
}

///////////////////////////////////////////////////////////////////////////////////////////
//respond to the slider commands
void CPerformanceDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	HandleSliders( pScrollBar );
    CPropertyPage::OnHScroll(nSBCode, nPos, pScrollBar);
}

/////////////////////////////////////////////////////////////////////////////
// 
void CPerformanceDlg::OnUseDefaultSizes() 
{
    UpdateBufferSettings( );	
    m_theApp->m_nLargestHostRecv = 0;
    SetModified( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CPerformanceDlg::UpdateBufferSettings( )
{
    EnableOptions( );
}


/////////////////////////////////////////////////////////////////////////////
// 
int CPerformanceDlg::CacheGetRatio( )
{

    LONG ratio = 0;
    LONG total = m_theApp->m_dwDNSCacheMisses + m_theApp->m_dwDNSCacheHits;
    
    if( total )
    {
        ratio = ( m_theApp->m_dwDNSCacheHits * 100 ) / total;
    }
    return( ratio );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CPerformanceDlg::UpdateDNSSettings( )
{
    EnableOptions( );
    
    SetDlgItemInt( IDC_DNS_HITS, m_theApp->m_dwDNSCacheHits );
    SetDlgItemInt( IDC_DNS_MISSES, m_theApp->m_dwDNSCacheMisses );

    char sz[30];
    sprintf( sz, "%u %%", (int)CacheGetRatio( ) );
    SetDlgItemText( IDC_DNS_RATIO, sz );
    
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CPerformanceDlg::OnApply() 
{
    WriteCookieCopFile( );
    SetModified( FALSE );	
    m_theApp->m_nLargestHostRecv = 0;
    if( m_bResetCache )
    {
        m_theApp->m_dwDNSCacheHits = m_theApp->m_dwDNSCacheMisses = 0;
        m_bResetCache = FALSE;
    }
	return CPropertyPage::OnApply();
}


/////////////////////////////////////////////////////////////////////////////
// 
void CPerformanceDlg::OnDnsCacheEnable() 
{
	SetModified( TRUE );
    m_bResetCache = TRUE;
    UpdateDNSSettings( );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CPerformanceDlg::OnCancel() 
{
	SetModified( FALSE );
    m_bResetCache = FALSE;
    ReadCookieCopFile( );	
    InitDisplay( );
	CPropertyPage::OnCancel();
}

/////////////////////////////////////////////////////////////////////////////
// 
void CPerformanceDlg::OnUseDefaultTimeout() 
{
    EnableOptions( );
    SetModified( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CPerformanceDlg::OnTimer(UINT nIDEvent) 
{
    //SetDlgItemText( IDC_MAX_RECV_BUF, FormatNumber( m_theApp->m_nLargestHostRecv / 1024) );
    char s[80];
    wsprintf( s, "%d KB", m_theApp->m_nLargestHostRecv / 1024 );
    SetDlgItemText( IDC_MAX_RECV_BUF, s );
    UpdateDNSSettings( );
	CPropertyPage::OnTimer(nIDEvent);
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CPerformanceDlg::OnSetActive() 
{
    EnableTimer( TRUE );
	return CPropertyPage::OnSetActive();
}

/////////////////////////////////////////////////////////////////////////////
// 
BOOL CPerformanceDlg::OnKillActive() 
{
    KillTimer( PERFORMANCE_TIMER );		
	return CPropertyPage::OnKillActive();
}

/////////////////////////////////////////////////////////////////////////////
// 
void CPerformanceDlg::EnableTimer( BOOL bEnable ) 
{
    if( bEnable )
        SetTimer( PERFORMANCE_TIMER, 1000, NULL );
    else
        KillTimer( PERFORMANCE_TIMER );		
}


/////////////////////////////////////////////////////////////////////////////
// 
void CPerformanceDlg::EnableOptions( )
{
    m_theApp->m_bUseDefaultBufferSizes = IsDlgButtonChecked( IDC_USE_DEFAULT_SIZES );
    GetDlgItem( IDC_BUFFER_SLIDER )->EnableWindow( !m_theApp->m_bUseDefaultBufferSizes );
    GetDlgItem( IDC_BUFFER_TEXT )->EnableWindow( !m_theApp->m_bUseDefaultBufferSizes );

    m_theApp->m_bDNSCacheDisable = IsDlgButtonChecked( IDC_DNS_CACHE_DISABLE );
    GetDlgItem( IDC_DNS_SLIDER )->EnableWindow( !m_theApp->m_bDNSCacheDisable );
    GetDlgItem( IDC_DNS_ENTRIES)->EnableWindow( !m_theApp->m_bDNSCacheDisable );

    m_theApp->m_bUseDefaultTimeOut = IsDlgButtonChecked( IDC_USE_DEFAULT_TIMEOUT );
    GetDlgItem( IDC_TIMEOUT_TEXT )->EnableWindow( !m_theApp->m_bUseDefaultTimeOut  );
    GetDlgItem( IDC_TIMEOUT_SLIDER )->EnableWindow( !m_theApp->m_bUseDefaultTimeOut );
  
}



/////////////////////////////////////////////////////////////////////////////
// Attempt to optimize settings.   This could be an entire utility by itself!
// for now, make best estimates based on data thoroughput
void CPerformanceDlg::OnOptimize() 
{
    BOOL bUpdate = FALSE;
    BOOL bEnoughData = FALSE;
  
    if( ( m_theApp->m_dwStats[STATS_BYTES_RECEIVED] - dwLastByteCount ) >= (1024 * 1024 ) )
    {
        bEnoughData = TRUE;
        dwLastByteCount = m_theApp->m_dwStats[STATS_BYTES_RECEIVED];
    }

    // see the following link for buffer sizes and optimum settings
    // msdn.microsoft.com/library/default.asp?URL=/library/techart/speedtrial.htm  
      
    
    //buffer sizes
    do {

        //if the option is not enabled, then enable it now
        if( m_theApp->m_bUseDefaultBufferSizes ) {
            m_theApp->m_nWinsockBufSize = SLIDER_BUFFER_INCREMENT;//m_theApp->m_nSendBufSize;
            CheckDlgButton( IDC_USE_DEFAULT_SIZES, FALSE );
            EnableOptions( );
            bUpdate = TRUE;
        }

        //wait until more data?
        if( m_theApp->m_nLargestHostRecv < m_theApp->m_nWinsockBufSize )
            if( !bEnoughData )
                break;
        
        //if buffer errors then reset to default size
        if( m_theApp->m_OptimizeStates.bBufferError )
        {
            m_theApp->m_nWinsockBufSize = SLIDER_BUFFER_INCREMENT;//m_theApp->m_nSendBufSize;
            m_theApp->m_OptimizeStates.bBufferError = FALSE;    //reset
            bUpdate = TRUE;
            break;
        }

        //if the largest buffer received is greater or equal to the buffer size, then increment it
        if( m_theApp->m_nLargestHostRecv >= m_theApp->m_nWinsockBufSize )
        {
            int n = ( m_theApp->m_nLargestHostRecv + SLIDER_BUFFER_INCREMENT ) / SLIDER_BUFFER_INCREMENT;            
            m_theApp->m_nWinsockBufSize = n * SLIDER_BUFFER_INCREMENT;
            bUpdate = TRUE;
            break;
        }
 

        //if the largest buffer received is less than the buffer size, then decrement it
        if( m_theApp->m_nLargestHostRecv < m_theApp->m_nWinsockBufSize )
        {
            if( m_theApp->m_nWinsockBufSize  > m_theApp->m_nSendBufSize )
            {
                 if( m_theApp->m_nWinsockBufSize > (m_theApp->m_nLargestHostRecv + SLIDER_BUFFER_INCREMENT ) )
                 {
                    int n = ( m_theApp->m_nLargestHostRecv ) / SLIDER_BUFFER_INCREMENT;            
                    m_theApp->m_nWinsockBufSize = max( n+1, 1 ) * SLIDER_BUFFER_INCREMENT;
                    bUpdate = TRUE;
                 }
            }
            break;
        }

    } while( 0 );


    //timeout
    if( m_theApp->m_bUseDefaultTimeOut )
    {
        CheckDlgButton( IDC_USE_DEFAULT_TIMEOUT, FALSE );        
        m_theApp->m_nConnectionTimeout = 240;
        EnableOptions( );
        bUpdate = TRUE;      
    } else {    
        if( m_theApp->m_OptimizeStates.bTimeout ) 
        {
            m_theApp->m_nConnectionTimeout += 30;
            m_theApp->m_nConnectionTimeout = min( 240, m_theApp->m_nConnectionTimeout );    
            m_theApp->m_OptimizeStates.bTimeout = 0;
            bUpdate = TRUE;
        }
    }

    //cache
    if( m_theApp->m_bDNSCacheDisable )
    {
        CheckDlgButton( IDC_DNS_CACHE_DISABLE, FALSE );      
        EnableOptions( );
        m_theApp->m_nDNS_CacheSize = SLIDER_CACHE_INCREMENT * 5;
        bUpdate = TRUE;      
    } else {  
        
        if( bEnoughData )    //wait until more data?
        {
            int ratio = CacheGetRatio( );
            if( ratio < 80 )
            {
                m_theApp->m_nDNS_CacheSize = max( m_theApp->m_nDNS_CacheSize - 10, 10 );
                bUpdate = TRUE;
            }
        }
    }
    
    if( bUpdate ) 
    {
        InitDisplay( );
        ZeroMemory( &m_theApp->m_OptimizeStates, sizeof(m_theApp->m_OptimizeStates) );
    } else {
          HWND hwnd = m_theApp->m_pDlg->GetSafeHwnd();

        ::MessageBox( hwnd, "Not enough data yet for optimizing.", "CookieCop 2", MB_OK | MB_SETFOREGROUND | MB_APPLMODAL );
    }
}
