/*=========================================================================*/
/*                            Proxy.cpp                                    */
/*                                                                         */
/*             Manages the requests sent from the browser                  */
/*               and the responses sent from the server.                   */
/*                                                                         */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include "stdafx.h"
#include "cookiecop.h"
#include "Globals.h"
#include "Log.h"
#include "Proxy.h"
#include <String.h>	
#include "DlgPropSheet.h"

#pragma warning( disable : 4706 ) // assignment within conditional expression

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern UINT MSG_UPDATE_STATUSBAR;

//counter for the number of active connections 
LONG nProxyCount = 0;

//for internal DNS caching
typedef struct 
{
    char szHost[256];  
    DWORD dwAddr;
} DNS_CACHE, *PDNS_CACHE;

static DNS_CACHE DNSCache[MAX_CACHE+1];


const char SZ_BADCONNECT[] = "HTTP/1.0 404 Error connecting to site\n"
					"Pragma: no-cache\n"
					"Content-Type: text/html\n\n"
					"<html>\n"
					"<head>\n"
					"Site does not exist.\n"
					"</head></html>\n\n";

const char SZ_SSL_HEADER[] = "HTTP/1.0 200 Connection established\n"
     				"Proxy-agent: CC-Proxy/1.0\n\n";
					//the first line must be exactly as shown!
					//and you must have this second line
					//see Netscape's -- SSL tunneling and the proxy, page 3

const char SZ_RESPONSE_DATA[] = "HTTP Response data from %s.  %s"; //site and mime type

/////////////////////////////////////////////////////////////////////////////
// CProxy
CProxy::CProxy( )
{
	InterlockedIncrement( &nProxyCount );
	m_HostState = PROCESS_HTTP_HEADER;
	m_BrowserBuf.RemoveAll( );
	m_bSSL = 0;
	m_hHostSocket = INVALID_SOCKET;
	m_sDomain = "";
	m_wMimeType = TYPE_UNKNOWN; //response mime type
	
    m_bCheckJSReferrer = FALSE;
    m_bInAcceptList = m_bInRejectList = FALSE;
    m_sPattern = "";
    m_nResponseCode = 0;
    m_nContentLength = -1;
    m_sLastDomain = "";
    m_sMimeType = "";
    m_nLastPort = 0;
    m_BrowserState = PROCESS_HTTP_HEADER;

	InterlockedIncrement( &m_theApp->m_nSessionID );
	InterlockedExchange( &m_nSessionID, m_theApp->m_nSessionID );
	InterlockedIncrement((LPLONG)&m_theApp->m_nSessionCount ); 

	InterlockedExchange((LPLONG)&m_theApp->m_CookieState, 
								 m_theApp->m_CookieState |= STATE_CONNECTED );

}



/////////////////////////////////////////////////////////////////////////////
// 
CProxy::~CProxy()
{

    //make sure the sockets have been closed
	CloseSockets( );

	//decrement the number of proxy sessions
    InterlockedDecrement( &nProxyCount );

	CString stmp;
	stmp.Format( "Closing connection: %s", m_sDomain );
	Viewer( VIEWER_CC, stmp, "",0  );

	//turn the yellow light off in the traffic light icon
	//once all connections have been closed
	if( nProxyCount < 1 )
		InterlockedExchange((LPLONG)&m_theApp->m_CookieState, 
		                             m_theApp->m_CookieState &= ~STATE_CONNECTED );

	InterlockedDecrement((LPLONG)&m_theApp->m_nSessionCount ); 
	
    UpdateViewerStatusBar( );									 
}

/////////////////////////////////////////////////////////////////////////////
// Initalize data members for host connections
void CProxy::InitHostStates( )
{
    m_HostState = PROCESS_HTTP_HEADER;
    m_nHostBytes = 0;
    m_nContentLength = -1; 
    m_bChunked = FALSE;
    m_wMimeType = TYPE_UNKNOWN;
    m_sMimeType = "";
    m_sTagArray.RemoveAll( );
    m_nTagArrayOffset.RemoveAll( );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CProxy::StartSession( )
{
    int nBufSize;

    //Determine send and recv buffer sizes.  
    //Note: these are buffer sizes when communicating with Winsock -- not 
    //the DefaultRcvWindow or any other TCP/IP settings in the registry.
    if( m_theApp->m_bUseDefaultBufferSizes )
    {
        nBufSize = max( m_theApp->m_nSendBufSize, m_theApp->m_nRecvBufSize );
        nBufSize = max( 256, nBufSize );  //make sure there is something
    } else {
        nBufSize = m_theApp->m_nWinsockBufSize;
        nBufSize = max( 1, nBufSize );  //make sure there is something (allow 1 for debugging)
    }
    m_nSendBufSize = m_nRecvBufSize = nBufSize;

    char *buf = new char[nBufSize];
    if( buf )
    {
        DoProxy( buf );
	    CloseSockets( );
        delete [] buf;
    }
}


/////////////////////////////////////////////////////////////////////////////
// 
void CProxy::DoProxy( char *buf )
{
    fd_set fdread;
	int ret;
	SOCKADDR_IN sockAddr;

	memset(&sockAddr,0,sizeof(sockAddr));
	sockAddr.sin_family = AF_INET;
	sockAddr.sin_port = htons( 0 );
	sockAddr.sin_addr.s_addr = htonl( INADDR_ANY );
    
    //if using a custom buffer size, set the socket send and recv buffers
    if( !m_theApp->m_bUseDefaultBufferSizes )
    {
        int e;
        e = setsockopt( m_hBrowserSocket,SOL_SOCKET, SO_SNDBUF, (char*)&m_nSendBufSize, sizeof(m_nRecvBufSize) );
        e = setsockopt( m_hBrowserSocket,SOL_SOCKET, SO_RCVBUF, (char*)&m_nRecvBufSize, sizeof(m_nRecvBufSize) );
    }

    //disable packet coalescing between cookiecop and the browser, they are on the same machine 
    int	nagle = 1;
	ret = setsockopt( m_hBrowserSocket, IPPROTO_TCP, TCP_NODELAY, (char*) &nagle, sizeof(nagle));
    if( ret )
		ShowWinsockError( m_hBrowserSocket, m_nSessionID );

	//for viewer Kbps display
	if( m_theApp->m_nSessionCount == 1 ) {	
		InterlockedExchange( &m_theApp->m_dwSessionStartTime, GetTickCount( ) );
		InterlockedExchange( &m_theApp->m_dwSessionBytes, m_theApp->m_dwStats[STATS_BYTES_RECEIVED] );
	}

    //init host members 
    InitHostStates( );
	

    //======================================================================================
	//read and process data from the browser and the web site until the connection is closed
    //======================================================================================
	while( m_theApp->m_Listen.m_hListenSocket != INVALID_SOCKET )
	{
        InterlockedExchange((LPLONG)&m_theApp->m_CookieState, 
		                             m_theApp->m_CookieState &= ~STATE_CONNECTED );

		//====================================================================
        //Prepare the Read and Write socket sets for network I/O notification.
        //====================================================================
        FD_ZERO( &fdread );
		FD_SET( m_hBrowserSocket, &fdread );
        if( m_hHostSocket != INVALID_SOCKET )
            FD_SET( m_hHostSocket, &fdread );
	
        if( m_theApp->m_bUseDefaultTimeOut == FALSE )
        {
            TIMEVAL timeout;
            timeout.tv_sec = m_theApp->m_nConnectionTimeout;
            timeout.tv_usec = 0;
            ret = select( 0, &fdread, NULL, NULL, &timeout );
        } else {
            ret = select( 0, &fdread, NULL, NULL, NULL );            
        }
		
		//timeout 
        if( ret == 0 )
        {
            Viewer( VIEWER_CC, "Timeout", m_sDomain, m_sDomain.GetLength( ) );
            m_theApp->m_OptimizeStates.bTimeout++;                
            return; 
        }
        
        if( ret == SOCKET_ERROR )
			return;

		//===================================
        //Read from browser and send to host
        //===================================
		if( FD_ISSET( m_hBrowserSocket, &fdread ) ) 
		{
			int nBytes = recv( m_hBrowserSocket, buf, m_nRecvBufSize, 0 );
			
            if( nBytes > 0 )
			{
            	InterlockedExchange((LPLONG)&m_theApp->m_CookieState, 
								    m_theApp->m_CookieState |= STATE_CONNECTED );

				AddToBrowserBuffer( (LPBYTE)buf, nBytes );                
            }

            #ifdef _DEBUG
                Viewer( VIEWER_DEBUG, "Received from browser", buf, nBytes  ); 
            #endif

            if( ProcessBrowserData( ) == FALSE )
                return; //site was blocked or couldn't connect

            //browser closed connection, we're done
        	if( nBytes <= 0 )
            {
                if( nBytes < 0 )
                    ShowWinsockError( m_hBrowserSocket, m_nSessionID  );

                //close our end immediately
                closesocket( m_hBrowserSocket );
                m_hBrowserSocket = INVALID_SOCKET;
				return;
            }
		}	

        //==================================
        //Read from host and send to browser
        //==================================
		if( m_hHostSocket != INVALID_SOCKET && FD_ISSET( m_hHostSocket, &fdread ) ) 
		{
            int nBytes = recv( m_hHostSocket, buf, m_nRecvBufSize, 0 );

            m_BrowserState = PROCESS_HTTP_HEADER;
            
            //save the largest buffer received for the performance tab
            if( nBytes > m_theApp->m_nLargestHostRecv )
            {
                m_theApp->m_nLargestHostRecv = nBytes;
            }

            #ifdef _DEBUG
            char sDebug[100];
            int by = 0;
            if( nBytes > 0 )
                by = nBytes;
            wsprintf(sDebug, "Host sent: %d", nBytes );
            Viewer( VIEWER_DEBUG, sDebug, buf, by);
            #endif

			if( nBytes > 0 )  
			{
            	InterlockedExchange((LPLONG)&m_theApp->m_CookieState, 
								    m_theApp->m_CookieState |= STATE_CONNECTED );

				AddToHostBuffer( (LPBYTE)buf, nBytes );
				InterlockedExchange( (LPLONG)&m_theApp->m_dwStats[STATS_BYTES_RECEIVED], 
				                              m_theApp->m_dwStats[STATS_BYTES_RECEIVED] + nBytes );	
				UpdateViewerStatusBar( UPDATE_BYTES, m_theApp->m_dwSessionStartTime );
                
			}

			if( m_HostBuffer.GetSize( ) )
                ProcessHostData( );

			if( nBytes <= 0 )
            {
                return;
            }
		}	
	}
	
    return;
}



/////////////////////////////////////////////////////////////////////////////
// Processes the data sent from the browser
BOOL CProxy::ProcessBrowserData( )
{

    if( m_bSSL )
        m_BrowserState = PROCESS_HTTP_BODY;

    switch( m_BrowserState )
    {
        case PROCESS_HTTP_HEADER:
            if( CheckHeader( &m_BrowserBuf ) ) 
            {
                //Parse the request
                if( GetRequest( ) == FALSE )
                {
                    Viewer( VIEWER_CC, "Invalid HTTP request", "", 0 );
                    return FALSE;
                }
            
                InitHostStates( );

                //make the connection -- and check for blocked sites
	            if( ConnectToHost( ) == FALSE )
                    return FALSE;


                //determine if the site is in the cookie accept or reject lists
                if( !m_theApp->m_bDisableCookieCop && !m_bSSL )
                {
                    if((m_bInAcceptList = CheckAcceptList( m_sDomain, &m_sPattern ) ) == FALSE )
                        m_bInRejectList = CheckRejectList( m_sDomain, &m_sPattern );        

                    //if the site is not in the accept list and if the default
                    //action is to reject the cookie, then force m_bInRejectList to TRUE
                    if( m_theApp->m_bDefaultAction == REJECT_COOKIE )
                        if( m_bInAcceptList == FALSE )
                            m_bInRejectList = TRUE;
                
                    CheckCookies( &m_sBrowserHeaders, m_sDomain );
                }                

                //update the status bar in the viewer to reflect the new connection
                Viewer( VIEWER_BROWSER, "HTTP Request header received from browser", (LPCSTR)m_BrowserBuf.GetData( ) , m_BrowserBuf.GetSize( ) );
                UpdateViewerStatusBar( );	
                Send_Data( m_hHostSocket, &m_BrowserBuf, STATS_BYTES_SENT );
            } 
            break;

        case PROCESS_HTTP_BODY:
            //SSL connection or passthrough mode, 
            Viewer( VIEWER_BROWSER, "Sending browser data", (LPCSTR)m_BrowserBuf.GetData(), m_BrowserBuf.GetSize()  );
            Send_Data( m_hHostSocket, &m_BrowserBuf, STATS_BYTES_SENT );
            break;

        }
    
    return( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// Send host data to the browser
void CProxy::SendHostBuffer( )
{

    if( m_hBrowserSocket == INVALID_SOCKET )
        return;

    //special check for sites that do not create valid HTTP headers (www.bcentral.com)
    //if the connection is closed and we still have not seen a valid header
    //append a blank line to the header and try the cookie check again.
    if( m_HostBuffer.GetSize( ) > 0 )
    {
        if(  m_HostState == PROCESS_HTTP_HEADER )
        {
            CheckHeader( &m_HostBuffer );
            char buf[4];
            wsprintf( buf, "%c%c%", CR, LF );
            AddToHostBuffer( (LPBYTE)buf, 2 );  //append blank line 
            CreateHostHeader( );                //and create the host header
        }
    }
        
    //now, process the remaining data
    if( m_HostBuffer.GetSize( ) )
    {
        CString s;
        //we need to recalc the content-length field
        if( m_wMimeType & TYPE_TEXT )
        {                
            if( !m_theApp->m_bDisableCookieCop )
            {
                //allocate a temporary buffer that contains the host buffer
                //convert the buffer to lowercase to speed searches
                int iLen = m_HostBuffer.GetSize( );
                char* buf = new char[iLen+1]; //need to include trailing null for strings
                if( buf == NULL )
                {
                    Send_Data( m_hBrowserSocket, &m_HostBuffer, 0 );    //send the data and get out
                    return; //should show memory error here
                }

                //copy the server data to the temp buffer and null terminate for string searching
                memcpy( buf, m_HostBuffer.GetData( ), iLen );
                buf[iLen] = 0;

                //do cookie checks
                if( m_theApp->m_bDefaultAction == SESSION_COOKIE )
                {
                    if( m_bInRejectList || m_bInAcceptList )
                        CheckJavaScriptCookies( &m_HostBuffer );    
                    else
                        ConvertToSessionOnly( (LPSTR)m_HostBuffer.GetData( ), buf ); 

                } else {
                    CheckJavaScriptCookies( &m_HostBuffer );
                }

                //tags must be done BEFORE disabling popups otherwise our popup JS will be converted
                strlwr( buf );  //covert to lower case
                GetTags( (LPCSTR)buf, iLen );

                RemoveOpenFromHTML( buf, &m_HostBuffer );
                CheckJavaScriptReferrer( &m_HostBuffer );
		        CheckMetaCookies( &m_HostBuffer );              

                //if our pop-up javascript code was inserted then adjust the content-length size
                if( DisablePopups( &m_HostBuffer, buf ) )
                    FixContentLength( &m_sHostHeaders, &m_HostBuffer );
                delete [] buf;
            }
        }  
    
        //send the server header 
        SendHostHeader( );

        s.Format( SZ_RESPONSE_DATA, m_sDomain, m_sMimeType );
        Viewer( VIEWER_SERVER, s, (LPCSTR)m_HostBuffer.GetData( ), m_HostBuffer.GetSize( ) );
		
	    //send the server data 
        Send_Data( m_hBrowserSocket, &m_HostBuffer, 0 );
        m_HostBuffer.RemoveAll( );
    } else {

        //some response codes, such as 302, only contain a header
        SendHostHeader( );  

        //send any data if present
        if( m_HostBuffer.GetSize( ) )    
        {
	        Send_Data( m_hBrowserSocket, &m_HostBuffer, 0 );  
            m_HostBuffer.RemoveAll( );
        }

    }

    //init the server member states
    InitHostStates( );
    
}

/////////////////////////////////////////////////////////////////////////////
// 
void CProxy::SendHostHeader( )
{
    if( m_sHostHeaders.GetSize( ) )
    {
        CByteArray bTmp;
        CString stmp;

        FormatHeader( &m_sHostHeaders, &stmp );
        ValidateHeader( &stmp );

        int nLen = stmp.GetLength( );
        bTmp.SetSize( nLen );
        memcpy( bTmp.GetData( ), (LPCSTR)stmp, nLen );

        CString s;
        s.Format( "HTTP Response header from %s", m_sDomain );        
        Viewer( VIEWER_SERVER, s, (LPCSTR)bTmp.GetData( ), bTmp.GetSize( ) );

        Send_Data( m_hBrowserSocket, &bTmp, 0 );
        m_sHostHeaders.RemoveAll( );
    }
 }   

/////////////////////////////////////////////////////////////////////////////
// 
void CProxy::CloseSockets( )
{
    
    #ifdef _DEBUG
    Viewer( VIEWER_DEBUG, "At CloseSockets( )", "", 0 );
    #endif

    //in case the host closed the session
    if( m_hHostSocket != INVALID_SOCKET )
        SendHostBuffer( );
    	

    #ifdef BLAST_CONNECTION
    //we could slam the connection closed, like IE, however this should not be necessary
    LINGER l;
    l.l_onoff  = 1;
    l.l_linger = 0;
    
     setsockopt (m_hBrowserSocket , SOL_SOCKET, SO_LINGER, (LPSTR)&l, sizeof (linger) );
     int ret = setsockopt (m_hHostSocket, SOL_SOCKET, SO_LINGER, (LPSTR)&l, sizeof (linger) );
    #endif


    int err_browser = 0;
    int err_host = 0;

    //close the server socket
	if( m_hHostSocket != INVALID_SOCKET )
    {
		err_host = closesocket( m_hHostSocket );
        if( err_host )
            ShowWinsockError( m_hHostSocket, m_nSessionID  );
    }

    //close the browser socket
    if( m_hBrowserSocket != INVALID_SOCKET )
    {
		err_browser = closesocket(  m_hBrowserSocket );
        if( err_browser )
            ShowWinsockError( m_hBrowserSocket , m_nSessionID  );
    }

    m_hBrowserSocket = m_hHostSocket = INVALID_SOCKET;
}


/////////////////////////////////////////////////////////////////////////////
// Sends data to either the server or the browser
int CProxy::Send_Data( SOCKET s, CByteArray* pData, int nStats )
{

	if( s == INVALID_SOCKET )
		return 0;

    int n = pData->GetSize( );
    if( n == 0 )
	    return 0;

    //total bytes sent
    int total = 0;

    while( 1 )
    {	
        int maxbuf = min( m_nSendBufSize, ( n - total ) );

        ASSERT (maxbuf >= 0 );

        if( maxbuf == 0 )
            break;
                       
	    int result = send( s, (char*)pData->GetData(), maxbuf , 0 );

	    //remove the sent data from the beginning of the buffer
	    if( result > 0 )
	    {
		    total += result;

            if( total == n )
		    {
			    pData->RemoveAll( );
                break;
		    } else {
                pData->RemoveAt( 0, result );
		    }
	    }

	    //update stats. This function is shared when sending data from the browser to the host
	    //and when sending data from the proxy to back to the browser. We don't want to update 
	    //the stats when sending data back to the browser because that is really *received* 
	    //data.  Those functions will send a -1 as the nStats.
	    if( result <= 0 )
	    {
            //If buffer error then automatically decrease the buffer size
            if( WSAGetLastError( ) == WSAENOBUFS )
            {
                m_theApp->m_OptimizeStates.bBufferError = TRUE;
                m_theApp->m_nWinsockBufSize = ( max( 2048, m_theApp->m_nWinsockBufSize - 1024 ) );
            }

            break;
	    }
    }

	if( total )
    {
        if( nStats == STATS_BYTES_SENT )
		    InterlockedExchange( (LPLONG)&m_theApp->m_dwStats[STATS_BYTES_SENT], 
									      m_theApp->m_dwStats[STATS_BYTES_SENT] + total );	
	}
    		
	return( total );
}


/////////////////////////////////////////////////////////////////////////////
// Formats the header received from the host
BOOL CProxy::CreateHostHeader( )
{
    BOOL bValidHeader = CheckHeader( &m_HostBuffer );

    m_nResponseCode = 0;    

	if( bValidHeader )
	{
		CString s;
		CreateHeader( &m_HostBuffer, &m_sHostHeaders, TRUE );
		CheckCookies( &m_sHostHeaders, m_sDomain );

        //http 1.1 get response code
        CString sResponse = m_sHostHeaders.GetAt(0);
        sResponse.MakeLower( );
        LPCSTR pResponse = strstr( sResponse, "http/" );
        while( pResponse && *pResponse )
        {
            if( *pResponse == ' ' )
            {
                m_nResponseCode = atoi( pResponse );
                break;
            }
            pResponse++;
        }

        #ifdef _DEBUG
        char ss[333];
        wsprintf(ss, "HTTP Response Header %s", sResponse );
        Viewer( VIEWER_DEBUG, ss,0,0 );
        #endif

    }
    return( bValidHeader );
}

/////////////////////////////////////////////////////////////////////////////
//  
void CProxy::ProcessHostData( )
{

    if( m_bSSL )
    {
        CString s;
        s.Format( "Received from %s (secure connection)", m_sDomain );
        Viewer( VIEWER_SERVER, s, "", m_HostBuffer.GetSize( ) );
        Send_Data( m_hBrowserSocket, &m_HostBuffer, 0 );
        return;
    }
	

	switch( m_HostState )
	{
    	case PROCESS_HTTP_HEADER:
		{

            BOOL bValidHeader = CreateHostHeader( );
            
            if( bValidHeader )
            {
		        m_HostState = PROCESS_HTTP_BODY;

                //check for chunked data, if the mime type indicates that it is an image
                //or something that doesn't need parsing, send it to the browser immediately
                //otherwise, when receiving something such as a chunked AVI or MP3, the user would
                //need to wait until the entire movie/audio was downloaded before seeing anything
                if( m_bChunked )
                {
                    if( ( m_wMimeType & TYPE_TEXT ) == 0 || ( m_nResponseCode >= 300 ) || ( m_nResponseCode == 204 ) )
                    {
                        m_bChunked = FALSE;
                        m_wMimeType = TYPE_IMAGE;   
                    }
                }

                if( m_bChunked )
                {
                    m_sChunkedHeaders.Copy( m_sHostHeaders );    
                    m_sHostHeaders.RemoveAll( );

                } else {
                   if( ( m_nResponseCode >= 300 ) || m_nResponseCode == 204 )
                   {
                        //if error or if no host body then send buffer now
                        m_wMimeType = TYPE_IMAGE;   
                   }

                   //if the data is not textual, send the header immediately
                   if( ( m_wMimeType & TYPE_TEXT ) == 0 )  
                   {
                        SendHostHeader( );
                   }
                }
			} 
            
            if( !bValidHeader )
                break;
        }


		/// FALL THROUGH no break; 
        //otherwise, codes 301 will hang until host closes the connection

		//process the data portion from the server
        case PROCESS_HTTP_BODY:
            
            //if the data is chunked, we need to wait until the last chuck is received
            //in order to properly format the content size in the header
            if( m_bChunked )
            {
                ProcessChunkedData( );
                if( m_bChunked )
                    return;  //fall through when the whole chunk has been received
            }

            
            //if non textual, then send the buffer
            if( ( m_wMimeType & TYPE_TEXT ) == 0 && m_sHostHeaders.GetSize( ) == 0 )
			{
				m_nHostBytes += m_HostBuffer.GetSize( ); 
                
                CString s;
                s.Format( SZ_RESPONSE_DATA, m_sDomain, m_sMimeType );
				Viewer( VIEWER_SERVER, s, (LPCSTR)m_HostBuffer.GetData( ), m_HostBuffer.GetSize( ) );
				Send_Data( m_hBrowserSocket, &m_HostBuffer, 0 );
			} else {
                m_nHostBytes = m_HostBuffer.GetSize( );    //potentially wrong, if could not send
            }


            //check to see if all data has been received. m_nContentLength will equal -1 if the length
            //is not yet known.
            if( m_nContentLength != -1 )
            {
                if( m_nHostBytes >= m_nContentLength )
                {
                    
                    #ifdef _DEBUG
                    //CString sss;
                    //sss.Format( "Content reached content = %u bytes = %u",   m_nContentLength,  m_HostBuffer.GetSize( ) );
                    //Viewer( VIEWER_CC, sss, "", 0  );
                    #endif

                    //we've reached the content length -- send the data
                    SendHostBuffer( );
                }
            }
			break;
	}

}

/////////////////////////////////////////////////////////////////////////////
//Parse the browser request string.  Returns FALSE if bad header of if the 
//site was blocked by CC.
BOOL CProxy::GetRequest( )
{

	//receive headers until blank line! else malformed header
	CString s;
	CreateHeader( &m_BrowserBuf, &m_sBrowserHeaders );
	FormatHeader( &m_sBrowserHeaders, &s );
	ValidateHeader( &s );

	if( m_sBrowserHeaders.GetUpperBound( ) >= 0 )
	{
		CString stmp = m_sBrowserHeaders.GetAt( 0 );	
		ParseRequest( &stmp );    //sets the m_ssl flag
	}
  

	//1.1 check if valid header
	if( m_sCommand.GetLength() == 0 || m_sDomain.IsEmpty( ) ) 
	{
		char BADHEADER[] = "HTTP/1.0 400 Invalid header received from browser\n\n";
		send( m_hBrowserSocket, BADHEADER, strlen( BADHEADER ), 0 );
		return FALSE;
	}

	//if we are not forwarding to a gateway then create a new request line
	//that does not contain the http://, host, and domain portions.
	if( m_theApp->m_bEnableGateway == FALSE && m_bSSL == 0 )
	{
		CString stmp = m_sRequestLine + " " + m_sPath + " " + m_sVersion + CR + LF;
		m_sBrowserHeaders.SetAt( 0, stmp );
	}


	//check blocked site list
	if( IsSiteBlocked( ) )
	{
		CString s;
		s.Format( "Site blocked: %s", m_sDomain );
		Viewer( VIEWER_CC, s, "", 0  );
		return FALSE;
	}

    //check if we are doing a POST or WebDAV command.  If so
    //change the browser state to passthrough mode
    if( IsIdempotent( m_sBrowserHeaders.GetAt( 0 ) ) == FALSE )
    {
        m_BrowserState = PROCESS_HTTP_BODY;

    }

	//if we made it this far, we are about to connect to a site, add a connect event
	if( !m_theApp->m_bDisableCookieCop )
	{
        if( m_bSSL )
        {
			AddEvent( EVENT_CONNECT_SSL, m_sDomain, m_sBrowserHeaders.GetAt( 0 ),"" );	
		} else {
            if( !CheckReferer( ) )
                AddEvent( EVENT_CONNECT, m_sDomain, m_sBrowserHeaders.GetAt( 0 ),"" );
        }

	}

	return( TRUE );

}


/////////////////////////////////////////////////////////////////////////////
// ConnectToHost( ) -- returns FALSE if unable to connect
BOOL CProxy::ConnectToHost( )
{

	CString s;
	CString sHost;

    
    if( m_sLastDomain.IsEmpty( ) == FALSE )
    {
        if( m_sLastDomain != m_sDomain || m_nLastPort != m_nHostPort )
        {
            if( m_hHostSocket != INVALID_SOCKET )
            {

                //a different domain or port has been requested by the browser
                //close the sockets to the previous host
                closesocket( m_hHostSocket );
                m_hHostSocket = INVALID_SOCKET ;
            }
        }
    }

    m_sLastDomain = m_sDomain;
    m_nLastPort = m_nHostPort;

	sHost = m_sDomain;
	
	if( m_theApp->m_bEnableGateway )
	{
		sHost = m_theApp->m_sGatewayIP;
		m_nHostPort = m_theApp->m_nGatewayPort;
	}

		
	if( m_bSSL )
	{
		m_HostState = PROCESS_HTTP_BODY;
	}

	//if we are using a gateway or upstream proxy, then we must forward the whole request 
	//if the connection is not SSL then go ahead and add the modified
	//request line to the host buffer which will be sent after the connection is made
	if( m_theApp->m_bEnableGateway || m_bSSL == 0 )
	{
		FormatHeader( &m_sBrowserHeaders, &s );
		ValidateHeader( &s );
		
		CByteArray byTmp;
        int nLength = s.GetLength(); 
        byTmp.SetSize( nLength );
        memcpy( byTmp.GetData( ), s, nLength );

		//add any remaining data
		if( m_BrowserBuf.GetUpperBound( ) > 0 )
		{
            byTmp.Append( m_BrowserBuf );
		}
        m_BrowserBuf.RemoveAll( );
        m_BrowserBuf.Copy( byTmp );

	} else { //SSL
		//if the connection is ssl, then tell the host that 
		//a connection has been made through our proxy
		m_bSSL = TRUE;
		m_HostState = PROCESS_HTTP_BODY;
		AddToBrowserBuffer( (LPBYTE)SZ_SSL_HEADER, strlen(SZ_SSL_HEADER) );
		Send_Data( m_hBrowserSocket, &m_BrowserBuf, -1 );
		m_BrowserBuf.RemoveAll( );
        m_sBrowserHeaders.RemoveAll( );
	}

    
    CString sInfo;
    sInfo.Format( "Connecting to '%s' port = %u",sHost , m_nHostPort );
    Viewer( VIEWER_CC, sInfo, "",0  );

    if( m_hHostSocket != INVALID_SOCKET ) {
        Viewer( VIEWER_DEBUG, "Reusing socket", sHost, sHost.GetLength( ) );
    } else {
	    m_hHostSocket = Connect( sHost, m_nHostPort );
    }

	//if we couldn't connect then bail out with a 404
	if( m_hHostSocket == INVALID_SOCKET )
	{
        AddToBrowserBuffer( (LPBYTE)SZ_BADCONNECT, strlen( SZ_BADCONNECT ) );
		Send_Data( m_hBrowserSocket, &m_BrowserBuf, -1 );
		CloseSockets( );
		return FALSE;
	}
		
	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
//Check our internal cache for the host URL 
DWORD CheckDNSCache( LPCSTR pHost ) 
{
    if( !m_theApp->m_bDNSCacheDisable )
    {
        ASSERT( m_theApp->m_nDNS_CacheSize );
        ASSERT( m_theApp->m_nDNS_CacheSize <= 100 );

        for( int i = 0; i < m_theApp->m_nDNS_CacheSize && DNSCache[i].dwAddr; i++ )
        {
           if( strcmp( DNSCache[i].szHost, pHost ) == 0 )
            {
                InterlockedIncrement( &m_theApp->m_dwDNSCacheHits );
                //DebugOut( "Cache hit: %u %s", i, pHost );
                return( DNSCache[i].dwAddr );
            }
        }
        //DebugOut( "Cache miss: %u %s", i, pHost );
        InterlockedIncrement( &m_theApp->m_dwDNSCacheMisses );
    }
    return( 0 );
}

/////////////////////////////////////////////////////////////////////////////
// This is a *very* simple array for caching host and IP addresses.  However, it
//  saves us from calling the gethostbyname( ) blocking function for every browser
// request -- which will be the site we just connected to in most cases! 
void AddToDNSCache( LPCSTR pHost, DWORD dwAddr )
{
    static int nLastUpdate = 0;

    if( !m_theApp->m_bDNSCacheDisable )
    {    
        for( int i = 0; i < m_theApp->m_nDNS_CacheSize; i++ )
        {
            if( DNSCache[i].dwAddr == 0 )
            {
                DNSCache[i].dwAddr = dwAddr;
                strncpy( DNSCache[i].szHost, pHost, sizeof(DNSCache[0].szHost) );
                return;
            }
        }

        m_theApp->m_criticalSectionDNS.Lock( );
        strncpy( DNSCache[nLastUpdate].szHost, pHost, sizeof(DNSCache[0].szHost) );
        DNSCache[nLastUpdate].dwAddr = dwAddr;
        m_theApp->m_criticalSectionDNS.Unlock( );
    
        if( ++nLastUpdate >= m_theApp->m_nDNS_CacheSize )
            nLastUpdate = 0;
    }
}


/////////////////////////////////////////////////////////////////////////////
// Connect to the host
SOCKET CProxy::Connect( LPCSTR pHost, int nPort )
{

	ASSERT( pHost != NULL);
	SOCKADDR_IN sockAddr;

	memset(&sockAddr,0,sizeof(sockAddr));
	
	sockAddr.sin_family = AF_INET;
	sockAddr.sin_addr.s_addr = inet_addr( pHost );

	if( sockAddr.sin_addr.s_addr == INADDR_NONE )
	{
        if( (sockAddr.sin_addr.s_addr = CheckDNSCache( pHost ) ) == 0  )
        {
		    LPHOSTENT lphost;
		    lphost = gethostbyname( pHost );
		    if( lphost != NULL ) 
		    {
			    sockAddr.sin_addr.s_addr = ((LPIN_ADDR)lphost->h_addr)->s_addr;
                AddToDNSCache( pHost, sockAddr.sin_addr.s_addr );
		    } else {
			    ShowWinsockError( INVALID_SOCKET, m_nSessionID  );
			    return INVALID_SOCKET ;
		    }
        }

	}

	sockAddr.sin_port = htons((u_short)nPort);

    SOCKET s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    
    if (s == INVALID_SOCKET )
    {
        //DebugOut("Connect: socket() failed: %d\n", WSAGetLastError());
        return INVALID_SOCKET;
    }

	ULONG block = 0;
	int ret;

    //disable packet coalescing between the browser and the server
    //IE does this, however, all the TCP/IP books recommend against it
    //so this code is disabled 
	#ifdef NAGLE
    int	nagle = 1;
	ret = setsockopt( s, IPPROTO_TCP, TCP_NODELAY, (char*) &nagle, sizeof(nagle));
    if( ret )
		ShowWinsockError( s, m_nSessionID );
    #endif
  

    //buffer size must be set before connecting
    if( !m_theApp->m_bUseDefaultBufferSizes )
    {
        int e;
        e = setsockopt( s,SOL_SOCKET, SO_SNDBUF, (char*)&m_nSendBufSize, sizeof(m_nRecvBufSize) );
        e = setsockopt( s,SOL_SOCKET, SO_RCVBUF, (char*)&m_nRecvBufSize, sizeof(m_nRecvBufSize) );
    }

	ret = ioctlsocket( s, FIONBIO,(unsigned long*)&block );
	ret = connect( s, (SOCKADDR*)&sockAddr, sizeof(sockAddr));

    if( ret == SOCKET_ERROR )
    {
        //the browser will display a connection failure message
		closesocket( s );
        return INVALID_SOCKET;
    }

    //return the opened socket
    return s;

}	



/////////////////////////////////////////////////////////////////////////////
// 	ParseString -- Scans a string for a list of delimiters.  Returns a string
//  array containing the parsed elements and the number of entries in the 
//  string array.
int ParseString( LPCSTR pString, char *pDelimiters, CStringArray* pElements )
{

	int i;
	BYTE ch;

	pElements->RemoveAll( );

	if( pString && pDelimiters )
	{
		int nDelimiters = strlen( pDelimiters );
		BOOL bDone = FALSE;

		//skip past any initial delimiters
		while( *pString && !bDone )
		{
			ch = *pString;	

			for( i = 0; i < nDelimiters; i++ )	
			{
				//found a non delimiter, end
				if( ch != *( pDelimiters + i ) )
				{
					bDone = TRUE;
					break;
				}
			}
			
			//end of string
			if( ch == 0 )
				break;

			//end of line
			if( ch == '\n' )
				break;

			if( !bDone )
				pString++;

		}

		CString sBuf;

		while( TRUE  )
		{
			ch = *pString;
			BOOL bDelimiter  = FALSE;
			for( i = 0; i < nDelimiters; i++ )
			{
				if( ch == *( pDelimiters +i ) || ch == 0 || ch == '\n' )
				{
					bDelimiter = TRUE;
					if( sBuf.GetLength( ) )
					{
						sBuf += "";
						pElements->Add( sBuf );
						sBuf = "";
					}
					break;
				}
			}
			
			if( ch == 0 )
				break;

			if( ch == '\n' )
				break;
			
			if( !bDelimiter )
				sBuf += ch;
			pString++;
		}

	}
	return( pElements->GetSize( ) );
}	


/////////////////////////////////////////////////////////////////////////////
// ParseRequest -- extract the http request from the string.

void CProxy::ParseRequest( CString *pRequest )
{
	int nElements;
	CStringArray sArray;

	m_sCommand = *pRequest;
	CString buf = *pRequest;

	nElements = ParseString( buf, " \r\n", &sArray );

	CString sBuf;
	m_sHostAddr = "";

	if(nElements == 3) 
	{
		sBuf = sArray.GetAt(0);

		//Check for ssl connect
		if( sBuf.CompareNoCase( "connect" ) == 0 ) 
		{
				m_bSSL = 1;
				m_sRequestLine = sBuf;
				m_sHostAddr = sArray.GetAt(1);
				m_sVersion = sArray.GetAt(2);
		} else {

			if( sBuf.GetLength( ) )	//1.2 
			{
					m_bSSL = 0;
					m_sRequestLine = sBuf;
					m_sUrl     = sArray.GetAt(1);
					m_sVersion = sArray.GetAt(2);
			
					LPCSTR p = m_sUrl;
					
					if( strnicmp( p, "http://", 7 ) == 0) 
					{
						m_sUrl = m_sUrl.Mid( 7 );
					} else if( strnicmp( p, "https://", 8 ) == 0) {
						m_sUrl = m_sUrl.Mid( 8 );
					} else {
						m_sUrl.Empty( );
					}
			}
		}


		 //extract the port
		 int i = m_sUrl.Find( '/' );

		 if( i >= 0 )
		 {
			m_sPath = m_sUrl.Mid( i );
			m_sUrl.SetAt( i, 0 );
			m_sHostAddr = m_sUrl;
		 }
	}


	if( m_sHostAddr.GetLength( ) == 0 )
		return;

	//extract the domain name and the optional port number
	nElements = ParseString( m_sHostAddr, ":", &sArray );

	//something is very wrong with the header, bail out
	if( nElements == 0 )
		return;

	m_nHostPort = 80;
	m_sDomain = sArray.GetAt( 0 );

	if( nElements == 2 ) 
		m_nHostPort = atoi( sArray.GetAt( 1 ) );

}

/////////////////////////////////////////////////////////////////////////////
// CheckHeader( ) -- Check for an empty line in the HTTP header.  Return 
// TRUE if one was found.
BOOL CheckHeader( CByteArray* pArray )
{
	int i = 0;
	int length = pArray->GetSize( );
	LPBYTE p = pArray->GetData();
	BOOL bFound = FALSE;
	BYTE ch;

	while( i < length )
	{
		ch = *( p + i );
		if( ch == LF )
			bFound++;
		else if( ch != CR )
			bFound = 0;

		if( bFound > 1 )
			return( TRUE );
		i++;
	}
	return( FALSE );
}



/////////////////////////////////////////////////////////////////////////////
//CreateHeader -- create a string array of headers return TRUE if success
//
// the bIsResponse parameter is TRUE if we are processing a response sent from
// the server and FALSE if it is a request from the browser.
BOOL CProxy::CreateHeader( CByteArray *pByteArray, CStringArray *pHeaders, BOOL bIsResponse /*= FALSE */ )
{
	CString sTmp;
	BOOL bDone = FALSE;

	int i;
	int nLength = pByteArray->GetSize();
	LPBYTE p = pByteArray->GetData( );
	pHeaders->RemoveAll( );

    //if processing the response header, default to non-text type
    if( bIsResponse )
        m_wMimeType = TYPE_IMAGE;
		 
	if( p )
	{
		for( i = 0; bDone == FALSE && i < nLength - 1; i++ )
		{
			//build a tmp string
			sTmp += (TCHAR)*( p + i );

			//check for end of line, and for the terminating empty line
			if( *( p + i ) == LF )
			{
				//create the header
				if( *(p + i + 1) == CR )
				{
					i++;
					sTmp += (TCHAR)*( p + i );
				}
				
				if( *(p + i + 1) == LF )
				{
					i++;
					sTmp += (TCHAR)*( p + i );
					bDone = TRUE;
					pByteArray->RemoveAt( 0, i + 1 );//remove the header from the string array
				}

				//examine the Content-Type: text/html
               
                BOOL bDontAdd = FALSE;
                CString s = sTmp;
				s.MakeLower( );

                //only set this stuff when sent from the server
                if( bIsResponse )
				{
					
					//if content-type is text/html or application/x-javascript or application/x-jscript then
					//set the flag to search the the current connection for javascript 
					if( s.Find( "content-type:" ) >= 0 ) 
					{
						
                        m_sMimeType = s;
                        m_sMimeType.TrimRight( );   //remove CR for viewer display
                        
                        //if( s.Find( "text/html" ) >= 0 )
                        if( s.Find( "text/" ) >= 0 )
                            m_wMimeType = TYPE_HTML;

                        if( s.Find( "image" ) >= 0 )
                            m_wMimeType = TYPE_IMAGE;


						if( s.Find( "javascript" ) >= 0 || s.Find( "jscript" ) >= 0 )
						{
                             m_wMimeType = TYPE_JAVASCRIPT;
						}
					}
                  
                    //http 1.1 Content-Length:   123456789012345
                    LPCSTR pLength = strstr( s, "content-length" ); 
                    if( pLength )
                    {
                        pLength += 16;
                        m_nContentLength = atoi( pLength );
                    }
                                                  
                    LPCSTR pEncoding = strstr( s, "transfer-encoding" );
                    if( pEncoding )
                    {
                        if( strstr( pEncoding, "chunked" ) )
                            m_bChunked = TRUE;
                    }   

                    #if 0
                    //the HTTP 1.1 spec states that the connection header should 
                    //not be forwarded.  However, since we only send the header 
                    //to the browser, and not to another proxy upstream, we should't
                    //remove the connection header.
                    if( strstr( s, "connection:" ) )
                    {
                        if( strstr( s, "close" ) )
                        {
                            //using bDontAdd causes videos to fail from Channel Guide
                            ///bDontAdd= TRUE;
                        }
                    }
                    #endif
                }
				
                //remove the gziped encoding option since CC does not support zip'ed data
                if( !m_theApp->m_bDisableCookieCop && !m_bSSL )
                {
                    if( strstr( s, "accept-encoding:" ) && strstr( s, "gzip" ) )
                        bDontAdd = TRUE;
                }

				#if 0
                //Older versions of CC had problems when connecting to caching proxies
                //However, it doesn't seem to be a problem when using HTTP 1.1 -- so this
                //section of code has been disabled.
                //
                //Remove header commands for Connection: and Proxy-Connection: 
				//required when connection to caching servers, like Squid -- 1.2
				static char SZ_CONNECTION[] = "Connection:";
				static char SZ_PROXY_CONNECTION[] = "Proxy-Connection:";
                //if( strstr( s, SZ_PROXY_CONNECTION ) )
                  //  bDontAdd = TRUE;
			    //if( strnicmp( sTmp, SZ_PROXY_CONNECTION, sizeof(SZ_PROXY_CONNECTION)-1 ) == 0 )  
                  //  bDontAdd = TRUE;
                #endif

                if( !bDontAdd )
                    pHeaders->Add( sTmp );

			    sTmp.Empty( );
			}	
		}	
	}		
    

    if( bIsResponse )       
    {
        if( m_wMimeType & TYPE_JAVASCRIPT || m_wMimeType & TYPE_HTML )
            m_wMimeType |= TYPE_TEXT;
    }

	return( bDone );
}


/////////////////////////////////////////////////////////////////////////////
//make sure the header ends with 0x0d 0x0a pairs before sending to host
void ValidateHeader( CString* p )
{
	char szTerminate[10];

	int iLen = p->GetLength( );
	if( iLen )
	{
		while( --iLen )
		{
			if( p->GetAt( iLen ) == LF  || p->GetAt( iLen ) == CR )
			{
				p->Delete( iLen );
				continue;
			} 
			break;

		}
		wsprintf( szTerminate, "%c%c%c%c", CR, LF, CR, LF );
		*p += szTerminate;
	}

	
}

/////////////////////////////////////////////////////////////////////////////
//CheckAcceptList -- return TRUE if the site is on the accept list for cookies
BOOL CheckAcceptList( CString sHost, CString* pPattern ) 
{
	int iLen = m_theApp->m_AcceptList.GetSize( );

	for( int i = 0; i < iLen; i++ )
	{
		if( MatchPattern( sHost, m_theApp->m_AcceptList.GetAt( i ) ) )
		{
			*pPattern = m_theApp->m_AcceptList.GetAt( i );
			return( TRUE );
		}	 

	}
	return( FALSE );
}

/////////////////////////////////////////////////////////////////////////////
//CheckRejectList -- return TRUE if the site is on the reject list for cookies
BOOL CheckRejectList( CString sHost, CString* pPattern ) 
{
	int iLen = m_theApp->m_RejectList.GetSize( );

	for( int i = 0; i < iLen; i++ )
	{
		if( MatchPattern( sHost, m_theApp->m_RejectList.GetAt( i ) ) )
		{
			*pPattern = m_theApp->m_RejectList.GetAt( i );
			return( TRUE );
		}	 

	}
	return( FALSE );

}


/////////////////////////////////////////////////////////////////////////////
//Scan the Browser and Server headers for cookie commands. 
//if the site is on the reject list remove the HTTP entry
//otherwise the entry is added to the Event Log
void CProxy::CheckCookies( CStringArray* pHeader, CString sHost )
{
	static char szSetCookie[] = {"Set-Cookie:"};	//server sends cookie
	#define SETCOOKIESIZE sizeof( szSetCookie )-1

	static char szCookie[] = {"cookie:"};	//browser sends cookie
	#define COOKIESIZE sizeof( szCookie )-1
	
	if( m_theApp->m_bDisableCookieList || m_theApp->m_bDisableCookieCop )
		return;

	for( int i = 0; i < pHeader->GetSize( ); i++ )
	{
			if( _strnicmp( pHeader->GetAt(i), szSetCookie, SETCOOKIESIZE ) == 0 )
			{
				//first check the accept list
				if( m_bInAcceptList )
                {
					AddEvent( EVENT_COOKIE_ACCEPTED, sHost, pHeader->GetAt(i), m_sPattern );
					continue;
				}

				//now check the reject list, if reject is the default then just remove it
				if( m_theApp->m_bDefaultAction == REJECT_COOKIE || m_bInRejectList )
				{
					AddEvent( EVENT_COOKIE_REJECTED, sHost, pHeader->GetAt(i), m_sPattern );
					pHeader->RemoveAt( i-- );	//could add wafer here!!!!!!!!!
					continue;
				}

			  	if( m_theApp->m_bDefaultAction == SESSION_COOKIE )
				{
					CString stmp;
					stmp = pHeader->GetAt(i);
					LPSTR p = stmp.GetBuffer( 0 );
					
					if( ConvertToSessionOnly( p,NULL, TRUE ) )
					{
						pHeader->SetAt( i, stmp );  //replace with the updated line
					}  else {
                        //add entry to log, even though it was already a session cookie
                        AddEvent( EVENT_SESSION_COOKIE, sHost, p, "" ); 
                    }
					stmp.ReleaseBuffer( -1 );
					continue; 
				}

				//default action is to accept the cookie
				AddEvent( EVENT_COOKIE_ACCEPTED, sHost, pHeader->GetAt(i), "" );

			} else { 
				if( _strnicmp( pHeader->GetAt(i), szCookie, COOKIESIZE ) == 0 )
				{
					//check accept list first
					if( m_bInAcceptList )
					{
						AddEvent( EVENT_COOKIE_SENT, sHost, pHeader->GetAt(i), m_sPattern );
						continue;
					}

					if( m_theApp->m_bDefaultAction == REJECT_COOKIE || m_bInRejectList )
					{
						AddEvent( EVENT_COOKIE_BLOCKED, sHost, pHeader->GetAt(i), m_sPattern );
						pHeader->RemoveAt( i-- );
						continue;
					} 
					
					//default action is to accept
					AddEvent( EVENT_COOKIE_SENT, sHost, pHeader->GetAt(i), "" );	
					
				}
			
			}
		}

}

/////////////////////////////////////////////////////////////////////////////
//Build the string array of headers into a single string for transfer
void FormatHeader( CStringArray* pHeaders, CString *p )
{
	p->Empty( );
	
	int iLen = pHeaders->GetSize( );

	for( int i = 0; i < iLen; i++ )
	{
		*p += pHeaders->GetAt( i );
	}

}

///////////////////////////////////////////////////////////////////////////////////////
//This section is for site blocking. 
BOOL CProxy::IsSiteBlocked( )
{
BYTE sz[1024];

static char SZ_BLOCKED_HTM[] = "HTTP/1.0 200 OK\n"
		 "Pragma: no-cache\n"
		 "Last-Modified: Fri Aug 10, 1999 12:00:00 am GMT\n"
		 "Expires: Fri Jan 15, 1999 12:00:00 am GMT\n"
		 "Connection: close\n"
		 "Content-Type: text/html\n\n";

static char SZ_BLOCKED_IMAGE[] = "HTTP/1.0 200 OK\n"
		 "Content-Type: image/%s\n"				//<- the extension is filled in
		 "Last-Modified: Fri Jan 15, 1999 00:00:00 GMT\n"
		 "Expires: Fri 15 Apr 1999 20:00:00 GMT\n"
		 "Pragma: no-cache\n"
		 "Connection: close\n"
		 "Content-Length: %lu\n\n";	



	if( m_theApp->m_bDisableBlockedList || m_theApp->m_bDisableCookieCop )
		return FALSE;

	//first check blocked site
    BOOL bUseBlockedSite = TRUE;    //select which file to use
	int iBlocked = InBlockedList( m_sDomain, &m_theApp->m_BlockedSiteList );
	if( iBlocked == -1 )
		iBlocked = InBlockedList( m_sCommand, &m_theApp->m_BlockedSiteList  );
	
	if( iBlocked != -1 )
		AddEvent( EVENT_SITE_BLOCKED, m_sDomain, m_sBrowserHeaders.GetAt( 0 ), m_theApp->m_BlockedSiteList.GetAt( iBlocked ) ); 

	//check blocked images
	if( iBlocked == -1 )	
	{
		CString stmp;
		BOOL bImage;
		stmp = m_sCommand;
		stmp.MakeLower( );
        //unfortunately, the browser does not specify a mime type so search for extensions
		bImage = ( strstr( stmp, ".gif" ) ||
				   strstr( stmp, ".cgi" ) ||
				   strstr( stmp, ".jpg" ) ||
				   strstr( stmp, ".jpe" ) ||
				   strstr( stmp, ".bmp" ) ||
				   strstr( stmp, ".pcx" ) ||
				   strstr( stmp, ".art" ) ||
				   strstr( stmp, ".cgm" ) ||
				   strstr( stmp, ".xbm" ) ||
				   strstr( stmp, ".g3f" ) ||
				   strstr( stmp, ".tif" ) ||
				   strstr( stmp, ".png" ) );

		if( bImage )
		{
			bUseBlockedSite = FALSE;
            
			iBlocked = InBlockedList( m_sDomain, &m_theApp->m_BlockedImageList  );
			if( iBlocked == -1 )
				iBlocked = InBlockedList( m_sCommand, &m_theApp->m_BlockedImageList   );					

			if( iBlocked != -1 )
				AddEvent( EVENT_IMAGE_BLOCKED, m_sDomain, m_sBrowserHeaders.GetAt( 0 ), m_theApp->m_BlockedImageList.GetAt( iBlocked ) ); 

		}
	}			
	
	
	if( iBlocked != -1 )
	{
        LPCSTR pFile;
        CByteArray* pFileData;

        if( bUseBlockedSite ) {
            pFile = m_theApp->m_sBlockedSiteFile;
            pFileData = &m_theApp->m_BlockedSiteFileData; 
        } else {
            pFile = m_theApp->m_sBlockedImageFile;
            pFileData = &m_theApp->m_BlockedImageFileData;
        }

        if( pFileData->GetSize( ) < 1 )
            return FALSE;
        
        //determine html or image
		CString stmp = pFile;
		stmp.MakeLower( );
		if( strstr( stmp, ".htm" ) )
		{
			AddToBrowserBuffer((LPBYTE)SZ_BLOCKED_HTM, strlen(SZ_BLOCKED_HTM));
	
		} else {
			//get the image extension from the selected image to block with
			LPCSTR p = strrchr( stmp, '.' );
			if( p )
				p++;
			else
				p = "gif";

			if( strstr( p, "jpg" ) )
				p = "jpeg";


            //create the header
			wsprintf( (LPSTR)sz, SZ_BLOCKED_IMAGE, p, pFileData->GetSize( ) ); 
			AddToBrowserBuffer( sz, strlen( (LPSTR)sz ) );
		}
				
		AddToBrowserBuffer( pFileData->GetData( ), pFileData->GetSize( ) ); 
        Send_Data( m_hBrowserSocket, &m_BrowserBuf, -1 );

		//shut down the connection
		CloseSockets( );
		return TRUE;
	}
		
	return( FALSE );
}


/////////////////////////////////////////////////////////////////////////////
//returns index or -1 if not blocked
int CProxy::InBlockedList( LPCSTR pDomain, CStringArray* pBlockedList )
{
	int iLen = pBlockedList->GetSize( );
	int i;

	for( i=0; i < iLen; i++ )
	{
		if( MatchPattern( pDomain, pBlockedList->GetAt( i ) ) )
		{
			return( i );
		}
	}

	return( -1 );
}



/////////////////////////////////////////////////////////////////////////////
//Examine the html code for JavaScript commands to set a cookie
//returns TRUE if the cookies have been removed
BOOL CProxy::CheckJavaScriptCookies( CByteArray *p )
{
BOOL bSendEvent = FALSE;
BOOL bRejectCookie;
BOOL bCookieRemoved = FALSE;

	//if disabled then just exit	
	if( m_theApp->m_bDisableCookieList || m_theApp->m_bDisableCookieCop )
		return FALSE;
	
	//kill java script document.cookie
	if( m_theApp->m_bSearchJavaScript && m_wMimeType & TYPE_TEXT )
	{
		//are cookies accepted from this site?
		if( m_bInAcceptList )
		{
			bRejectCookie = FALSE;
		} else	{
			bRejectCookie = ( m_theApp->m_bDefaultAction == REJECT_COOKIE || m_bInRejectList );	
		}
	
		int iLen = p->GetUpperBound( );
		if( iLen < 1 )
			return FALSE;

		LPSTR pCookie = (LPSTR)p->GetData( );
		LPSTR pSave = pCookie;
		LPSTR pTmp;
		BYTE chSave = pCookie[iLen];
		pCookie[iLen] = 0;

		
		#define SZ_COOKIE_TEXT ".cookie"
		#define DOC_COOKIE_LEN sizeof( SZ_COOKIE_TEXT ) - 1
		
		while( pCookie = strstr( pCookie, SZ_COOKIE_TEXT ) )
		{
		    //check for buffer overrun
		    if( ( pCookie - pSave ) > iLen )
			    break;

            bSendEvent = FALSE;

			//search for the equals sign
			pTmp = pCookie + DOC_COOKIE_LEN;
			while( *pTmp )
			{
				if( *pTmp == '=' && *(pTmp+1) != '=' ) 	  //Check for comparison == 
				{
					if( IsJavaScript( pSave, pCookie - pSave ) )
					{
						if( bRejectCookie )
                            *pTmp = '+';  //remove the assignment
						bSendEvent = TRUE;
					}
					break;
				} else {
					if( *pTmp > ' ' ) {	//not an assignment, get out
						break;
					}
				}
				pTmp++;
			}				
			
            //Grab the cookie text for displaying in the event log.  Search backwards for space.
            if( bSendEvent )
            {
			    LPSTR pStart = pCookie;
			    int i = 0;
			    char buf[512];	//max chars to copy
			    while( TRUE )
			    {
				    char ch = *(--pStart);
				    if( ch <= ' ' || pStart == pSave )
					    break;					
			    }
			    while( TRUE )
			    {
				    char ch = *(++pStart);
				    if( ch < ' ' || ( i >=  sizeof( buf )-1 ) )
					    break;					
				    buf[i++] = ch;
			    }
			    
                ASSERT( i < sizeof(buf) );
                buf[i] = 0;


                if( bRejectCookie == FALSE )
                {
                    AddEvent( EVENT_COOKIE_JS_ACCEPTED, m_sDomain, buf, "" ); 
                    break;   //no need to search further
                } else {
			        //the cookie has been disabled
                    AddEvent( EVENT_COOKIE_JS, m_sDomain, buf, m_sPattern ); 
                    bCookieRemoved = TRUE;
                }
            }
           
            pCookie += DOC_COOKIE_LEN; //keep searching

        }   //end while
		
        pSave[iLen] = chSave;
    }

    return bCookieRemoved;
}

/////////////////////////////////////////////////////////////////////////////
// 
void CProxy::AddToBrowserBuffer( LPBYTE p, int nLength )
{
	int iEnd = m_BrowserBuf.GetUpperBound( ) + 1;
	m_BrowserBuf.SetSize( iEnd + nLength );
	LPBYTE lp = m_BrowserBuf.GetData( );
	if( lp )
		memmove( lp+iEnd, p, nLength );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CProxy::AddToHostBuffer( LPBYTE p, int nLength )
{
	int iEnd = m_HostBuffer.GetUpperBound( ) + 1;
	m_HostBuffer.SetSize( iEnd + nLength );
	LPBYTE lp = m_HostBuffer.GetData( );
	if( lp )
		memmove( lp+iEnd, p, nLength );
}

/////////////////////////////////////////////////////////////////////////////
// AddEvent( ) -- Add an event to the event log
void CProxy::AddEvent( EVENT_TYPE event, LPCSTR pURL, LPCSTR pCommand, LPCSTR pPattern )
{
	if( m_theApp->m_bDisableLog )
        return;

    m_theApp->m_criticalSectionLog.Lock( );
	m_theApp->m_pLog->AddEvent( event, pURL, pCommand, pPattern ); 
	m_theApp->m_criticalSectionLog.Unlock( );
}


/////////////////////////////////////////////////////////////////////////////
// 
void CProxy::UpdateViewerStatusBar( WPARAM wParam, LPARAM lParam )
{
	if( ( m_theApp->m_pViewerDlg ) )
		 m_theApp->m_pViewerDlg->PostMessage( MSG_UPDATE_STATUSBAR, wParam, lParam );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CProxy::Viewer( VIEWER_EVENT event, LPCSTR pEvent, LPCSTR buf, int nBytes )
{
    if( !m_theApp->m_pViewerDlg ) 
        return;
   
    if( nBytes < 0 )
        return;

    try
    {
	
		//need to lock the viewer so that it won't be called with multiple threads ( intermixes the output )
        m_theApp->m_criticalSectionViewer.Lock( );

		//add exception handling in case user closes the viewer window
		//while we are updating
		CString s;

		if( nBytes )	
			s.Format("\nConnection: %u -- %s %u bytes\n", m_nSessionID, pEvent, nBytes );
		else
			s.Format("\nConnection: %u -- %s ", m_nSessionID, pEvent );

		m_theApp->m_pViewerDlg->AddText( (VIEWER_EVENT)(VIEWER_MSG | event), s );

		if( nBytes )
		{
			//truncate binary data -- but allow for HTTP header
			if( ( m_wMimeType & TYPE_TEXT ) == 0 )
			{
				int i = 0;
				while( i < nBytes /* && i < 500 */ )
				{
					if( buf[i] < 9 || buf[i] > 126 )
						break;
					i++;
				}
				nBytes = i;
			}

			
			//format string for Viewer
            LPSTR p = s.GetBufferSetLength( nBytes + 3 );
			strncpy( p, buf, nBytes  );	
			p[nBytes] = 0;
			strcat( p, "\n" );
			s.ReleaseBuffer( -1 );
			m_theApp->m_pViewerDlg->AddText( event, p );
		}
        m_theApp->m_criticalSectionViewer.Unlock( );
    }
    
    catch(...)
    {
		DebugOut( "Exception in CProxy::UpdateViewerStatusBar" );
    }

}



/////////////////////////////////////////////////////////////////////////////
//Scan the host buffer for meta tag cookies
void CProxy::CheckMetaCookies( CByteArray *p )
{
BOOL bRejectCookie;

	//if disabled then just exit	
	if( m_theApp->m_bDisableCookieList || m_theApp->m_bDisableCookieCop || ( m_wMimeType & TYPE_HTML == 0 ) )
		return;

	//search for <meta http-equv = "Set-Cookie" 
	#define SZ_META_TEXT "Set-Cookie"
	#define META_TEXT_LEN sizeof( SZ_META_TEXT ) - 1

	//are cookies accepted from this site?
	if( m_bInAcceptList )
	{
		bRejectCookie = FALSE;
	} else	{
		bRejectCookie = ( m_theApp->m_bDefaultAction == REJECT_COOKIE || m_bInRejectList );	
	}

	int iLen = p->GetUpperBound( );
	if( iLen < 1 )
		return;
	LPSTR pCookie = (LPSTR)p->GetData( );
	LPSTR pSave = pCookie;

	BYTE chSave = pCookie[iLen];
	pCookie[iLen] = 0;

	while( pCookie = strstr( pCookie, SZ_META_TEXT ) )
	{
		UINT offset = pCookie - pSave;
		
		if( IsMetaTag( offset ) )
		{
			LPSTR pStart = pCookie;
			//find start of tag
			while( pStart > pSave )
			{
				char ch = *pStart;

				if( ch == '<' )
					break;

				if( ch < ' ' )
				{
					pStart = NULL;
					break;
				}
				pStart--;
			}

			if( !pStart )
				break;

			LPSTR pEnd = pCookie;
			while( *pEnd )
			{
				char ch = *pEnd++;
				if( ch == '>' )
					break;
				if( ch < ' ' )
				{
					pEnd = NULL;
					break;
				}
			}
			
			if( pStart && pEnd )
			{
				int length = pEnd - pStart;
				ASSERT( length > 0 && length < (pEnd - pSave ) );

				if( length > 0 )
				{
					LPSTR pBuf = (LPSTR)malloc( length + 1 );
					if( pBuf )
					{
						memcpy( pBuf, pStart, length );
						pBuf[length] = 0;
                        
                        if( m_theApp->m_bDefaultAction == SESSION_COOKIE && !m_bInRejectList && !m_bInAcceptList )
                        {
                            ConvertToSessionOnly( pBuf, NULL, TRUE );
                            memcpy( pStart, pBuf, length );

                        } else {
						    if( bRejectCookie )
							    AddEvent( EVENT_COOKIE_REJECTED, m_sDomain, pBuf, m_sPattern ); 
						    else
							    AddEvent( EVENT_COOKIE_ACCEPTED, m_sDomain, pBuf, m_sPattern ); 
                        
						    if( bRejectCookie )
						    {
							    for( int i = 0; i < META_TEXT_LEN; i++ )
								    *(pCookie + i) = ' ';
						    }
                        }

                       free( pBuf );
					}

			   }
			
			}	

		}
		pCookie += META_TEXT_LEN;
		
	}//end while		

	pSave[iLen] = chSave;

}		


/////////////////////////////////////////////////////////////////////////////
// Searches both HTTP headers and JavaScript commands for cookie commands 
// that contain the expires =  command
// Only called if the default action is to convert the cookie into session only
// and if the site in not in the accept list
BOOL CProxy::ConvertToSessionOnly( LPSTR pCookieText, LPSTR pLowercase /*LPSTR pLowercase = NULL*/, BOOL bInHeader /*BOOL bInHeader = FALSE*/  )
{
    BOOL bFoundExpires = FALSE;
    LPSTR pText = NULL;

	//if disabled then just exit	
	if( m_theApp->m_bDisableCookieList || m_theApp->m_bDisableCookieCop || m_bInAcceptList )
		return FALSE;

    ASSERT( m_theApp->m_bDefaultAction == SESSION_COOKIE );
	
	//kill java script document.cookie
	if( ( m_theApp->m_bSearchJavaScript && (m_wMimeType & TYPE_TEXT) ) || bInHeader )
	{

        //allocate a buffer if necessary
        if( !pLowercase )
        {
		    int iLen = strlen( pCookieText );
		    if( iLen < 1 )
			    return FALSE;

		    pText = (LPSTR)malloc( iLen + 1 );
		    if( pText == NULL )
			    return FALSE;
		    strncpy( pText, pCookieText, iLen );
		    pText[iLen] = 0;
		    strlwr( pText );
        } else {
            pText = pLowercase;
        }

		LPSTR  pCookieSearch = pText;
		LPCSTR pSave = pCookieSearch;
		
		#define SZ_EXPIRES_TEXT "expires"
		#define DOC_EXPIRES_LEN sizeof( SZ_EXPIRES_TEXT ) - 1
				
		while( pCookieSearch = strstr( pCookieSearch, SZ_EXPIRES_TEXT ) )
		{
			//search for =
			pCookieSearch += DOC_EXPIRES_LEN;	//move past 'expires'
			LPSTR pEquals = pCookieSearch;
		
			char ch;
			while( TRUE )
			{
				ch = *pEquals; 
				
				if( ch == '=' )
				{
		            if( bInHeader || IsJavaScript( pSave, pEquals-pSave ) )
					{
						//do an extra check for text preceding the expires string
                        //it must contain either a ; or a quote to be valid
                        LPSTR pTmp = pCookieSearch;
                        BOOL bOk = FALSE;
                        while( --pTmp != pSave )
                        {
                            ch = *pTmp;
                            if( ch < ' ' )
                                break;

                            if( ch == '"' || ch == '\'' || ch == ';' )
                            {
                                bOk = TRUE;
                                break;
                            }
                        }

                        if( bOk )
                        {                        
                            int offset = pEquals - pSave;
						    *(pCookieText + offset) = '+';
						    bFoundExpires = TRUE;
						    pCookieSearch = pEquals;
                        }
					}
					break;
				}

				//search for the end of the expires = statement
				if( ch == ';' || ch == '<' || ch < ' ' )
					break;
				pEquals++;

			}
		}

        if( !pLowercase )
            free( pText );
	}

    if( bFoundExpires )
    {
        if( bInHeader == FALSE )
        {
            AddEvent( EVENT_SESSION_COOKIE, m_sDomain, "JavaScript", "" );
        } else {		
            AddEvent( EVENT_SESSION_COOKIE, m_sDomain, pCookieText, "" );
		}

        CString s;
		s.Format( "Converted cookie to session-only from %s", m_sDomain );
		Viewer( VIEWER_CC, s, "", 0 );
    }

	return( bFoundExpires );
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CProxy::IsMetaTag( UINT nOffset )
{
    CString s;
    int nXMP = 0;
    int nMetaTag = 0;
    int i = 0;

    int nArrayLength = m_sTagArray.GetSize( );

    while( i < nArrayLength )
    {
        
        if( m_nTagArrayOffset[i] > nOffset )
            break;
        
        s = m_sTagArray[i++];


		#ifdef _DEBUG
		Viewer( VIEWER_CC, s, "",0  );
		#endif

        if( s == "<xmp>" )
            nXMP++;
        if( s == "</xmp>" )
            nXMP--;

        if( s.Find( "<meta" ) == 0 )
            nMetaTag++;
        if( s.Find( "</meta" ) == 0 )
            nMetaTag--;

    }

	if( nXMP > 0 )
        return FALSE;

    return(BOOL)( nMetaTag > 0 );

}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CProxy::IsJavaScript( LPCSTR pBuf, UINT nOffset, BOOL* pBodyTag /* = NULL*/ )
{
    CString s;
    int nXMP = 0;
    int nPRE = 0;
    int nCODE = 0;
    int nBodyTag = 0;
    int nText = 0;
    int nJavaScript = 0;
    int i = 0;

    if( m_wMimeType & TYPE_JAVASCRIPT ) 
        return( TRUE );

    int nArrayLength = m_sTagArray.GetSize( );

    while( i < nArrayLength )
    {
        s = m_sTagArray[i];

        if( m_nTagArrayOffset[i] > nOffset )
            break;
       
        do
        {
            nBodyTag = 0;
            if( s.Find( "<xmp")  == 0)   { nXMP++; break; }
            if( s.Find( "</xmp") == 0)   { nXMP = max( 0, nXMP -1); break; }
            if( s.Find( "<pre")  == 0)   { nPRE++; break; }
            if( s.Find( "</pre") == 0)   { nPRE = max( 0, nPRE -1); break; }
            if( s.Find( "<cod")  == 0)   { nCODE++; break; }
            if( s.Find( "</cod") == 0)   { nCODE = max( 0, nCODE-1); break; }
            if( s.Find( "<bod")  == 0)   { nBodyTag++; break; }
            if( s.Find( "</bod") == 0)   { nBodyTag = max( 0, nBodyTag-1); break; }
            if( s.Find( "<tex")  == 0)   { nText++; break; }
            if( s.Find( "</tex") == 0)   { nText = max( 0, nText-1); break; }

            //window.media and other sites use
            //document.write('</SCR'+'IPT>'); to fake out AOL apparently
            //not much we can do
            if( s.Find( "<scr")  == 0)  {nJavaScript++; break; }
            if( s.Find( "</scr") == 0)  {nJavaScript = max( 0, nJavaScript - 1); break; }
        } while( 0 );
        
        i++;
       
    }
	
    
	if( nText > 0 || nXMP > 0 || nCODE > 0 || nPRE > 0 )
        return FALSE;   //not in javascript


    if( pBodyTag )
        *pBodyTag = nBodyTag; //return check for <BODY> tag


    return(BOOL)( nJavaScript > 0 );

}


/////////////////////////////////////////////////////////////////////////////
//returns zero if failure or end of buffer else returns lowercase tag in sTag and 
//the new search position
BOOL CProxy::MakeTag( LPCSTR pBuf, int nLength, int iStart,  CString& sTag )
{
    int iEnd = 0;
    
    while( (iStart + iEnd) <= nLength )
    {
        if( pBuf[iStart + iEnd] == '>' )
        {
            iEnd++;
            LPSTR p = sTag.GetBuffer( iEnd );
            memcpy( p, pBuf+iStart, iEnd  );
            sTag.ReleaseBuffer( iEnd );
            sTag.MakeLower( );
            return( iStart + iEnd );
        }
        iEnd++;
    }

    return( 0 );
}


/////////////////////////////////////////////////////////////////////////////
// 
int CProxy::FindEndScript( LPCSTR pBuf, int nLength, int i )
{
    //if we are in a <SCRIPT> tag then
    //search until we find the </SCRIPT>
    //this eliminates comment tags and having to check for 
    //greater than and less than operators in the code

    CString sTag;

    while( pBuf[i] && i < nLength )
    {
        if( pBuf[i] == '<' && pBuf[i+1] == '/' )
        {
            int j = MakeTag( pBuf, nLength, i, sTag );
            if( j )
            {
                //if( sTag == "</script>" )
                if( strncmp( "</scr", sTag, 5 ) == 0 )
                {
                    m_sTagArray.Add( sTag );            
                    m_nTagArrayOffset.Add( i );//offset in buffer to the END of the tag
                    return( j );
                }
            } 
        }
        i++;
    }
    return( i );
}


/////////////////////////////////////////////////////////////////////////////
//GetTags -- This code scans the text received from the server for tags that 
//indicate scripts, meta commands, or the body tag.   All other tags are ignored.
//
//The offset of each tag is stored in m_nTagArrayOffset and the tag string in m_sTagArray.
//When CC needs to block an item, such as the 'open' command for pop-ups, the offset of
//the 'open' command is compared to the offsets of the HTML tags and a determination is
//made if the 'open' text is really a JavaScript command, or is just ordinary text on the
//web page.  By comparing just the offsets, CC can quickly determine if the text is within 
//a <script>...</script> section and does not need to parse the entire HTML code.  The tag
//arrays only need to be built once for each page, and are used when blocking referrer, cookie, 
//meta, and pop-up windows.

void CProxy::GetTags( LPCSTR pBuf, int nLength )
{
	CString sTag;
    int i = 0;
	
    //pBuf must be lowercase
	ASSERT( pBuf );

    while( pBuf[i] && i < nLength )
    {
        if( pBuf[i] == '<' )
        {

            int start = i;
            i = MakeTag( pBuf, nLength, i, sTag );

            if( i )
            {
                //only add tags of interest
                static char szTags[][8]= {  "<scr", "</scr",
                                            "<xmp", "</xmp",  
                                            "<met", "</met", 
                                            "<pre", "</pre", 
                                            "<code","</code",
                                            "<text","</text", //textarea
                                            "<body", "" };
                
                BOOL bAdd = FALSE;
                int tag = 0;
                while( !bAdd && szTags[tag][0] )
                {
                    if( strncmp( sTag, szTags[tag], 4 ) == 0 ) 
                        bAdd = TRUE;
                    tag++;
                }

                if( bAdd )
                {
                    if( strncmp( sTag, "<bod", 4 ) == 0 )
                    {
                        m_sTagArray.Add( sTag );
                        m_nTagArrayOffset.Add( start );//offset in buffer to the START of the tag
                        m_sTagArray.Add( "</body>" );
                        m_nTagArrayOffset.Add( i );//offset in buffer to the END of the tag

                    } else {
                        m_sTagArray.Add( sTag );
                        m_nTagArrayOffset.Add( i );//offset in buffer to the END of the tag
                    }

                    //if within script code then skip to end of the script tag
                    if( strncmp( sTag, "<scr", 4 ) == 0 )
                    {
                        i = FindEndScript( pBuf,  nLength, i );
                    }
                }
            
            } else {
                return; //end of buffer
            }    
        } else {
            i++;
        }
    }
    
#ifdef _DEBUG
int l = m_sTagArray.GetSize( );
char ss[5102];
Viewer( VIEWER_DEBUG, "\n\nDUMPING TAGS", 0,0 );
Viewer( VIEWER_DEBUG,     "============", 0,0 );
for( i = 0; i < l; i++ )
{
    wsprintf( ss, "%6u %s", m_nTagArrayOffset[i], m_sTagArray[i] );
    Viewer( VIEWER_DEBUG, ss, 0,0 );
}
#endif    
    
    return;
}

/////////////////////////////////////////////////////////////////////////////
//Remove referrer HTTP header commands.
//Does not remove referer if making a secure connection
BOOL CProxy::CheckReferer( )
{
    
    static char SZ_REFERER[] = "Referer:";
    CString sTmp;
    CStringArray sNewArray;
    BOOL bCopyArray = FALSE;
    int nLength = m_sBrowserHeaders.GetSize( );

	//if disabled then just exit	
	if( m_theApp->m_bDisableCookieCop || !m_theApp->m_bCheckReferer )
		return FALSE;

    LPSTR pDomain = (LPSTR)(LPCSTR)m_sDomain;
    if( !pDomain )                 
        return FALSE;

        
    for( int i = 0; i < nLength; i++ )
    {
        sTmp = m_sBrowserHeaders.GetAt( i );

        if( strnicmp( sTmp, SZ_REFERER, sizeof(SZ_REFERER)-1 ) == 0 ) 
        {
            //search for the domain string within the referrer field
            if( strstr( sTmp, pDomain ) == 0 )
            {
                CString s = m_sBrowserHeaders.GetAt( 0 );   //show both the request line
                s = s + sTmp;                                //and the Referer tag
                AddEvent( EVENT_REFERER_REMOVED, m_sDomain, s, "" );
            
                s.Format( "Removed cross site referer field.   %s", sTmp );
                Viewer( VIEWER_CC, s, "", 0 );
                bCopyArray = TRUE;          //flag to rebuild the header array     
                m_bCheckJSReferrer = TRUE;  //flag to scan javascript referrer tags
                continue;
            }

        }
        sNewArray.Add( sTmp );
    }

    if( bCopyArray )
    {
        m_sBrowserHeaders.Copy( sNewArray );
    }        

    return( bCopyArray );    
}


/////////////////////////////////////////////////////////////////////////////
//Examine the html code for referrer commands
void CProxy::CheckJavaScriptReferrer( CByteArray *p )
{

	//always check javascript referrers
	//if( !m_bCheckJSReferrer )
	//	return;

    //if disabled then just exit	
	if(  !m_theApp->m_bCheckReferer || m_theApp->m_bDisableCookieCop )
		return;

	
	//kill document.referrer
	if( m_wMimeType & TYPE_TEXT )
	{
		int iLen = p->GetUpperBound( );
		if( iLen < 1 )
			return;

		LPSTR pDocument = (LPSTR)p->GetData( );
		LPSTR pSave = pDocument;
		BYTE chSave = pDocument[iLen];
		pDocument[iLen] = 0;
		
		#define SZ_REFERRER_TEXT "document.referrer"
		#define REFERRER_LEN sizeof( SZ_REFERRER_TEXT ) - 1

		while( pDocument = strstr( pDocument, SZ_REFERRER_TEXT ) )
		{						
			//check for buffer overrun
			if( ( pDocument - pSave ) > iLen )
				break;
			
			//replace the document.referrer string to document.url 
            //so that it simply refers to itself
			if( IsJavaScript( pSave, pDocument - pSave ) )
            {
                char szBuf[512];
                LPSTR pStart, pEnd;
                pStart = pEnd = pDocument;
                //search for start of line
                while( TRUE )
                {
                    if( *( pStart - 1 ) < ' ' || pStart == pSave )
                        break;
                    pStart--;
                }
                //search for end of line
                while( TRUE )
                {
                    if( *pEnd < ' '  )
                        break;
                    pEnd++;
                }

                //this section formats the text for the Event Log
                int nCopy = min( sizeof(szBuf) - 1, pEnd - pStart );
                memcpy( szBuf, pStart, nCopy );
                szBuf[nCopy] = 0;
                AddEvent( EVENT_REFERRER_REMOVED_JS, m_sDomain, szBuf, "" ); 

                //CC fails to identify document.referrer as being text with this site
                //http://web-wise-wizard.com/javascript-reference/javascript-jscript-get-user-information.html
                //because the 'text' document.referrer is embedded in JavaScript and is used as a title
                                
                
                //Replace referrer with an uppercase R.  This does not generate a JavaScript
                //error and causes the .referrer object to return undefined
                #define DOC_LEN sizeof( "document." )-1
                memcpy( pDocument+DOC_LEN, "Referrer", 8 ); //replace the text
                                            
                
                //you could replace the .referrer text with .URL using the line below
                //however, some users may not understand the URL points to the current
                //page and not the referring site.
                //memcpy( pDocument+DOC_LEN, "URL     ", 8 ); //replace the text
            }
            
            pDocument += REFERRER_LEN;
	    }

	    //restore 
        pSave[iLen] = chSave;
    }	

}



/////////////////////////////////////////////////////////////////////////////
// Inject our JavaScript code into the host buffer
BOOL CProxy::DisablePopups( CByteArray *p, LPCSTR pBuf )
{
    BOOL ret = FALSE;

	//if disabled then just exit	
	if( m_theApp->m_bDisableCookieCop || ( ( m_wMimeType & TYPE_TEXT ) == 0) )
		return FALSE;

    if( m_theApp->m_bPopupCheckDisabled )
        return FALSE;


	//search for <head> or <html>
    #define SZ_HEAD_TEXT "<head"
    #define SZ_HTML_TEXT "<html"

	int iLen = p->GetUpperBound( );
	if( iLen < 1 )
		return FALSE;
	LPSTR pDocument = (LPSTR)p->GetData( );
    LPSTR pEnd = NULL;
    
    //http://www.vastvideo.com/i/2.2/
    //creates pop-ups using document.write and does not set mime_javascript

    
    int tag_length = 0;

    if(( m_wMimeType & TYPE_JAVASCRIPT ) == 0 )
    {

        //http://www.codeave.com/javascript/category.asp?u_category=Document+Info
        //search for <HEAD>
        if( pDocument = strstr( pBuf, SZ_HEAD_TEXT ) )
        {
            tag_length = sizeof( SZ_HEAD_TEXT )-1 ;
            if( (pDocument > pBuf ) && *(pDocument-1) == '\"' )
                tag_length = 0;

        } 

        if( tag_length == 0 )
        {
            //search for <HTML>
            if( pDocument = strstr( pBuf, SZ_HTML_TEXT ) )
            {
                tag_length = sizeof( SZ_HTML_TEXT )-1 ;
                if( (pDocument > pBuf ) && *(pDocument-1) == '\"' )    //need to add more checking for tags within quotes!
                    tag_length = 0;
            }

        }

        pEnd = pDocument;

        if( tag_length )	
	    {
		    while( *pEnd )
		    {
			    char ch = *pEnd++;
			    if( ch == '>' )
				    break;
							    
			    if( ch < ' ' )
			    {
				    pEnd = NULL;
				    break;
			    }
            }    
	    }

    }

    CByteArray* pJavaScript = &m_theApp->m_PopupFileData;

    // windows media sends javascript bodys but does not set the proper mime type
    if( tag_length == 0 )
    {       
        if( m_wMimeType & TYPE_JAVASCRIPT ) 
        {
            //insert code at the beginning of the page
            pEnd = pDocument = (LPSTR)pBuf;
            pJavaScript = &m_theApp->m_PopupFileDataJS;
        } else {

                //windows media uses a text mimetype when writing javascript code
                //we need to inject the header, search for script commands
                if( strstr( pBuf, "</script" )  )
                    pEnd = pDocument = (LPSTR)pBuf; 

                //bail out
                if( pDocument == NULL )
                    pEnd = NULL; 

        }
    }


    if( pEnd )
    {
        CByteArray byNewArray;
        UINT nSize = pJavaScript->GetSize( );

        if( nSize )
        {
            int nOldSize = p->GetSize( );
            byNewArray.SetSize( nSize + nOldSize  );
        
            LPBYTE pOldArray = p->GetData( );
            LPBYTE pNewArray = byNewArray.GetData( );

            //calculate where our js should be inserted
            int length = pEnd - pBuf ;
            memcpy( pNewArray, pOldArray, length );
            pNewArray += length;
            memcpy( pNewArray, pJavaScript->GetData(), nSize );
            pNewArray += nSize ;//- 1;                
            memcpy( pNewArray, pOldArray+length, nOldSize - length );

            p->RemoveAll( );
            p->Copy( byNewArray ); 
            ret = TRUE;

            #ifdef _DEBUG
            Viewer( VIEWER_CC, "Popup blocked HTML", "", 0  );           
            #endif

       } else {
        Viewer( VIEWER_CC, "nopopup.js file not found.", "", 0  );           
       }
    }

    return( ret );
}		


/////////////////////////////////////////////////////////////////////////////
//Removes the 'open' command from JavaScript.  The command is replaced with
//our CCop function to prompt the user to open popups. 
void CProxy::RemoveOpenFromHTML( LPSTR p, CByteArray* pArray )
{
	
    //search for .open  -- note that the text has already been converted to lower case
	#define SZ_OPEN_TEXT "open"
	#define OPEN_LEN sizeof( SZ_OPEN_TEXT ) - 1

    //if disabled then just exit	
	if( m_theApp->m_bDisableCookieCop || ( ( m_wMimeType & TYPE_TEXT ) == 0) )
		return ;

    if( m_theApp->m_nPopupMode == 1 )
        return ;    //no prompts 

    if( m_theApp->m_bPopupCheckDisabled )
        return ;

    LPSTR pOpen = p;
    const LPSTR pBufferStart = p;

    //search the host buffer for JavaScript open commands
	while( pOpen = strstr( pOpen, SZ_OPEN_TEXT ) )
	{
		UINT offset = pOpen - pBufferStart;

        //see http://www.coolfonts.de/index/index.html _open
        //check the byte before 'open', if it is not whitespace or a period skip it
        LPBYTE pTest = (LPBYTE)pOpen-1;
        if( *pTest > ' ' && *pTest != '.' )
        {
        	pOpen += OPEN_LEN;
            continue;
        }
		
        BOOL bBodyTag = FALSE; //we want to scan the <BODY> tags as well
        BOOL bJavaScript = IsJavaScript( pBufferStart, offset, &bBodyTag );

        //look for the function's closing paren
        if( bJavaScript || bBodyTag )
    	{
			LPSTR pEnd = pOpen + OPEN_LEN;
			while( *pEnd )
			{
				char ch = *pEnd++;
				if( ch == '(' )
					break;

				if( ch != ' ' )
				{
					pEnd = NULL;
					break;
				}
			}

            //see http://www.coolfonts.de/index/index.html
            //if the end of the open function was found then 
            //do some more checks
			if( pEnd )
			{     
                BOOL bDontChange = FALSE;

                //verify that the object calling the open command is either
                //window or document -- as in document.open( )
                int len = pOpen - pBufferStart;
                if( len > 9 )
                {
                    LPCSTR pTmp;
                    pTmp = pOpen - 1;
                    if( *pTmp == '.' )
                    {
                       
                        static char SZ_DOCUMENT[] = "document.";
                        static char SZ_WINDOW[]   = "window.";
                        
                        LPCSTR pDoc = pOpen - (sizeof(SZ_DOCUMENT)-1);
                        LPCSTR pWin = pOpen - (sizeof(SZ_WINDOW)-1);

                        bDontChange = TRUE;

                        //verify that it is document.open
                        if( memcmp( pDoc, SZ_DOCUMENT, sizeof(SZ_DOCUMENT)-1 ) == 0 )
                        {
                           if( *(pDoc-1) != '.' )
                                bDontChange = FALSE;
                        } 

                        //verify that it is window.open
                        if( memcmp( pWin, SZ_WINDOW, sizeof(SZ_WINDOW)-1 ) == 0 )
                        {
                            if( *(pWin-1) != '.' )
                                bDontChange = FALSE;
                        }
                    }
                }

                //replace the 'open' function with CCop
                //note that the buffer size does not change -> 4 bytes each
                if( !bDontChange )
                {
                    LPBYTE pOldArray = pArray->GetData( );
                    memcpy( pOldArray+offset, "CCop", 4);

                }
			}	

		}
		pOpen += OPEN_LEN;  //skip past the open text
		
	}//end while		
}		




/////////////////////////////////////////////////////////////////////////////
// Search the host data for chunk sizes
int CProxy::GetChunk( )
{
   //search for length
    int nChunk = -1;    //not found

    char*  pEnd;
    LPBYTE pChunk = m_HostBuffer.GetData( );
    LPBYTE pStart = pChunk;
    int nLengthHostBuffer = m_HostBuffer.GetSize( );
    int length;

    while( pChunk && *pChunk )
    {
        length = pChunk - pStart;
        if( length > nLengthHostBuffer )
            return -1;
        
        if( *pChunk > ' ' ) 
        {
            nChunk = strtol( (LPSTR)pChunk, &pEnd, 16 ); 

            if( ((LPBYTE)pEnd - pChunk ) > nLengthHostBuffer )
                return -1;
       
            //now read up to the new line
            while( ++pChunk )
            {
                if( *pChunk == 0x0a )
                    break;
            }

            while( ++pChunk ) //remove extra 0x0d 0x0a -- www.google.com groups (sends gzip'ed chunked HTML)
            {   
                length = pChunk - pStart;
                if( length >= nLengthHostBuffer )
                    break;

                if( nChunk == 0 )
                {

                char ch = *pChunk;
                if( ch != 0x0a && ch != 0x0d )
                    break;
                } else {
                    break;
                }
            }

            //make sure the buffer contains the whole chunk!!!
            length = pChunk - pStart;
            if( nLengthHostBuffer < ( nChunk + length ) )
                return -1;  //error

            m_HostBuffer.RemoveAt( 0, length );
            break;
        }
        pChunk++;
    }

    //return the size of the chunk, zero if it is the last chunk
    return nChunk;
}



/////////////////////////////////////////////////////////////////////////////
//Appends chunked data into the m_ChunkBuffer array 
BOOL CProxy::ProcessChunkedData( )
{
    int nNewLength = -1;

    while( TRUE )
    {
        nNewLength = GetChunk( );
                                       
        if( nNewLength > 0 )        //  -1 == not found 
        {
            if( m_nContentLength == -1 )
                m_nContentLength = 0;         //init
            
            m_nContentLength += nNewLength;  //append

            int nLengthChunkBuffer = m_ChunkBuffer.GetSize( );
            m_ChunkBuffer.SetSize( nLengthChunkBuffer + nNewLength );

            LPBYTE pHost = m_HostBuffer.GetData( );
            LPBYTE pChunk = m_ChunkBuffer.GetData( );
           
            memcpy( pChunk + nLengthChunkBuffer, pHost, nNewLength );
            m_HostBuffer.RemoveAt( 0, nNewLength );
      
            
        } else {
            break;
        }
    
    }
   
    //wait until all the chunked data has been received
    //so that you know what the content-length is
    if( nNewLength == 0 )   //found the end?
    {
            m_HostBuffer.Append( m_ChunkBuffer );
            m_ChunkBuffer.RemoveAll( );
            m_bChunked = 0;

            //fix the header array
            CString stmp;
            int iLen = m_sChunkedHeaders.GetSize( );
            for( int i = 0; i < iLen; i++ )
            {
                stmp = m_sChunkedHeaders.GetAt(i);
                stmp.MakeLower( );
                if( strstr( stmp, "transfer-encoding" ) )
                {
                    m_nHostBytes = m_nContentLength = m_HostBuffer.GetSize ( );
                    stmp.Format( "Content-Length: %u\n" ,m_nContentLength );
                    m_sChunkedHeaders.SetAt(i, stmp );
                    break;
                }
          }
            //copy to m_sHostHeaders so we can check for popups and other stuff
            m_sHostHeaders.RemoveAll( );
            m_sHostHeaders.Copy( m_sChunkedHeaders );
            m_sChunkedHeaders.RemoveAll( );
    }

    return( TRUE );
}


/////////////////////////////////////////////////////////////////////////////
// This function is called if we have inserted our popup JavaScript code into
// the HTML data.  The content-length header must be updated to reflect the 
// new size of the page.
void CProxy::FixContentLength( CStringArray* pHostHeaders, CByteArray* pHostBuffer )
{
    int n = pHostHeaders->GetSize( );
    CString stmp;
    BOOL    bDone = FALSE;
    
    for( int i = 0; i < n && !bDone; i++ )
    {
        stmp = pHostHeaders->GetAt( i );
        stmp.MakeLower( );
        if( strstr( stmp, "content-length:" )  )           
        {
            stmp.Format( "Content-Length: %u\n", pHostBuffer->GetSize( ) );
            pHostHeaders->SetAt( i, stmp );
            bDone = TRUE;
       }

    }    
}

/////////////////////////////////////////////////////////////////////////////
// Check browser request commands to see if the browser will send additional
// data with the request -- as in a POST.
// returns TRUE if the request does NOT send additional data.
BOOL CProxy::IsIdempotent( LPCSTR pHeader )
{
    static char szMethods[ ][7] =  {"GET", 
                                    "HEAD", 
                                    "PUT", 
                                    "DELETE",
                                    "" };
    
    
    if( pHeader )
    {    
        char szTmp[10];
        strncpy( szTmp, pHeader, sizeof( szTmp )-1 );
        szTmp[sizeof( szTmp )-1 ] = 0;
        strupr( szTmp );
        int i = 0;

        while( szMethods[i][0] )
        {
            if( strstr( szTmp, &szMethods[i][0] ) )
                return TRUE;
            i++;
        }

    }

    return( FALSE );

}
