/*=========================================================================*/
/*                           Registry.cpp                                  */
/*                    Various registry functions                           */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include "stdafx.h"
#include "Registry.h"

/////////////////////////////////////////////////////////////////////////////
// CRegistry


LONG CRegistry::Close()
{
	LONG lResult = ERROR_SUCCESS;
	if (m_hKey != NULL)
	{
		lResult = RegCloseKey( m_hKey );
		m_hKey = NULL;
	}
	return lResult;
}

LONG CRegistry::Create(HKEY hKeyParent, LPCTSTR lpszKeyName,
							LPTSTR lpszClass, DWORD dwOptions, REGSAM samDesired,
							LPSECURITY_ATTRIBUTES lpSecAttr, LPDWORD lpdwDisposition)
{
	_ASSERTE(hKeyParent != NULL);
	DWORD dw;
	HKEY hKey = NULL;
	LONG lResult = RegCreateKeyEx( hKeyParent, lpszKeyName, 0,
		lpszClass, dwOptions, samDesired, lpSecAttr, &hKey, &dw);

	if (lpdwDisposition != NULL)
		*lpdwDisposition = dw;

	if (lResult == ERROR_SUCCESS)
	{
		lResult = Close();
		m_hKey = hKey;
	}

	return lResult;
}

LONG CRegistry::QueryValueLength( DWORD& dwValue, LPCSTR lpszKeyName )
{
	_ASSERTE(m_hKey != NULL);
	
	DWORD dwType;
	LONG lRet = RegQueryValueEx( m_hKey,lpszKeyName, NULL, &dwType, NULL, &dwValue );

	if( lRet == ERROR_SUCCESS )
	{
		//not finished -- does not do all reg types
		switch( dwType )
		{
			case REG_DWORD:
				dwValue = dwValue * (sizeof(int));
				break;
		}

	} else {
		dwValue = 0;
	}

	return( lRet );
}		     

LONG CRegistry::Open( LPCTSTR lpszKeyName, REGSAM samDesired )
{
	return( Open( HKEY_CLASSES_ROOT, lpszKeyName, samDesired) );
}

LONG CRegistry::Open( HKEY hKeyParent, LPCTSTR lpszKeyName, REGSAM samDesired )
{
	_ASSERTE(hKeyParent != NULL);
	HKEY hKey = NULL;
	LONG lResult = RegOpenKeyEx(hKeyParent, lpszKeyName, 0, samDesired, &hKey);
	if (lResult == ERROR_SUCCESS)
	{
		lResult = Close( );
		_ASSERTE(lResult == ERROR_SUCCESS);
		m_hKey = hKey;
	}
	return lResult;
}

LONG CRegistry::EnumKey( DWORD dwIndex, LPTSTR lpszBuf, DWORD dwBuf )
{
	_ASSERTE( m_hKey != NULL );
	return( RegEnumKey( m_hKey, dwIndex, lpszBuf, dwBuf ) );

}

//reg_binary
LONG CRegistry::QueryValue( LPBYTE pByte, int nBytes, LPCTSTR lpszValueName )
{
	DWORD dwType = NULL;
	DWORD dwCount = nBytes;
	LONG lResult = RegQueryValueEx( m_hKey,lpszValueName, NULL, &dwType, pByte, &dwCount);
	
	_ASSERTE(lResult == ERROR_SUCCESS);
	
	return lResult;
}

LONG CRegistry::QueryValue( DWORD& dwValue, LPCTSTR lpszValueName )
{
	DWORD dwType = NULL;
	DWORD dwCount = sizeof(DWORD);
	LONG lResult = RegQueryValueEx(m_hKey, (LPTSTR)lpszValueName, NULL, &dwType,
		(LPBYTE)&dwValue, &dwCount);
	_ASSERTE((lResult!=ERROR_SUCCESS) || (dwType == REG_DWORD));
	_ASSERTE((lResult!=ERROR_SUCCESS) || (dwCount == sizeof(DWORD)));
	return lResult;
}

LONG CRegistry::QueryValue( LPTSTR lpValueName, LPDWORD dwType, PBYTE lpData, LPDWORD lpcbData )
{
	UNREFERENCED_PARAMETER( dwType );

	return( RegQueryValue( m_hKey, lpValueName, (LPSTR)lpData, (PLONG)lpcbData ) );
}


LONG CRegistry::QueryValue(LPTSTR szValue, LPCTSTR lpszValueName, DWORD* pdwCount)
{
	_ASSERTE(pdwCount != NULL);
	DWORD dwType = NULL;
	LONG lResult = RegQueryValueEx(m_hKey, (LPTSTR)lpszValueName, NULL, &dwType,
		(LPBYTE)szValue, pdwCount);
	_ASSERTE((lResult!=ERROR_SUCCESS) || (dwType == REG_SZ) ||
			 (dwType == REG_MULTI_SZ) || (dwType == REG_EXPAND_SZ));
	return lResult;
}

LONG WINAPI CRegistry::SetValue(HKEY hKeyParent, LPCTSTR lpszKeyName, LPCTSTR lpszValue, LPCTSTR lpszValueName)
{
	_ASSERTE(lpszValue != NULL);
	CRegistry key;
	LONG lResult = key.Create(hKeyParent, lpszKeyName);
	if (lResult == ERROR_SUCCESS)
		lResult = key.SetValue(lpszValue, lpszValueName);
	return lResult;
}

LONG CRegistry::SetKeyValue(LPCTSTR lpszKeyName, LPCTSTR lpszValue, LPCTSTR lpszValueName)
{
	_ASSERTE(lpszValue != NULL);
	CRegistry key;
	LONG lResult = key.Create(m_hKey, lpszKeyName);
	if (lResult == ERROR_SUCCESS)
		lResult = key.SetValue(lpszValue, lpszValueName);
	return lResult;
}

LONG CRegistry::SetValueBinary(LPBYTE pByte, DWORD dwLen, LPCTSTR lpszValueName)
{
	_ASSERTE(m_hKey != NULL);
	return RegSetValueEx(m_hKey, lpszValueName, NULL, REG_BINARY,
		pByte, dwLen );
}


LONG CRegistry::SetValue(DWORD dwValue, LPCTSTR lpszValueName)
{
	_ASSERTE(m_hKey != NULL);
	return RegSetValueEx(m_hKey, lpszValueName, NULL, REG_DWORD,
		(BYTE * const)&dwValue, sizeof(DWORD));
}

HRESULT CRegistry::SetValue(LPCTSTR lpszValue, LPCTSTR lpszValueName)
{
	_ASSERTE(lpszValue != NULL);
	_ASSERTE(m_hKey != NULL);
	return RegSetValueEx(m_hKey, lpszValueName, NULL, REG_SZ,
		(BYTE * const)lpszValue, (lstrlen(lpszValue)+1)*sizeof(TCHAR));
}

//RecurseDeleteKey 
LONG CRegistry::RecurseDeleteKey(LPCTSTR lpszKey)
{
	CRegistry key;
	LONG lResult = key.Open(m_hKey, lpszKey);
	if (lResult != ERROR_SUCCESS)
		return lResult;
	FILETIME time;
	TCHAR szBuffer[256];
	DWORD dwSize = sizeof( szBuffer ) * sizeof (TCHAR);
	while (RegEnumKeyEx(key.m_hKey, 0, szBuffer, &dwSize, NULL, NULL, NULL, &time)==ERROR_SUCCESS)
	{
		lResult = key.RecurseDeleteKey(szBuffer);
		if (lResult != ERROR_SUCCESS)
			return lResult;
		dwSize = sizeof( szBuffer ) * sizeof (TCHAR);
	}
	key.Close();
	return DeleteSubKey(lpszKey);
}
