/*=========================================================================*/
/*                          Viewer.cpp                                     */
/*                                                                         */
/*                    CookieCop's Viewer Window                            */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/

#include "stdafx.h"
#include "cookiecop.h"
#include "Viewer.h"
#include "ViewerOptionsDlg.h"
#include "Hourglass.h"
#include "hlp/helpids.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static UINT WM_FINDREPLACE = RegisterWindowMessage(FINDMSGSTRING);
UINT MSG_UPDATE_STATUSBAR  = RegisterWindowMessage("UpdateStatusBar");	

static BOOL bInUpdate;

enum 
{
    ID_CONNECTIONS,
    ID_KBPS,
    ID_LINES,
};

static UINT BASED_CODE indicators[] =
{
	ID_CONNECTIONS,           // status line indicator
	ID_KBPS,
	ID_LINES,
};


static UINT BASED_CODE buttons[] =
{
    ID_FILE_SAVE,
    //ID_SEPARATOR,
    ID_EDIT_CUT	,
    ID_EDIT_COPY,
	ID_EDIT_FIND,
    ID_CLEAR,
    ID_SEPARATOR ,
    ID_SHOW_BROWSER,
    ID_SHOW_SERVER,
	ID_SHOW_COOKIECOP,
	ID_AUTOSCROLL,
    ID_PUSHPIN,
	ID_SEPARATOR,
	ID_PROPERTIES,
    IDHELP
};


BEGIN_MESSAGE_MAP(CViewer, CDialog)
	//{{AFX_MSG_MAP(CViewer)
	ON_WM_SIZE()
	ON_BN_CLICKED(IDC_CLEAR, OnClear)
	ON_COMMAND(ID_EDIT_FIND, OnViewFindreplaceDialog)
	ON_COMMAND(ID_EDIT_CUT, OnEditCut)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_NOTIFY(EN_SELCHANGE, IDC_RICHEDIT, OnSelchangeRichedit)
	ON_WM_TIMER()
	ON_COMMAND(ID_CLEAR, OnClear)
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
	ON_NOTIFY_EX( TTN_NEEDTEXT, 0, OnToolTipNotify )
	ON_REGISTERED_MESSAGE(WM_FINDREPLACE, OnFindReplace)
	ON_REGISTERED_MESSAGE(MSG_UPDATE_STATUSBAR, OnUpdateViewer )
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CViewer dialog


CViewer::CViewer(CWnd* pParent /*=NULL*/)
	: CDialog(CViewer::IDD, pParent)
{
	//{{AFX_DATA_INIT(CViewer)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	ZeroMemory( &m_FindText, sizeof(m_FindText) );
	m_sSearchText = "";
}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CViewer)
	DDX_Control(pDX, IDC_RICHEDIT, m_RichEdit);
	//}}AFX_DATA_MAP
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CViewer::OnInitDialog() 
{
	m_EventMask = (WORD)-1;
	m_bModified = FALSE;	//save options
	m_pFindDialog = NULL;

	CDialog::OnInitDialog();

	HICON hIcon = AfxGetApp()->LoadIcon(IDI_ICON_IDLE); 
	SetIcon( hIcon, TRUE);
	SetIcon( hIcon, FALSE);

	CreateStatusBar( );
	m_RichEdit.SetReadOnly( );
    MoveControls(  );

	// Create toolbar at the top of the dialog window
	if( m_toolBar.Create(this) )
	{
	    m_toolBar.LoadBitmap( IDR_MAINFRAME );
	    m_toolBar.SetButtons( buttons, sizeof(buttons)/sizeof(UINT) );
		m_toolBar.EnableToolTips(TRUE );
		m_toolBar.SetBarStyle( m_toolBar.GetBarStyle() | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC );
	}     

	//EM_AUTOURLDETECT add this as an option???
    CRect rc;
    LoadViewerPosition( &rc );
    BOOL bCenter = FALSE;
    if( rc.left < 0 || rc.left >= GetSystemMetrics(SM_CXFULLSCREEN) )
        bCenter = TRUE;

    if( rc.top < 0 || rc.top >= GetSystemMetrics(SM_CYFULLSCREEN) )
        bCenter = TRUE;

    if( bCenter ) 
    {
        GetWindowRect( rc );
        rc.OffsetRect(-rc.left, -rc.top);
        MoveWindow( ((GetSystemMetrics(SM_CXSCREEN) - rc.right ) / 2 + 4) & ~7,
			         (GetSystemMetrics(SM_CYSCREEN) - rc.bottom) / 2, rc.right, rc.bottom, 0);
    } else {
        SetWindowPos( NULL,rc.left,rc.top,rc.Width(),rc.Height( ),SWP_SHOWWINDOW | SWP_NOZORDER );
	}

	// We need to resize the dialog to make room for control bars.
	// First, figure out how big the control bars are.
	CRect rcClientStart;
	CRect rcClientNow;
	GetClientRect( rcClientStart );
	RepositionBars( AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST, 0, reposQuery, rcClientNow );
	
	// Now move all the controls so they are in the same relative
	// position within the remaining client area as they would be
	// with no control bars.
	CPoint ptOffset(rcClientNow.left - rcClientStart.left,
					rcClientNow.top - rcClientStart.top); 

	CRect rcChild;					
	CWnd* pwndChild = GetWindow( GW_CHILD );
	while( pwndChild )
	{                               
		pwndChild->GetWindowRect(rcChild);
		ScreenToClient(rcChild);
		rcChild.OffsetRect(ptOffset);
		pwndChild->MoveWindow(rcChild, FALSE);
		pwndChild = pwndChild->GetNextWindow();
	}


	// position the control bars
	RepositionBars(AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST, 0);
	MoveControls( );

	//load in the colors
	CString sFileName;
	QualifyPathName( &sFileName, SZ_COOKIECOP_INI );
	m_crBackground = (COLORREF)GetPrivateProfileInt( SZ_APPNAME, SZ_COLOR_BACKGROUND, RGB(0xFF,0xFF,0xFF), sFileName );
	m_crHTTP       = (COLORREF)GetPrivateProfileInt( SZ_APPNAME, SZ_COLOR_HTTP      , COLOR_RED          , sFileName );
	m_crHTML       = (COLORREF)GetPrivateProfileInt( SZ_APPNAME, SZ_COLOR_HTML      , COLOR_BLUE         , sFileName );
	m_crMessages   = (COLORREF)GetPrivateProfileInt( SZ_APPNAME, SZ_COLOR_MESSAGES  , COLOR_PURPLE       , sFileName );

	m_bAutoScroll  = GetPrivateProfileInt( SZ_APPNAME, SZ_AUTOSCROLL, 1, sFileName );
	m_EventMask    = GetPrivateProfileInt( SZ_APPNAME, SZ_VIEWERMASK,-1, sFileName );
	m_bLimitLines  = GetPrivateProfileInt( SZ_APPNAME, SZ_LIMIT_LINES, 0, sFileName );
	m_nLines       = GetPrivateProfileInt( SZ_APPNAME, SZ_LIMIT_LINES_AMOUNT, 5000, sFileName );
	
    m_toolBar.GetToolBarCtrl().PressButton(ID_PUSHPIN, m_EventMask & VIEWER_ONTOP);
    m_toolBar.GetToolBarCtrl().PressButton(ID_SHOW_BROWSER, m_EventMask & VIEWER_BROWSER );
	m_toolBar.GetToolBarCtrl().PressButton(ID_SHOW_SERVER, m_EventMask & VIEWER_SERVER );
	m_toolBar.GetToolBarCtrl().PressButton(ID_SHOW_COOKIECOP, m_EventMask & VIEWER_CC );
	m_toolBar.GetToolBarCtrl().PressButton(ID_AUTOSCROLL, m_bAutoScroll );

	//set the colors and defaults for the rich edit control
	m_RichEdit.SetBackgroundColor( FALSE, m_crBackground );
	
	CHARFORMAT cfm;
	ZeroMemory( &cfm, sizeof(cfm) );
	cfm.cbSize = sizeof(cfm);
	cfm.dwMask = CFM_FACE | CFM_SIZE;
	cfm.yHeight = GetPrivateProfileInt( SZ_APPNAME, SZ_FONTHEIGHT, 0, sFileName );
	cfm.bCharSet = (BYTE)GetPrivateProfileInt( SZ_APPNAME, SZ_FONTFAMILY, 0, sFileName );
	GetPrivateProfileString( SZ_APPNAME, SZ_FONTNAME, "Arial", cfm.szFaceName, LF_FACESIZE, sFileName );

	m_RichEdit.SetDefaultCharFormat( cfm );
	SetAutoScroll( m_bAutoScroll );

	DWORD dwMask = m_RichEdit.SendMessage( EM_GETEVENTMASK, 0, 0); 	
	m_RichEdit.SendMessage( EM_SETEVENTMASK, 0, dwMask | ENM_SELCHANGE );//| ENM_SCROLL ); 	

    //disable the cut and copy toolbar options
    EnableControls( FALSE );

    DisplayOnTop( );

    OnUpdateViewer( 0, 0 );
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


/////////////////////////////////////////////////////////////////////////////
// CViewer message handlers

void CViewer::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize( nType, cx, cy );
	
	RepositionBars( AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST, 0 );
    MoveControls( );

}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::MoveControls( ) 
{
	CRect rc,rc2;
	GetClientRect( &rc );

	RedrawWindow( );

	CWnd* pWnd;
	pWnd =	GetDlgItem( IDC_RICHEDIT );
	
	if( pWnd )
	{
		rc.DeflateRect( 1, 1 );

        if( m_toolBar )
        {
            m_toolBar.GetWindowRect( rc2 );
            rc.top += rc2.Height( );
        }

		if( m_wndStatusBar )
        {
            m_wndStatusBar.GetWindowRect( rc2 );
		    rc.bottom -= rc2.Height( );
        }
		pWnd->MoveWindow( rc, TRUE );
	}		
}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::OnOK() 
{
	// TODO: Add extra validation here

	m_theApp->m_criticalSectionViewer.Lock( );
	DestroyWindow( );
	delete m_theApp->m_pViewerDlg;
	m_theApp->m_pViewerDlg = NULL;
	m_theApp->m_criticalSectionViewer.Unlock( );    
}


/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::AddText( VIEWER_EVENT event , LPCSTR pText )
{
	COLORREF colr = 0;

    try
    {

	//if message type is not selected then return
	if( ( (event & ~VIEWER_MSG ) & m_EventMask ) == 0 )
		return;

    bInUpdate = TRUE;

    switch( event & ~VIEWER_MSG )
	{
		case VIEWER_BROWSER:    colr = m_crHTTP; break;
		case VIEWER_SERVER:     colr = m_crHTML; break;
		case VIEWER_CC:	        colr = m_crMessages; break;
            
        #ifdef _DEBUG    
        case VIEWER_DEBUG:      colr = m_crMessages; break;
        #endif
		default:
            bInUpdate = FALSE;
            return;
			//MessageBox( "Error in eventtype ");
	}

	
	CHARRANGE crSave;
	m_RichEdit.GetSel( crSave ); 
	
	CHARFORMAT cfm;
	cfm.cbSize = sizeof( cfm );
	cfm.dwMask =  CFM_COLOR | CFM_BOLD;
	cfm.dwEffects = 0;
	cfm.crTextColor = colr;
	if( event & VIEWER_MSG )
	{
		cfm.dwEffects = CFE_BOLD;
		cfm.dwMask |= CFM_BOLD ;
	}


	//append selection to end
	CHARRANGE crInsertSel;
	crInsertSel.cpMin = -1;
	crInsertSel.cpMax = -1;
	m_RichEdit.SetSel( crInsertSel );
    m_RichEdit.SetSelectionCharFormat( cfm );

	m_RichEdit.ReplaceSel( pText );


	if( m_bAutoScroll ) 
	{
		m_RichEdit.SendMessage( EM_SCROLLCARET );
	} 

	if( m_bLimitLines )
	{
		int iLines = m_RichEdit.GetLineCount( );
		int MAX_LINES = m_nLines;
		int iOver = ( iLines - MAX_LINES );

		if( iOver > 0 )
		{
			iOver += m_nLines/10;	//divide by 10 to reduce updates

			int x = m_RichEdit.LineIndex( iOver + 1 );
			if( x > 0 )
			{
				m_RichEdit.SetSel( 0, x );
				m_RichEdit.ReplaceSel( "" );

				if( !m_bAutoScroll )
				{
					m_RichEdit.SetSel( max( crSave.cpMin - x , 0 ), 
									   max( crSave.cpMax - x , 0 ) );
				} else {
					m_RichEdit.SetSel( -1, -1 );
				}

			}
		}
	}


    bInUpdate = FALSE;

    } catch(...)
    {
        DebugOut( "Exception in CViewer::AddText" );
    }
}


/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::OnClear() 
{
	CHARRANGE cr;
	cr.cpMin =  0;
	cr.cpMax = -1;
	m_RichEdit.SetWindowText( "" );

    OnUpdateViewer( 0, 0);
}

/////////////////////////////////////////////////////////////////////////////
// 
BOOL CViewer::CreateStatusBar()
{
	
	// Create status bar at the bottom of the dialog window	               
	if( m_wndStatusBar.Create(this) )
	{                           
		m_wndStatusBar.SetIndicators(indicators, sizeof(indicators)/sizeof(UINT) );
		CRect rc;
		GetClientRect( rc );
		int cx = rc.right / sizeof(indicators)/sizeof(UINT) ;

		for( int i = 0; i < sizeof(indicators)/sizeof(UINT) ; i++ )
			m_wndStatusBar.SetPaneInfo(i, m_wndStatusBar.GetItemID( i ),SBPS_STRETCH, cx );
	}   
	return( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
BOOL CViewer::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	switch( LOWORD(wParam) )
	{

		case ID_FILE_SAVE:	
			OnStream();
		break;
		
		case ID_PROPERTIES:
		{
			ShowPropertiesDlg( );
		}
		break;
		
		case ID_SHOW_BROWSER:
		{
			if( m_EventMask & VIEWER_BROWSER )
				m_EventMask &= ~VIEWER_BROWSER;
			else
				m_EventMask |= VIEWER_BROWSER;

			m_toolBar.GetToolBarCtrl().PressButton(ID_SHOW_BROWSER, m_EventMask & VIEWER_BROWSER );
			m_bModified = TRUE;
			break;
		}

		case ID_SHOW_SERVER:
		{
			if( m_EventMask & VIEWER_SERVER )
				m_EventMask &= ~VIEWER_SERVER;
			else
				m_EventMask |= VIEWER_SERVER;

			m_toolBar.GetToolBarCtrl().PressButton(ID_SHOW_SERVER, m_EventMask & VIEWER_SERVER );
			m_bModified = TRUE;
			break;
		}

		case ID_SHOW_COOKIECOP:
		{
			if( m_EventMask & VIEWER_CC )
				m_EventMask &= ~VIEWER_CC;
			else
				m_EventMask |= VIEWER_CC;

			m_toolBar.GetToolBarCtrl().PressButton(ID_SHOW_COOKIECOP, m_EventMask & VIEWER_CC );
			m_bModified = TRUE;
			break;
		}


		case ID_AUTOSCROLL:
		{
			if( m_bAutoScroll ==1 )
				m_bAutoScroll = 0;
			else
				m_bAutoScroll = 1;

			if( m_bAutoScroll )
			{
				CHARRANGE crInsertSel;
				crInsertSel.cpMin = -1;
				crInsertSel.cpMax = -1;
				m_RichEdit.SetSel( crInsertSel );
			}
			
			m_toolBar.GetToolBarCtrl().PressButton(ID_AUTOSCROLL, m_bAutoScroll );
			m_bModified = TRUE;
			SetAutoScroll( m_bAutoScroll );
		}
		break;

        case ID_PUSHPIN:
		{
			if( m_EventMask & VIEWER_ONTOP )
				m_EventMask &= ~VIEWER_ONTOP;
			else
				m_EventMask |= VIEWER_ONTOP;

			m_toolBar.GetToolBarCtrl().PressButton(ID_PUSHPIN, m_EventMask & VIEWER_ONTOP );
			m_bModified = TRUE;
            DisplayOnTop( );
			break;
		}


		case ID_CLEAR:
			OnClear( );
			break;

        case IDHELP:
            m_theApp->CWinApp::WinHelp(IDH_viewer_window);
            break;


		default:
			return CDialog::OnCommand(wParam, lParam);
	}		

	return( TRUE );	
	
}

/////////////////////////////////////////////////////////////////////////////
// 
BOOL CViewer::OnToolTipNotify( UINT id, NMHDR * pNMHDR, LRESULT * pResult )
{

	UNREFERENCED_PARAMETER( pResult );
	UNREFERENCED_PARAMETER( id );

    TOOLTIPTEXT *pTTT = (TOOLTIPTEXT *)pNMHDR;
    UINT nID = pNMHDR->idFrom;
    
	// nID is actually the HWND of the tool
	if( pTTT->uFlags & TTF_IDISHWND )
		 nID = ::GetDlgCtrlID((HWND)nID);

    if(nID)
    {
        pTTT->lpszText = MAKEINTRESOURCE( nID );
        pTTT->hinst = AfxGetResourceHandle();
        return(TRUE);
    }
    
    return(FALSE);
}


/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::OnViewFindreplaceDialog() 
{
    //MFC 'deletes' the memory when the dialog is closed
 
	if( !m_pFindDialog )
	{
        SetAutoScroll( TRUE );
       
        CFindReplaceDialog *pDlg = new CFindReplaceDialog;
		pDlg->Create(TRUE, m_sSearchText, NULL, FR_DOWN | FR_HIDEUPDOWN, this);

		m_pFindDialog = pDlg;

		m_FindText.chrg.cpMin = 0;
		m_FindText.chrg.cpMax = -1;
	} else {
		m_pFindDialog->SetFocus( );
	}
}


/////////////////////////////////////////////////////////////////////////////
//called via message from CProxy to update the viewer
//lparam contains the time the connection was established
LRESULT CViewer::OnUpdateViewer(WPARAM wParam, LPARAM lParam)
{
	UNREFERENCED_PARAMETER( wParam );
	UNREFERENCED_PARAMETER( lParam );

	CString s;

	s.Format( "Connections: %lu", m_theApp->m_nSessionCount );
	m_wndStatusBar.SetPaneText( ID_CONNECTIONS, s );

	s.Format( "Lines: %s", FormatNumber( m_RichEdit.GetLineCount( )-1 ) );
	m_wndStatusBar.SetPaneText( ID_LINES, s );
	
	if( wParam == UPDATE_BYTES )
	{	
		if( m_theApp->m_nSessionCount )
		{
			//calc bytes per second
			DWORD seconds = ( GetTickCount( ) - lParam );//m_theApp->m_dwSessionStartTime );
	
			if( seconds )
			{
				double b1 = m_theApp->m_dwStats[STATS_BYTES_RECEIVED];
				double b2 = m_theApp->m_dwSessionBytes;
				double bytes = b1 - b2;

				bytes *= 1000.0;
				double bits = ( ( bytes ) / seconds );
                bits *= 8;
                bits /= 1000.0;

				s.Format( "Last Kb/sec: %6.2f", bits );
				m_wndStatusBar.SetPaneText( ID_KBPS, s );

			}

		}
	}
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// 
LRESULT CViewer::OnFindReplace(WPARAM wParam, LPARAM lParam)
{
	UNREFERENCED_PARAMETER( wParam );

	LPFINDREPLACE pfr = (LPFINDREPLACE)lParam;

	if( pfr->Flags & FR_DIALOGTERM )
	{
		SetAutoScroll( m_bAutoScroll );
        m_pFindDialog = NULL;
		return 1;
	}


    if( ((LPFINDREPLACE)lParam)->Flags & FR_FINDNEXT )
    {
		LONG lMatch; 
		CHARRANGE chrgSave; 
		
		m_RichEdit.GetSel( chrgSave );
		
		m_FindText.lpstrText = ((LPFINDREPLACE)lParam)->lpstrFindWhat;
		m_sSearchText = m_FindText.lpstrText;

		lMatch = GetDlgItem( IDC_RICHEDIT )->SendMessage( EM_FINDTEXTEX, (WPARAM)pfr->Flags, (LPARAM)(FINDTEXTEX*) &m_FindText ); 

		if( lMatch >= 0 )
		{
            m_RichEdit.HideSelection( FALSE, FALSE );
			m_RichEdit.SetSel( m_FindText.chrgText.cpMin, m_FindText.chrgText.cpMax );
            GetDlgItem( IDC_RICHEDIT )->SendMessage( EM_SCROLLCARET,0, 0 );
            

			if( pfr->Flags & FR_DOWN ) {
				m_FindText.chrg.cpMin = m_FindText.chrgText.cpMax ;///else not found
				m_FindText.chrg.cpMax = -1;
			} else {
				m_FindText.chrg.cpMax = m_FindText.chrgText.cpMin;
				m_FindText.chrg.cpMin = 0;
			}
			
			AdjustDialogPosition( m_pFindDialog );

		}  else {
			//not found
            MessageBeep( MB_OK );
		}
			
    }

    return 0;

}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CViewer::PreTranslateMessage(MSG* pMsg) 
{
				 
	if( pMsg->message == WM_KEYUP )	
	{
		if( pMsg->wParam == VK_F3 )
			OnViewFindreplaceDialog( );
	}
    
	if( pMsg->message == WM_RBUTTONUP )
    {
		ShowPropertiesDlg( );
        return TRUE;
    }

	if( pMsg->message == WM_KEYDOWN )	
    {
        //prevent the user from moving the cursor while we are
        //in the middle of updating the edit control
        if( bInUpdate )
            return TRUE;

        switch( pMsg->wParam  )
        {
            case VK_DELETE:
                OnEditCut();
                return TRUE;
                break;
            
            case VK_HOME:
            case VK_END:
            case VK_UP:
            case VK_DOWN:
            {
	            SetAutoScroll( TRUE );
                CDialog::PreTranslateMessage(pMsg);
                SetAutoScroll( m_bAutoScroll );
                return TRUE;
            }

        }

    }
	return CDialog::PreTranslateMessage(pMsg);
}


/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::PostNcDestroy() 
{
    
	m_theApp->m_pViewerDlg = NULL;
	delete this;
}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::OnCancel() 
{
	if( m_bModified )
	{
		WritePrivateProfile( SZ_AUTOSCROLL, m_bAutoScroll );
		WritePrivateProfile( SZ_VIEWERMASK, m_EventMask );
	}
    m_theApp->m_bViewerVisible = FALSE;
    WritePrivateProfile( SZ_VIEWER_OPEN, m_theApp->m_bViewerVisible );

	DestroyWindow( );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::OnEditCut() 
{
	m_RichEdit.SetReadOnly( FALSE );
	m_RichEdit.Cut( );
	m_RichEdit.SetReadOnly( TRUE );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::SetAutoScroll( BOOL bEnable )
{
    WORD wOption;
    if( bEnable )
        wOption = ECOOP_OR;
    else
        wOption = ECOOP_XOR;

    m_RichEdit.SetOptions( wOption, ECO_AUTOHSCROLL | ECO_AUTOVSCROLL );
    m_RichEdit.SetOptions( ECOOP_OR, ECO_SAVESEL );  

}


/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::OnEditCopy() 
{
	m_RichEdit.Copy( );	
}


/////////////////////////////////////////////////////////////////////////////
// Position the find dialog so that it does not obscure the text
void CViewer::AdjustDialogPosition(CDialog* pDlg)
{
	ASSERT(pDlg != NULL);
	long lStart, lEnd;
	m_RichEdit.GetSel(lStart, lEnd);
	CPoint point = m_RichEdit.GetCharPos(lStart);
	ClientToScreen(&point);
	CRect rectDlg;
	pDlg->GetWindowRect(&rectDlg);

    
	if (rectDlg.PtInRect(point))
	{
		if (point.y > rectDlg.Height())
			rectDlg.OffsetRect(0, point.y - rectDlg.bottom - 20);
		else
		{
			int nVertExt = GetSystemMetrics(SM_CYSCREEN);
			if (point.y + rectDlg.Height() < nVertExt)
				rectDlg.OffsetRect(0, 40 + point.y - rectDlg.top);
		}
		pDlg->MoveWindow(&rectDlg);
	}
}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::EnableControls(BOOL bEnable ) 
{
	m_toolBar.GetToolBarCtrl( ).EnableButton( ID_EDIT_CUT, bEnable );	
	m_toolBar.GetToolBarCtrl( ).EnableButton( ID_EDIT_COPY, bEnable );	

}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::OnSelchangeRichedit(NMHDR* pNMHDR, LRESULT* pResult) 
{
	SELCHANGE *pSelChange = reinterpret_cast<SELCHANGE *>(pNMHDR);
	// TODO: The control will not send this notification unless you override the
	// CDialog::OnInitDialog() function to send the EM_SETEVENTMASK message
	// to the control with the ENM_SELCHANGE flag ORed into the lParam mask.
    
	 BOOL bEnable = pSelChange->seltyp != SEL_EMPTY;
     EnableControls( bEnable );
	
	*pResult = 0;
}


/////////////////////////////////////////////////////////////////////////////
// 
DWORD CALLBACK WriteEditData(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG FAR *pcb)
{
   CFile* pFile = (CFile*) dwCookie;

   try {
      
        BYTE ch, last = 0;
        char buf[3];
        wsprintf( buf, "%c%c", CR, LF );  //append CR, LF to strings      


        for( int i = 0; i < cb; i++ )
        {
            ch = *(pbBuff + i);
            if( ch == LF && last != CR )
            {
                pFile->Write( buf, 2);
            } else {
                pFile->Write( &ch, 1 );
            }
            last = ch;
        }

   }
   catch (CFileException* pEx)
   {
      pEx;
      *pcb = 0;
      return 1; //error
   };

   *pcb = cb;
   return 0;    //continue
}



/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::OnStream() 
{

	CFileDialog dlg(FALSE,
					".txt",
					NULL, 0,
					"Text Files (*.txt)|*.txt|All Files (*.*)|*.*||",
					this );

	dlg.m_ofn.lpstrTitle = "Save Log File";
	dlg.m_ofn.Flags |= OFN_NOCHANGEDIR | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT;

	if( dlg.DoModal() == IDOK ) 
	{
		

		CFile fWrite( dlg.GetPathName() , CFile::modeCreate | CFile::modeWrite );

		EDITSTREAM strm;
        strm.dwError = 0;
		strm.dwCookie = (DWORD) &fWrite;
		strm.pfnCallback = WriteEditData;

        CHourglass wait;
        UpdateWindow( );
        m_RichEdit.StreamOut( SF_TEXT, strm );
		fWrite.Close();

		if( strm.dwError == 0 )
			AfxMessageBox( IDS_FILESAVED, MB_OK );

	}
}


/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::OnTimer(UINT nIDEvent) 
{
    UNREFERENCED_PARAMETER( nIDEvent );

	OnUpdateViewer( UPDATE_BYTES, m_theApp->m_dwSessionStartTime );	
}



/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::ShowPropertiesDlg( )
{
	CViewerOptionsDlg* pDlg = new CViewerOptionsDlg;
	pDlg->m_pViewer = this;
	pDlg->DoModal( );	
	delete pDlg;
}



/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::OnClose() 
{
    CRect rc;
    m_theApp->m_pViewerDlg->GetWindowRect( rc );
    SaveViewerPosition( &rc );
	
	CDialog::OnClose();
}


/////////////////////////////////////////////////////////////////////////////
// 
void CViewer::DisplayOnTop( )
{
    if( m_EventMask & VIEWER_ONTOP )
		m_theApp->m_pViewerDlg->SetWindowPos( &wndTopMost,0,0,0,0,SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE );
	else 
		m_theApp->m_pViewerDlg->SetWindowPos( &wndNoTopMost,0,0,0,0,SWP_SHOWWINDOW | SWP_NOMOVE | SWP_NOSIZE);

}
