/*=========================================================================*/
/*                       ViewerOptionsDlg.cpp                              */
/*                                                                         */
/*                   Options for the Viewer Window                         */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/

#include "stdafx.h"
#include "cookiecop.h"
#include "ViewerOptionsDlg.h"
#include "WarningDlgs.h"
#include "hourglass.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define LINES_WARNING 2000

static int nFontSizes[] =
	{8, 9, 10, 11, 12, 14, 16, 18, 20, 22, 24, 26, 28, 36, 48, 72, 0};


BEGIN_MESSAGE_MAP(CViewerOptionsDlg, CDialog)
	//{{AFX_MSG_MAP(CViewerOptionsDlg)
	ON_BN_CLICKED(IDC_BACKGROUND_COLOR, OnBackgroundColor)
	ON_WM_DRAWITEM()
	ON_BN_CLICKED(IDC_HTTP_COLOR, OnHttpColor)
	ON_BN_CLICKED(IDC_CC_COLOR, OnCcColor)
	ON_BN_CLICKED(IDC_HTML_COLOR, OnHtmlColor)
	ON_CBN_SELCHANGE(IDC_FONTS, OnSelchangeFonts)
	ON_CBN_SELCHANGE(IDC_FONTSIZES, OnSelchangeFontsizes)
	ON_BN_CLICKED(IDC_LIMIT_LINES, OnLimitLines)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()




/////////////////////////////////////////////////////////////////////////////
// CViewerOptionsDlg dialog


CViewerOptionsDlg::CViewerOptionsDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CViewerOptionsDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CViewerOptionsDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CViewerOptionsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CViewerOptionsDlg)
	DDX_Control(pDX, IDC_FONTSIZES, m_FontSizes);
	DDX_Control(pDX, IDC_FONTS, m_FontNames);
	//}}AFX_DATA_MAP
}




BOOL CViewerOptionsDlg::GetColor( COLORREF* pColorRef )
{

	CColorDialog dlg;
	dlg.m_cc.Flags |= CC_ANYCOLOR | CC_SOLIDCOLOR | CC_RGBINIT;
    dlg.m_cc.rgbResult = *pColorRef;
	if( dlg.DoModal( ) == IDOK )
	{
		*pColorRef = dlg.m_cc.rgbResult;
		return TRUE;
	}
	return FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// CViewerOptionsDlg message handlers


BOOL CViewerOptionsDlg::OnInitDialog() 
{
	//save current colors
	m_crDlgBackground = m_pViewer->m_crBackground;
	m_crDlgHTTP       = m_pViewer->m_crHTTP;
	m_crDlgHTML       = m_pViewer->m_crHTML;
	m_crDlgMessages   = m_pViewer->m_crMessages;

	CDialog::OnInitDialog();

	//enumerate available fonts
	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));
	lf.lfCharSet = DEFAULT_CHARSET;
	lf.lfCharSet =  ANSI_CHARSET;

	HDC hDC = ::GetDC(NULL);
	int nVert = ::GetDeviceCaps(hDC, LOGPIXELSY );
	::EnumFontFamiliesEx(hDC, &lf,(FONTENUMPROC)EnumFontNamesCallBack, (LPARAM) this, 0);
	::ReleaseDC(NULL, hDC );

    m_bDirty = FALSE;
    m_bWarningDialogDisplayed = FALSE;
    m_bWarningResponse = FALSE;

	//save the current selection
	m_pViewer->m_RichEdit.GetSel( m_nStart, m_nEnd );
	m_nStart = m_pViewer->m_RichEdit.LineIndex( m_pViewer->m_RichEdit.GetFirstVisibleLine( ) );
	m_nEnd = m_nStart;

	
	//Get the current font and size
	CHARFORMAT ch;
	ch.cbSize = sizeof( ch );
	ch.dwMask = CFM_SIZE;

	m_pViewer->m_RichEdit.GetSelectionCharFormat( ch );

	//save the facename and height in case the user cancels
	m_sFontFace = ch.szFaceName;
	m_nFontSize = ch.yHeight;


	int i = m_FontNames.FindStringExact( 0, ch.szFaceName );
	if( i == CB_ERR )
		i = 0;

	m_FontNames.SetCurSel( i );	
	EnumFontSizes( );

	//convert twips to points and find the closest match
	int iHeight = MulDiv( ch.yHeight, nVert, 1440 );
	int iCount = m_FontSizes.GetCount( );
	CString s;

	for( i = 0; i < iCount; i++ )
	{
		m_FontSizes.GetLBText( i, s );
		int nSize = atoi( s );
		if( nSize <= iHeight )
			m_FontSizes.SetCurSel( i );
	}
	
	//

	CheckDlgButton( IDC_LIMIT_LINES, m_pViewer->m_bLimitLines );

	s.Format( "%u", m_pViewer->m_nLines );
	SetDlgItemText( IDC_LIMIT_LINES_AMOUNT, s ); 

	OnLimitLines( );

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewerOptionsDlg::OnOK() 
{
	
	// save to INI file
   	WritePrivateProfile( SZ_COLOR_HTTP, m_pViewer->m_crHTTP );
	WritePrivateProfile( SZ_COLOR_HTML, m_pViewer->m_crHTML );
    WritePrivateProfile( SZ_COLOR_MESSAGES, m_pViewer->m_crMessages );
    WritePrivateProfile( SZ_COLOR_BACKGROUND, m_pViewer->m_crBackground );


	CHARFORMAT cfm;
	cfm.cbSize = sizeof( cfm );
	cfm.dwMask = CFM_FACE | CFM_SIZE;
	m_pViewer->m_RichEdit.GetDefaultCharFormat( cfm );
	
	WritePrivateProfile( SZ_FONTNAME, cfm.szFaceName );
	WritePrivateProfile( SZ_FONTHEIGHT, cfm.yHeight );
	WritePrivateProfile( SZ_FONTFAMILY, cfm.bPitchAndFamily );	
	
	CString s;
	GetDlgItemText( IDC_LIMIT_LINES_AMOUNT, s ); 
	WritePrivateProfile( SZ_LIMIT_LINES_AMOUNT, s);
	m_pViewer->m_nLines = max( atoi( s ), 100 );
		
	m_pViewer->m_bLimitLines = IsDlgButtonChecked( IDC_LIMIT_LINES );
	WritePrivateProfile( SZ_LIMIT_LINES, m_pViewer->m_bLimitLines );	
	
    CDialog::OnOK();
}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewerOptionsDlg::OnCancel() 
{
	//restore colors
    if( m_bDirty )
    {
        UpdateEditControl( m_pViewer->m_crHTTP, m_crDlgHTTP  );
	    UpdateEditControl( m_pViewer->m_crHTML, m_crDlgHTML );
	    UpdateEditControl( m_pViewer->m_crMessages,  m_crDlgMessages );

        //restore
	    m_pViewer->m_crHTTP = m_crDlgHTTP;
	    m_pViewer->m_crHTML = m_crDlgHTML;
	    m_pViewer->m_crMessages = m_crDlgMessages;
    }

    m_pViewer->m_crBackground = m_crDlgBackground;
	m_pViewer->m_RichEdit.SetBackgroundColor( FALSE, m_pViewer->m_crBackground );

	//restore the font
	CHARFORMAT cfm;
    ZeroMemory( &cfm, sizeof(cfm) );
	cfm.cbSize = sizeof( cfm );
	cfm.dwMask = CFM_FACE | CFM_SIZE;
	cfm.yHeight = m_nFontSize;
	strncpy( cfm.szFaceName, m_sFontFace, LF_FACESIZE );

	m_pViewer->m_RichEdit.SetSel( 0,-1 );
	m_pViewer->m_RichEdit.SetSelectionCharFormat( cfm );

	m_pViewer->m_RichEdit.SetSel( m_nStart, m_nEnd );
	m_pViewer->m_RichEdit.HideSelection(TRUE, FALSE);

	CDialog::OnCancel();
}


/////////////////////////////////////////////////////////////////////////////
// 
void CViewerOptionsDlg::OnBackgroundColor() 
{
	COLORREF crNewColor = m_crDlgBackground;

	if( GetColor( &crNewColor ) )
	{
		m_pViewer->m_crBackground = crNewColor;
		GetDlgItem( IDC_BACKGROUND_COLOR)->InvalidateRect( NULL );
		m_pViewer->m_RichEdit.SetBackgroundColor( FALSE, crNewColor );
	}
}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewerOptionsDlg::OnHttpColor() 
{
	COLORREF crNewColor = m_crDlgHTTP;

	if( GetColor( &crNewColor ) )
	{
        if( ChangeColorWarning( ) )
            UpdateEditControl( m_pViewer->m_crHTTP, crNewColor );
        m_pViewer->m_crHTTP = crNewColor;
        GetDlgItem( IDC_HTTP_COLOR )->InvalidateRect( NULL );
    }
}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewerOptionsDlg::OnCcColor() 
{
	COLORREF crNewColor = m_crDlgMessages;

	if( GetColor( &crNewColor ) )
	{
        if( ChangeColorWarning( ) )
            UpdateEditControl( m_pViewer->m_crMessages, crNewColor );
        m_pViewer->m_crMessages = crNewColor;
        GetDlgItem( IDC_CC_COLOR)->InvalidateRect( NULL );
	}
	
}


/////////////////////////////////////////////////////////////////////////////
// 
void CViewerOptionsDlg::OnHtmlColor() 
{
	COLORREF crNewColor = m_pViewer->m_crHTML;

	if( GetColor( &crNewColor ) )
	{

        if( ChangeColorWarning( ) )
            UpdateEditControl( m_pViewer->m_crHTML, crNewColor );
        m_pViewer->m_crHTML = crNewColor;
        GetDlgItem( IDC_HTML_COLOR )->InvalidateRect( NULL );

	}
	
}


/////////////////////////////////////////////////////////////////////////////
// 
void CViewerOptionsDlg::OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
	HBRUSH hbr;
	COLORREF cr;

	int iStyle;
	if( (lpDrawItemStruct->itemAction & ODA_SELECT ) && (lpDrawItemStruct->itemState & ODS_SELECTED ) )
		iStyle = EDGE_SUNKEN;
	else
		iStyle = EDGE_RAISED;

	switch( nIDCtl )
	{
		case IDC_BACKGROUND_COLOR:	cr = m_pViewer->m_crBackground; break;
		case IDC_HTTP_COLOR:		cr = m_pViewer->m_crHTTP; break;
		case IDC_HTML_COLOR:		cr = m_pViewer->m_crHTML; break;
		case IDC_CC_COLOR:			cr = m_pViewer->m_crMessages; break;
		default:
			{
				CDialog::OnDrawItem( nIDCtl, lpDrawItemStruct );
				return;
			}
	}

	hbr = CreateSolidBrush( cr );
	FillRect( lpDrawItemStruct->hDC, &lpDrawItemStruct->rcItem, hbr );		
	DrawEdge( lpDrawItemStruct->hDC, &lpDrawItemStruct->rcItem, iStyle, BF_RECT );
	DeleteObject( hbr );
	return;

}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewerOptionsDlg::UpdateEditControl( COLORREF crOldColor, COLORREF crNewColor )
{
	CHARFORMAT cfm;
	CHARRANGE crInsertSel, crSaveSel;
	int i, iLen;

	if( crOldColor == crNewColor )
		return;	

    //update the individual lines
	int iLines = m_pViewer->m_RichEdit.GetLineCount( );
    m_bDirty = TRUE;
	CHourglass wait( this );

	m_pViewer->m_RichEdit.GetSel( crSaveSel );
	
	CRect rc;
	GetDlgItem( IDC_PROGRESS )->GetWindowRect( rc );
	ScreenToClient( rc );
	
	CProgressCtrl progress;
	progress.Create( WS_VISIBLE | WS_CHILD | PBS_SMOOTH, rc, this, -1 ); 
	progress.SetRange( 0, iLines );

	//for percent display
	CDC* pDC = progress.GetDC( );
	CString s;
	CSize size;
	CRect rc1, rc2; 
	CFont* pFont = GetFont( );
	CFont* pOldFont = pDC->SelectObject( pFont );
    UpdateWindow( );

	for( i = 0; i < iLines; i++ )
	{
        int nPos = m_pViewer->m_RichEdit.LineIndex( i );
		iLen = m_pViewer->m_RichEdit.LineLength( nPos );
		if( iLen )
		{
			crInsertSel.cpMin = nPos;
			crInsertSel.cpMax = crInsertSel.cpMin + iLen;

			m_pViewer->m_RichEdit.SetSel( crInsertSel );

			cfm.cbSize = sizeof( cfm );

			m_pViewer->m_RichEdit.GetSelectionCharFormat( cfm );
			if( cfm.crTextColor == crOldColor )
			{
				cfm.crTextColor = crNewColor;
				m_pViewer->m_RichEdit.SetSelectionCharFormat( cfm );

				progress.SetPos( i );
	
				if( i )
				{
					s.Format( "Updating %3u%%", ( i * 100 / iLines ) );
	
					DWORD dx, dy;

					progress.GetClientRect( rc1 );
					rc1.InflateRect( -1,-1 );
					rc2 = rc1;

					dx = rc1.right;
					dy = rc1.bottom;
					int x  = (WORD)((DWORD)i * dx / iLines ) + 1;
					rc1.right = rc2.left  = x;

					int len = s.GetLength( );
					size = pDC->GetTextExtent( s, len );
					
					#define WHITE RGB(255,255,255)
					COLORREF crBack = pDC->GetPixel( 1,1 ); 

					pDC->SetBkColor( crBack );
					pDC->SetTextColor( WHITE );
					pDC->ExtTextOut ((dx - size.cx)/2, (dy - size.cy)/2,
									 ETO_OPAQUE | ETO_CLIPPED,
									&rc1,
									s,len,NULL);

					pDC->SetBkColor( WHITE );
					pDC->SetTextColor( crBack );
					pDC->ExtTextOut ((dx - size.cx)/2, (dy - size.cy)/2,
								ETO_OPAQUE | ETO_CLIPPED,
								&rc2,
								s,len,NULL);
					
				}
			}
		}

	}

	pDC->SelectObject( pOldFont );
	progress.ReleaseDC( pDC );

	progress.SetPos( iLines );
	m_pViewer->m_RichEdit.SetSel( crSaveSel );
	m_pViewer->m_RichEdit.HideSelection(TRUE, FALSE);

}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL AFX_EXPORT CALLBACK CViewerOptionsDlg::EnumFontNamesCallBack(ENUMLOGFONTEX* pelf,
		NEWTEXTMETRICEX* /*lpntm*/, int /*FontType*/, LPARAM pThis)
{
	// don't put in non-printer raster fonts
    //	if (FontType & RASTER_FONTTYPE)
    //		return 1;

    if( strlen( (LPCSTR)pelf->elfFullName ) )
	    ((CViewerOptionsDlg*) pThis)->m_FontNames.AddString( (LPCSTR)pelf->elfFullName );

	return 1;
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CALLBACK AFX_EXPORT CViewerOptionsDlg::EnumSizeCallBack(LOGFONT FAR* /*lplf*/,
		LPNEWTEXTMETRIC /*lpntm*/, int FontType, LPVOID pThis )
{
	
	ASSERT( pThis != NULL );

	char buf[10];
	// if truetype or vector font
	if(	(FontType & TRUETYPE_FONTTYPE) || !( (FontType & TRUETYPE_FONTTYPE) || (FontType & RASTER_FONTTYPE) ) ) 
	{
		// this occurs when there is a truetype and nontruetype version of a font
		//if (((CViewerOptionsDlg*) pThis)->m_FontSizes.GetCount() != 0)
		//	((CViewerOptionsDlg*) pThis)->m_FontSizes.ResetContent();

		if (((CViewerOptionsDlg*) pThis)->m_FontSizes.GetCount() != 0)
			return 0 ;

		int i = 0;
		while( nFontSizes[i] )
		{
			wsprintf( buf, "%d", nFontSizes[i] );
			((CViewerOptionsDlg*) pThis)->m_FontSizes.AddString( buf );
			i++;								
		}
		
		return FALSE; // don't call again
	}

	return TRUE; // call again
}



/////////////////////////////////////////////////////////////////////////////
// 
void CViewerOptionsDlg::EnumFontSizes( ) 
{
	int i = m_FontNames.GetCurSel( );
	if( i != CB_ERR )
	{		
	
		CString sFontName;
		m_FontNames.GetLBText( i, sFontName );

		if( sFontName.GetLength( ) )
		{
			HDC hDC = ::GetDC(NULL);
			::EnumFontFamilies(hDC, sFontName,	(FONTENUMPROC) EnumSizeCallBack, (LPARAM) this);
			::ReleaseDC(NULL, hDC );
		}
	}
}


/////////////////////////////////////////////////////////////////////////////
// 
void CViewerOptionsDlg::OnSelchangeFontsizes() 
{
	UpdateFontDisplay( );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewerOptionsDlg::OnSelchangeFonts() 
{
	UpdateFontDisplay( );   	
}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewerOptionsDlg::UpdateFontDisplay( ) 
{

	int i = m_FontNames.GetCurSel( );
	
	if( i != CB_ERR )
	{
		CString sFontName;
		m_FontNames.GetLBText( i, sFontName );
		EnumFontSizes( );

		CHARFORMAT cfm;
		cfm.cbSize = sizeof( cfm );
		cfm.dwMask = CFM_FACE | CFM_SIZE;
		strncpy( cfm.szFaceName, sFontName, LF_FACESIZE );

		HDC dc = ::GetDC(NULL);		
		int nVert = ::GetDeviceCaps( dc, LOGPIXELSY );				
		::ReleaseDC( NULL, dc );

		CString sSize;
		i = m_FontSizes.GetCurSel( );
		if( i == CB_ERR )
			i = 0;
		m_FontSizes.GetLBText( i, sSize );

		int nSize = atoi( sSize );

		cfm.yHeight = MulDiv( nSize,1440, nVert );
		m_pViewer->m_RichEdit.SetSel( 0,-1 );
		m_pViewer->m_RichEdit.SetSelectionCharFormat( cfm );
        m_pViewer->m_RichEdit.SetDefaultCharFormat( cfm );
		
       	m_pViewer->m_RichEdit.SetSel( m_nStart, m_nEnd );
	    m_pViewer->m_RichEdit.HideSelection(TRUE, FALSE);

	}

}


/////////////////////////////////////////////////////////////////////////////
// 
void CViewerOptionsDlg::OnLimitLines() 
{
	GetDlgItem( IDC_LIMIT_LINES_AMOUNT )->EnableWindow( IsDlgButtonChecked( IDC_LIMIT_LINES ) );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CViewerOptionsDlg::OnUpdateViewer() 
{
    UpdateEditControl( m_pViewer->m_crHTTP, m_crDlgHTTP );
	UpdateEditControl( m_pViewer->m_crHTML, m_crDlgHTML );
	UpdateEditControl( m_pViewer->m_crMessages, m_crDlgMessages );
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CViewerOptionsDlg::ChangeColorWarning( )
{

	int iLines = m_pViewer->m_RichEdit.GetLineCount( );

    if( iLines > LINES_WARNING )
    {
        if( m_bWarningDialogDisplayed == FALSE )
            m_bWarningResponse = ( AfxMessageBox( IDS_LONGTIME, MB_YESNO ) == IDYES );

        m_bWarningDialogDisplayed = TRUE;
        return( m_bWarningResponse );

    }
    return( TRUE );

}
