/*=========================================================================*/
/*                           WarningDlgs.cpp                               */
/*                                                                         */
/*                     Manages the warnings dialogs                        */
/*                                                                         */
/*                CookieCop 2 Copyright (c) 2000,2002                      */
/*                       Ziff Davis Media, Inc.                            */
/*                        All rights reserved.							   */
/*																		   */
/*                     Programmed by Mark Sweeney                          */
/*                                                                         */
/*=========================================================================*/
#include "stdafx.h"
#include "cookiecop.h"
#include "Globals.h"
#include "WarningDlgs.h"
#include "hlp/helpids.h"



/////////////////////////////////////////////////////////////////////////////
// CWarningDlgs dialog


CWarningDlgs::CWarningDlgs(CWnd* pParent /*=NULL*/)
	: CDialog(CWarningDlgs::IDD, pParent)
{
	//{{AFX_DATA_INIT(CWarningDlgs)
	m_sWarningText = _T("");
	//}}AFX_DATA_INIT
}


void CWarningDlgs::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CWarningDlgs)
	DDX_Text(pDX, IDC_WARNING_TEXT, m_sWarningText);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CWarningDlgs, CDialog)
	//{{AFX_MSG_MAP(CWarningDlgs)
	ON_BN_CLICKED(IDNO, OnNo)
	ON_WM_CTLCOLOR()
	ON_BN_CLICKED(IDHELP, OnHelp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWarningDlgs message handlers


int ShowWarning( int nWarningMsg, int nDefault )
{
	//check to see if this warning has been disabled
	//if so, return nDefault
	char sz[30];
	wsprintf( sz, "%u", nWarningMsg );

	CString sFileName;
	QualifyPathName( &sFileName, SZ_COOKIECOP_INI );
	
	if( GetPrivateProfileInt( SZ_WARNINGS, sz, 0, sFileName ) )
		return( nDefault );

	CWarningDlgs* pDlg = new CWarningDlgs;
	pDlg->m_nWarningMsg = nWarningMsg; 
	pDlg->m_nDefault = nDefault;

	CString sTmp;
	sTmp.LoadString( nWarningMsg );	
	pDlg->m_sWarningText = "\n" + sTmp;

	int ret = pDlg->DoModal( );	
	delete pDlg;
	return( ret );
}


/////////////////////////////////////////////////////////////////////////////
// 
BOOL CWarningDlgs::OnInitDialog() 
{
	CDialog::OnInitDialog();

	//v1.2
	HANDLE hBitmap = LoadImage( AfxGetInstanceHandle( ), MAKEINTRESOURCE( IDB_SMALL_COP ),IMAGE_BITMAP,0,0,LR_LOADTRANSPARENT|LR_LOADMAP3DCOLORS);
	hBitmap = (HBITMAP)GetDlgItem( IDC_COP )->SendMessage( STM_SETIMAGE, IMAGE_BITMAP, (LPARAM)(HBITMAP)hBitmap );  
	if( hBitmap )
		::DeleteObject( hBitmap );

	if( m_nDefault == IDOK )
	{
		GetDlgItem( IDNO )->ShowWindow( SW_HIDE );
		GetDlgItem( IDOK )->SetWindowText( "OK" );
	}

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

/////////////////////////////////////////////////////////////////////////////
// 
void CWarningDlgs::OnCancel() 
{
	EndDialog( IDCANCEL );	
}

/////////////////////////////////////////////////////////////////////////////
// 
void CWarningDlgs::OnNo() 
{
	DoShowAgain( );	
	EndDialog( IDNO );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CWarningDlgs::OnOK() 
{
	DoShowAgain( );
	EndDialog( IDYES );
}

/////////////////////////////////////////////////////////////////////////////
// 
void CWarningDlgs::DoShowAgain( )
{
	if( IsDlgButtonChecked( IDC_DONT_SHOW_AGAIN ) )
	{
		char sz[30];
		wsprintf( sz, "%u", m_nWarningMsg );

		CString sFileName;
		QualifyPathName( &sFileName, SZ_COOKIECOP_INI  );

		WritePrivateProfileString( SZ_WARNINGS, sz, "1", sFileName );
	}

}

/////////////////////////////////////////////////////////////////////////////
// 
HBRUSH CWarningDlgs::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
	
	if( pWnd->GetDlgCtrlID() == IDC_WARNING_TEXT )
	{
		CRect rc;
		GetDlgItem( IDC_WARNING_TEXT )->GetWindowRect( &rc );
		hbr = (HBRUSH)GetStockObject( WHITE_BRUSH );
		pDC->SetBkColor( RGB(0xff,0xff,0xff ) );
		pDC->SetTextColor( RGB( 0x00, 0x00, 0xFF ) );
		pDC->SetBkMode( TRANSPARENT );
	}

	return hbr;
}

/////////////////////////////////////////////////////////////////////////////
// 
void CWarningDlgs::OnHelp() 
{
	m_theApp->CWinApp::WinHelp(IDH_warning_messages);
}
